//-----------------------------------------------------
// name: "zita_rev1"
//
// Code generated with Faust 0.9.59 (http://faust.grame.fr)
//-----------------------------------------------------
/* link with  */
#include <math.h>
#include <cmath>
template <int N> inline float faustpower(float x) 		{ return powf(x,N); } 
template <int N> inline double faustpower(double x) 	{ return pow(x,N); }
template <int N> inline int faustpower(int x) 			{ return faustpower<N/2>(x) * faustpower<N-N/2>(x); } 
template <> 	 inline int faustpower<0>(int x) 		{ return 1; }
template <> 	 inline int faustpower<1>(int x) 		{ return x; }
/************************************************************************

	IMPORTANT NOTE : this file contains two clearly delimited sections :
	the ARCHITECTURE section (in two parts) and the USER section. Each section
	is governed by its own copyright and license. Please check individually
	each section for license and copyright information.
*************************************************************************/

/*******************BEGIN ARCHITECTURE SECTION (part 1/2)****************/

/************************************************************************
    FAUST Architecture File
	Copyright (C) 2003-2011 GRAME, Centre National de Creation Musicale
    ---------------------------------------------------------------------
    This Architecture section is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3 of
	the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
	along with this program; If not, see <http://www.gnu.org/licenses/>.

	EXCEPTION : As a special exception, you may create a larger work
	that contains this FAUST architecture section and distribute
	that work under terms of your choice, so long as this FAUST
	architecture section is not modified.


 ************************************************************************
 ************************************************************************/

#define JACK_IOS 1

#import "FICocoaUI.h"

#ifndef __jack_dsp__
#define __jack_dsp__

#include <stdio.h>
#include <cstdlib>
#include <list>
#include <string.h>
#include <jack/jack.h>
#ifdef JACK_IOS
#include <jack/custom.h>
#endif

/******************************************************************************
*******************************************************************************

						An abstraction layer over audio layer

*******************************************************************************
*******************************************************************************/

#ifndef __audio__
#define __audio__
			
class dsp;

typedef void (* shutdown_callback)(const char* message, void* arg);

typedef void (* buffer_size_callback)(int frames, void* arg);

class audio {
    
 public:
			 audio() {}
	virtual ~audio() {}
	
	virtual bool init(const char* name, dsp*)               = 0;
	virtual bool start()                                    = 0;
	virtual void stop()                                     = 0;
    virtual void shutdown(shutdown_callback cb, void* arg)  {}
    
};
					
#endif

/******************************************************************************
*******************************************************************************

								FAUST DSP

*******************************************************************************
*******************************************************************************/

#ifndef __dsp__
#define __dsp__

#ifndef FAUSTFLOAT
#define FAUSTFLOAT float
#endif

class UI;

//----------------------------------------------------------------
//  signal processor definition
//----------------------------------------------------------------

class dsp {
 protected:
	int fSamplingFreq;
 public:
	dsp() {}
	virtual ~dsp() {}

	virtual int getNumInputs() 										= 0;
	virtual int getNumOutputs() 									= 0;
	virtual void buildUserInterface(UI* interface) 					= 0;
	virtual void init(int samplingRate) 							= 0;
 	virtual void compute(int len, FAUSTFLOAT** inputs, FAUSTFLOAT** outputs) 	= 0;
};

// On Intel set FZ (Flush to Zero) and DAZ (Denormals Are Zero)
// flags to avoid costly denormals
#ifdef __SSE__
    #include <xmmintrin.h>
    #ifdef __SSE2__
        #define AVOIDDENORMALS _mm_setcsr(_mm_getcsr() | 0x8040)
    #else
        #define AVOIDDENORMALS _mm_setcsr(_mm_getcsr() | 0x8000)
    #endif
#else
    #define AVOIDDENORMALS
#endif

#endif

/******************************************************************************
*******************************************************************************

							JACK AUDIO INTERFACE

*******************************************************************************
*******************************************************************************/

class jackaudio : public audio {

    private:

        dsp*			fDsp;               // FAUST DSP
        jack_client_t*	fClient;            // JACK client
    
        int				fNumInChans;		// number of input channels
        int				fNumOutChans;       // number of output channels
    
        jack_port_t**	fInputPorts;        // JACK input ports
        jack_port_t**	fOutputPorts;       // JACK output ports
    
        shutdown_callback fShutdown;        // Shutdown callback to be called by libjack
        void*           fShutdownArg;
        void*           fIconData;
        int             fIconSize;
        bool            fAutoConnect;
        
        std::list<std::pair<std::string, std::string> > fConnections;		// Connections list
    
        static int  _jack_srate(jack_nframes_t nframes, void* arg);
        static void _jack_shutdown(void* arg);
        static void _jack_info_shutdown(jack_status_t code, const char* reason, void* arg);
        static int  _jack_process(jack_nframes_t nframes, void* arg);
        static int  _jack_buffersize(jack_nframes_t nframes, void* arg);
    #ifdef _OPENMP
        static void* _jack_thread(void* arg);
    #endif
        void shutdown(const char* message)
        {
            if (fShutdown) {
                fShutdown(message, fShutdownArg);
            } else {
                exit(1); // By default
            }
        }
        
        // Save client connections
        void save_connections()
        {
            fConnections.clear();
            
             for (int i = 0; i < fNumInChans; i++) {
                const char** connected_port = jack_port_get_all_connections(fClient, fInputPorts[i]);
                if (connected_port != NULL) {
                    for (int port = 0; connected_port[port]; port++) {
                        fConnections.push_back(make_pair(connected_port[port], jack_port_name(fInputPorts[i])));
                        printf("INPUT %s ==> %s\n", connected_port[port], jack_port_name(fInputPorts[i]));
                    }
                    jack_free(connected_port);
                }
            }
       
            for (int i = 0; i < fNumOutChans; i++) {
                const char** connected_port = jack_port_get_all_connections(fClient, fOutputPorts[i]);
                if (connected_port != NULL) {
                    for (int port = 0; connected_port[port]; port++) {
                        fConnections.push_back(make_pair(jack_port_name(fOutputPorts[i]), connected_port[port]));
                        printf("OUTPUT %s ==> %s\n", jack_port_name(fOutputPorts[i]), connected_port[port]);
                    }
                    jack_free(connected_port);
                }
            }
        }

        // Restore previous client connections
        void restore_connections()
        {
            list<pair<string, string> >::const_iterator it;
            for (it = fConnections.begin(); it != fConnections.end(); it++) {
                pair<string, string> connection = *it;
                jack_connect(fClient, connection.first.c_str(), connection.second.c_str());
            }
        }
        
        // Connect to physical inputs/outputs
        void default_connections()
        {
            char** physicalOutPorts = (char**)jack_get_ports(fClient, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsOutput);
            if (physicalOutPorts != NULL) {
                for (int i = 0; i < fNumInChans && physicalOutPorts[i]; i++) {
                    jack_connect(fClient, physicalOutPorts[i], jack_port_name(fInputPorts[i]));
                }
                jack_free(physicalOutPorts);
            }
            
            char** physicalInPorts = (char**)jack_get_ports(fClient, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsInput);
            if (physicalInPorts != NULL) {
                for (int i = 0; i < fNumOutChans && physicalInPorts[i]; i++) {
                    jack_connect(fClient, jack_port_name(fOutputPorts[i]), physicalInPorts[i]);
                }
                jack_free(physicalInPorts);
            }
        }

    public:
        jackaudio(const void* icon_data = 0, size_t icon_size = 0, bool auto_connect = true) 
            : fDsp(0), fClient(0), fNumInChans(0), fNumOutChans(0), 
            fInputPorts(0), fOutputPorts(0), 
            fShutdown(0), fShutdownArg(0),
            fAutoConnect(auto_connect)
        {
            if (icon_data) {
                fIconData = malloc(icon_size);
                fIconSize = icon_size;
                memcpy(fIconData, icon_data, icon_size);
            } else {
                fIconData = NULL;
                fIconSize = 0;
            }
        }
        virtual ~jackaudio() 
        { 
            stop(); 
            if (fIconData) {
                free(fIconData);
            }
        }
        
        jack_client_t* getClient() { return fClient; }

        virtual bool init(const char* name, dsp* DSP) 
        {
            fDsp = DSP;
            if ((fClient = jack_client_open(name, JackNullOption, NULL)) == 0) {
                fprintf(stderr, "JACK server not running ?\n");
                return false;
            }
        #ifdef JACK_IOS
            jack_custom_publish_data(fClient, "icon.png", fIconData, fIconSize);
        #endif
            
        #ifdef _OPENMP
            jack_set_process_thread(fClient, _jack_thread, this);
        #else
            jack_set_process_callback(fClient, _jack_process, this);
        #endif

            jack_set_sample_rate_callback(fClient, _jack_srate, this);
            jack_set_buffer_size_callback(fClient, _jack_buffersize, this);
            jack_on_info_shutdown(fClient, _jack_info_shutdown, this);

            fNumInChans  = fDsp->getNumInputs();
            fNumOutChans = fDsp->getNumOutputs();
            
            fInputPorts = new jack_port_t*[fNumInChans];
            fOutputPorts = new jack_port_t*[fNumOutChans];
        
            for (int i = 0; i < fNumInChans; i++) {
                char buf[256];
                snprintf(buf, 256, "in_%d", i);
                fInputPorts[i] = jack_port_register(fClient, buf, JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
            }
            for (int i = 0; i < fNumOutChans; i++) {
                char buf[256];
                snprintf(buf, 256, "out_%d", i);
                fOutputPorts[i] = jack_port_register(fClient, buf, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
            }
            fDsp->init(jack_get_sample_rate(fClient));
            return true;
        }

        virtual bool start() 
        {
            if (jack_activate(fClient)) {
                fprintf(stderr, "Cannot activate client");
                return false;
            }
            
            if (fConnections.size() > 0) {
                restore_connections();
            } else if (fAutoConnect) {
                default_connections();
            }
            return true;
        }

        virtual void stop() 
        {
            if (fClient) {
                save_connections();
                jack_deactivate(fClient);
                for (int i = 0; i < fNumInChans; i++) {
                    jack_port_unregister(fClient, fInputPorts[i]);
                }
                for (int i = 0; i < fNumOutChans; i++) {
                    jack_port_unregister(fClient, fOutputPorts[i]);
                }
                jack_client_close(fClient);
                fClient = 0;
                
                delete[] fInputPorts;
                delete[] fOutputPorts;
            }
        }
    
        virtual void shutdown(shutdown_callback cb, void* arg)
        {
            fShutdown = cb;
            fShutdownArg = arg;
        }

        // jack callbacks
        int	process(jack_nframes_t nframes) 
        {
            AVOIDDENORMALS;
            // Retrieve JACK inputs/outputs audio buffers
            float* fInChannel[fNumInChans];
            for (int i = 0; i < fNumInChans; i++) {
                fInChannel[i] = (float*)jack_port_get_buffer(fInputPorts[i], nframes);
            }
            float* fOutChannel[fNumOutChans];
            for (int i = 0; i < fNumOutChans; i++) {
                fOutChannel[i] = (float*)jack_port_get_buffer(fOutputPorts[i], nframes);
            }
            fDsp->compute(nframes, fInChannel, fOutChannel);
            return 0;
        }

    #ifdef _OPENMP
        void process_thread() 
        {
            jack_nframes_t nframes;
            while (1) {
                nframes = jack_cycle_wait(fClient);
                process(nframes);
                jack_cycle_signal(fClient, 0);
            }
        }
    #endif
};

//----------------------------------------------------------------------------
// Jack Callbacks
//----------------------------------------------------------------------------

int jackaudio::_jack_srate(jack_nframes_t nframes, void* arg)
{
  	fprintf(stdout, "The sample rate is now %u/sec\n", nframes);
	return 0;
}

int jackaudio::_jack_buffersize(jack_nframes_t nframes, void* arg)
{
  	fprintf(stdout, "The buffer size is now %u/sec\n", nframes);
	return 0;
}

void jackaudio::_jack_info_shutdown(jack_status_t code, const char* reason, void* arg)
{
    fprintf(stderr, "%s\n", reason);
	jackaudio* audio = (jackaudio*)arg;
    audio->shutdown(reason);
}

int jackaudio::_jack_process(jack_nframes_t nframes, void* arg)
{
	jackaudio* audio = (jackaudio*)arg;
	return audio->process(nframes);
}

#ifdef _OPENMP
void* jackaudio::_jack_thread(void* arg)
{
	jackaudio* audio = (jackaudio*)arg;
	audio->process_thread();
	return 0;
}
#endif

#endif
/************************************************************************

	IMPORTANT NOTE : this file contains two clearly delimited sections :
	the ARCHITECTURE section (in two parts) and the USER section. Each section
	is governed by its own copyright and license. Please check individually
	each section for license and copyright information.
*************************************************************************/

/*******************BEGIN ARCHITECTURE SECTION (part 1/2)****************/

/************************************************************************
    FAUST Architecture File
	Copyright (C) 2003-2011 GRAME, Centre National de Creation Musicale
    ---------------------------------------------------------------------
    This Architecture section is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3 of
	the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
	along with this program; If not, see <http://www.gnu.org/licenses/>.

	EXCEPTION : As a special exception, you may create a larger work
	that contains this FAUST architecture section and distribute
	that work under terms of your choice, so long as this FAUST
	architecture section is not modified.


 ************************************************************************
 ************************************************************************/
/* link with  */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <math.h>
#include <errno.h>
#include <time.h>


#include <AudioToolbox/AudioConverter.h>
#include <AudioToolbox/AudioServices.h>
#include <AudioUnit/AudioUnit.h>

using namespace std;

/******************************************************************************
*******************************************************************************

							COREAUDIO INTERFACE

*******************************************************************************
*******************************************************************************/

#define OPEN_ERR -1
#define NO_ERR 0

class TiPhoneCoreAudioRenderer
{

private:

    AudioUnit fAUHAL;

    int	fDevNumInChans;
    int	fDevNumOutChans;

    int fHWNumInChans;
    int fHWNumOutChans;
    
    dsp* fDSP;

    AudioBufferList* fCAInputData;

    static OSStatus Render(void *inRefCon,
                           AudioUnitRenderActionFlags *ioActionFlags,
                           const AudioTimeStamp *inTimeStamp,
                           UInt32 inBusNumber,
                           UInt32 inNumberFrames,
                           AudioBufferList *ioData);

    static void InterruptionListener(void *inClientData, UInt32 inInterruption);
    
    static void AudioSessionPropertyListener(void* inClientData,
                                             AudioSessionPropertyID inID,
                                             UInt32 inDataSize,
                                             const void* inData);
    
    OSStatus Render(AudioUnitRenderActionFlags *ioActionFlags,
                     const AudioTimeStamp *inTimeStamp,
                     UInt32 inNumberFrames,
                     AudioBufferList *ioData);
                     
    int SetupMixing();

public:

    TiPhoneCoreAudioRenderer()
        :fAUHAL(0), fDevNumInChans(0), fDevNumOutChans(0),
        fHWNumInChans(0), fHWNumOutChans(0),
        fDSP(0), fCAInputData(NULL)
    {}

    virtual ~TiPhoneCoreAudioRenderer()
    {
        if (fCAInputData) {
            for (int i = 0; i < fDevNumInChans; i++) {
                free(fCAInputData->mBuffers[i].mData);
            }
            free(fCAInputData);
        }
    }

    int SetParameters(int bufferSize, int sampleRate);

    int Open(dsp* dsp, int inChan, int outChan, int bufferSize, int sampleRate);
    int Close();

    int Start();
    int Stop();

};

typedef TiPhoneCoreAudioRenderer* TiPhoneCoreAudioRendererPtr;

static void PrintStreamDesc(AudioStreamBasicDescription *inDesc)
{
    printf("- - - - - - - - - - - - - - - - - - - -\n");
    printf("  Sample Rate:%f\n", inDesc->mSampleRate);
    printf("  Format ID:%.*s\n", (int) sizeof(inDesc->mFormatID), (char*)&inDesc->mFormatID);
    printf("  Format Flags:%lX\n", inDesc->mFormatFlags);
    printf("  Bytes per Packet:%ld\n", inDesc->mBytesPerPacket);
    printf("  Frames per Packet:%ld\n", inDesc->mFramesPerPacket);
    printf("  Bytes per Frame:%ld\n", inDesc->mBytesPerFrame);
    printf("  Channels per Frame:%ld\n", inDesc->mChannelsPerFrame);
    printf("  Bits per Channel:%ld\n", inDesc->mBitsPerChannel);
    printf("- - - - - - - - - - - - - - - - - - - -\n");
}

static void printError(OSStatus err)
{
    switch (err) {
      	case kAudioConverterErr_FormatNotSupported:
            printf("error code : kAudioConverterErr_FormatNotSupported\n");
            break;
        case kAudioConverterErr_OperationNotSupported:
            printf("error code : kAudioConverterErr_OperationNotSupported\n");
            break;
        case kAudioConverterErr_PropertyNotSupported:
            printf("error code : kAudioConverterErr_PropertyNotSupported\n");
            break;
        case kAudioConverterErr_InvalidInputSize:
            printf("error code : kAudioConverterErr_InvalidInputSize\n");
            break;
        case kAudioConverterErr_InvalidOutputSize:
            printf("error code : kAudioConverterErr_InvalidOutputSize\n");
            break;
        case kAudioConverterErr_UnspecifiedError:
            printf("error code : kAudioConverterErr_UnspecifiedError\n");
            break;
        case kAudioConverterErr_BadPropertySizeError:
            printf("error code : kAudioConverterErr_BadPropertySizeError\n");
            break;
        case kAudioConverterErr_RequiresPacketDescriptionsError:
            printf("error code : kAudioConverterErr_RequiresPacketDescriptionsError\n");
            break;
        case kAudioConverterErr_InputSampleRateOutOfRange:
            printf("error code : kAudioConverterErr_InputSampleRateOutOfRange\n");
            break;
        case kAudioConverterErr_OutputSampleRateOutOfRange:
            printf("error code : kAudioConverterErr_OutputSampleRateOutOfRange\n");
            break;
        default:
            printf("error code : unknown\n");
            break;
    }
}

OSStatus TiPhoneCoreAudioRenderer::Render(void *inRefCon,
                                         AudioUnitRenderActionFlags *ioActionFlags,
                                         const AudioTimeStamp *inTimeStamp,
                                         UInt32,
                                         UInt32 inNumberFrames,
                                         AudioBufferList *ioData)
{
    return static_cast<TiPhoneCoreAudioRendererPtr>(inRefCon)->Render(ioActionFlags, inTimeStamp, inNumberFrames, ioData);
}

OSStatus TiPhoneCoreAudioRenderer::Render(AudioUnitRenderActionFlags *ioActionFlags,
                                         const AudioTimeStamp *inTimeStamp,
                                         UInt32 inNumberFrames,
                                         AudioBufferList *ioData)
{
    AudioUnitRender(fAUHAL, ioActionFlags, inTimeStamp, 1, inNumberFrames, fCAInputData);
    
    float* fInChannel[fDevNumInChans];
    float* fOutChannel[fDevNumOutChans];
    
    if (fHWNumInChans == 1) {
        // Mono ==> stereo
        for (int chan = 0; chan < fDevNumInChans; chan++) {
            fInChannel[chan] = (float*)fCAInputData->mBuffers[0].mData;
        }
    } else {
        for (int chan = 0; chan < fDevNumInChans; chan++) {
            fInChannel[chan] = (float*)fCAInputData->mBuffers[chan].mData;
        }
    }
    
    for (int chan = 0; chan < fDevNumOutChans; chan++) {
        fOutChannel[chan] = (float*)ioData->mBuffers[chan].mData;
    }
    
    fDSP->compute((int)inNumberFrames, fInChannel, fOutChannel);
 	return 0;
}

void TiPhoneCoreAudioRenderer::InterruptionListener(void *inClientData, UInt32 inInterruption)
{
	TiPhoneCoreAudioRenderer *obj = (TiPhoneCoreAudioRenderer*)inClientData;
	printf("Session interrupted! --- %s ---", inInterruption == kAudioSessionBeginInterruption ? "Begin Interruption" : "End Interruption");

	if (inInterruption == kAudioSessionEndInterruption) {
		// make sure we are again the active session
		AudioSessionSetActive(true);
        obj->SetupMixing();
        AudioOutputUnitStart(obj->fAUHAL);
	}

	if (inInterruption == kAudioSessionBeginInterruption) {
		AudioOutputUnitStop(obj->fAUHAL);
    }
}

int TiPhoneCoreAudioRenderer::SetupMixing()
{
    UInt32 allowMixing = true;
    OSStatus err = AudioSessionSetProperty(kAudioSessionProperty_OverrideCategoryMixWithOthers, sizeof(allowMixing), &allowMixing);
    if (err != noErr) {
        printf("Could not set audio session mixing\n");
        printError(err);
        return -1;
    } else {
        return 0;
    }
}
   
void TiPhoneCoreAudioRenderer::AudioSessionPropertyListener(void* inClientData,
                                                 AudioSessionPropertyID inID,
                                                 UInt32 inDataSize,
                                                 const void* inData)
{
    TiPhoneCoreAudioRenderer *obj = (TiPhoneCoreAudioRenderer*)inData;
    switch (inID) {
        case kAudioSessionProperty_ServerDied: {
            printf("kAudioSessionProperty_ServerDied\n");
            break;
        }
        case kAudioSessionProperty_AudioRouteChange: {
            printf("kAudioSessionProperty_AudioRouteChange\n");
            obj->SetupMixing();
            break;
        }
        case kAudioSessionProperty_AudioInputAvailable: {
            printf("kAudioSessionProperty_AudioInputAvailable\n");
            obj->SetupMixing();
            break;
        }
    }
}

int TiPhoneCoreAudioRenderer::SetParameters(int bufferSize, int samplerate)
{
    OSStatus err;
    UInt32 outSize;
    UInt32 enableIO;
	AudioStreamBasicDescription srcFormat, dstFormat;
    
    printf("OpenDefault fDevNumInChans = %d fDevNumOutChans = %d bufferSize = %d samplerate = %d\n", fDevNumInChans, fDevNumOutChans, bufferSize, samplerate);
     
    err = AudioSessionSetActive(true);
    if (err != noErr) {
        printf("Couldn't set audio session active\n");
        printError(err);
        return OPEN_ERR;
    }
    
    AudioSessionAddPropertyListener(kAudioSessionProperty_AudioRouteChange, AudioSessionPropertyListener, this);
    AudioSessionAddPropertyListener(kAudioSessionProperty_AudioInputAvailable, AudioSessionPropertyListener, this);
    AudioSessionAddPropertyListener(kAudioSessionProperty_ServerDied, AudioSessionPropertyListener, this);
    
    UInt32 audioCategory;
    if ((fDevNumInChans > 0) && (fDevNumOutChans > 0)) {
        audioCategory = kAudioSessionCategory_PlayAndRecord;
        printf("kAudioSessionCategory_PlayAndRecord\n");
    } else if (fDevNumInChans > 0) {
        audioCategory = kAudioSessionCategory_RecordAudio;
        printf("kAudioSessionCategory_RecordAudio\n");
    } else  if (fDevNumOutChans > 0) {
        audioCategory = kAudioSessionCategory_MediaPlayback;
        printf("kAudioSessionCategory_MediaPlayback\n");
    }
    
	err = AudioSessionSetProperty(kAudioSessionProperty_AudioCategory, sizeof(audioCategory), &audioCategory);
    if (err != noErr) {
        printf("Couldn't set audio category\n");
        printError(err);
        return OPEN_ERR;
    }
    
    if (SetupMixing() < 0) {
        return OPEN_ERR;
    }
    
    Float64 hwSampleRate;
    outSize = sizeof(hwSampleRate);
	err = AudioSessionGetProperty(kAudioSessionProperty_CurrentHardwareSampleRate, &outSize, &hwSampleRate);
    if (err != noErr) {
        printf("Couldn't get hw sample rate\n");
        printError(err);
        return OPEN_ERR;
    } else {
        printf("Get hw sample rate %f\n", hwSampleRate);
    }
    
    Float32 hwBufferSize;
    outSize = sizeof(hwBufferSize);
	err = AudioSessionGetProperty(kAudioSessionProperty_CurrentHardwareIOBufferDuration, &outSize, &hwBufferSize);
    if (err != noErr) {
        printf("Couldn't get hw buffer duration\n");
        printError(err);
        return OPEN_ERR;
    } else {
        printf("Get hw buffer duration %f\n", hwBufferSize);
    }
    
    outSize = sizeof(fHWNumInChans);
	err = AudioSessionGetProperty(kAudioSessionProperty_CurrentHardwareInputNumberChannels, &outSize, &fHWNumInChans);
    if (err != noErr) {
        printf("Couldn't get hw input channels\n");
        fHWNumInChans = 0;
        printError(err);
    } else {
        printf("Get hw input channels %d\n", fHWNumInChans);
    }
    
    if (fDevNumInChans > fHWNumInChans) {
        if ((fDevNumInChans == 2 && fHWNumInChans == 1)) { // Only mono ==> stereo is accepted
            printf("hardware ins = %d wanted = %d, so mono to stereo\n", fHWNumInChans, fDevNumInChans);
        } else {
            printf("Error: hardware ins = %d wanted = %d\n", fHWNumInChans, fDevNumInChans);
            return OPEN_ERR;
        }
    }
    
    outSize = sizeof(fHWNumOutChans);
	err = AudioSessionGetProperty(kAudioSessionProperty_CurrentHardwareOutputNumberChannels, &outSize, &fHWNumOutChans);
    if (err != noErr) {
        printf("Couldn't get hw output channels\n");
        printError(err);
        fHWNumOutChans = 0;
    } else {
        printf("Get hw output channels %d\n", fHWNumOutChans);
    }
    
    if (fDevNumOutChans > fHWNumOutChans) {
        printf("Error: hardware outs = %d wanted = %d\n", fHWNumOutChans, fDevNumOutChans);
        return OPEN_ERR;
    }
    
    Float32 preferredPeriodDuration = float(bufferSize) / float(samplerate);
    printf("preferredPeriodDuration %f \n", preferredPeriodDuration);
    
	err = AudioSessionSetProperty(kAudioSessionProperty_PreferredHardwareIOBufferDuration, sizeof(preferredPeriodDuration), &preferredPeriodDuration);
    if (err != noErr) {
        printf("Couldn't set i/o buffer duration\n");
        printError(err);
        return OPEN_ERR;
    }
    
    Float32 actualPeriodDuration;
    
    err = AudioSessionGetProperty(kAudioSessionProperty_PreferredHardwareIOBufferDuration, &outSize, &actualPeriodDuration);
    if (err != noErr) {
        printf("Couldn't get hw buffer duration\n");
        printError(err);
        return OPEN_ERR;
    }
    
    printf("preferredPeriodDuration %f actualPeriodDuration %f\n", preferredPeriodDuration, actualPeriodDuration);
    if (preferredPeriodDuration != actualPeriodDuration) {
        printf("Couldn't set hw buffer duration\n");
        return OPEN_ERR;
    }
    
    Float64 preferredSamplerate = float(samplerate);
	err = AudioSessionSetProperty(kAudioSessionProperty_PreferredHardwareSampleRate, sizeof(preferredSamplerate), &preferredSamplerate);
    if (err != noErr) {
        printf("Couldn't set i/o sample rate\n");
        printError(err);
        return OPEN_ERR;
    }
    
    // AUHAL
    AudioComponentDescription cd = {kAudioUnitType_Output, kAudioUnitSubType_RemoteIO, kAudioUnitManufacturer_Apple, 0, 0};
    AudioComponent HALOutput = AudioComponentFindNext(NULL, &cd);
    
    err = AudioComponentInstanceNew(HALOutput, &fAUHAL);
    if (err != noErr) {
		printf("Error calling OpenAComponent\n");
        printError(err);
        goto error;
	}
    
    enableIO = 1;
    err = AudioUnitSetProperty(fAUHAL, kAudioOutputUnitProperty_EnableIO, kAudioUnitScope_Output, 0, &enableIO, sizeof(enableIO));
    if (err != noErr) {
        printf("Error calling AudioUnitSetProperty - kAudioOutputUnitProperty_EnableIO, kAudioUnitScope_Output\n");
        printError(err);
        goto error;
    }
    
    enableIO = 1;
    err = AudioUnitSetProperty(fAUHAL, kAudioOutputUnitProperty_EnableIO, kAudioUnitScope_Input, 1, &enableIO, sizeof(enableIO));
    if (err != noErr) {
        printf("Error calling AudioUnitSetProperty - kAudioOutputUnitProperty_EnableIO, kAudioUnitScope_Input\n");
        printError(err);
        goto error;
    }
    
    UInt32 maxFPS;
    outSize = sizeof(maxFPS);
    err = AudioUnitGetProperty(fAUHAL, kAudioUnitProperty_MaximumFramesPerSlice, kAudioUnitScope_Global, 0, &maxFPS, &outSize);
    if (err != noErr) {
        printf("Couldn't get kAudioUnitProperty_MaximumFramesPerSlice\n");
        printError(err);
        goto error;
    } else {
        printf("Get kAudioUnitProperty_MaximumFramesPerSlice %d\n", (unsigned int)maxFPS);
    }
    
    err = AudioUnitSetProperty(fAUHAL, kAudioUnitProperty_MaximumFramesPerSlice, kAudioUnitScope_Global, 1, (UInt32*)&bufferSize, sizeof(UInt32));
    if (err != noErr) {
        printf("Error calling AudioUnitSetProperty - kAudioUnitProperty_MaximumFramesPerSlice\n");
        printError(err);
        goto error;
    }
    
    err = AudioUnitSetProperty(fAUHAL, kAudioUnitProperty_MaximumFramesPerSlice, kAudioUnitScope_Global, 0, (UInt32*)&bufferSize, sizeof(UInt32));
    if (err != noErr) {
        printf("Error calling AudioUnitSetProperty - kAudioUnitProperty_MaximumFramesPerSlice\n");
        printError(err);
        goto error;
    }
    
    err = AudioUnitInitialize(fAUHAL);
    if (err != noErr) {
		printf("Cannot initialize AUHAL unit\n");
		printError(err);
        goto error;
	}
    
    // Setting format
    
    if (fDevNumInChans > 0) {
        outSize = sizeof(AudioStreamBasicDescription);
        err = AudioUnitGetProperty(fAUHAL, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Output, 1, &srcFormat, &outSize);
        if (err != noErr) {
            printf("Error calling AudioUnitGetProperty - kAudioUnitProperty_StreamFormat kAudioUnitScope_Output\n");
            printError(err);
        }
        PrintStreamDesc(&srcFormat);
        
        srcFormat.mFormatID = kAudioFormatLinearPCM;
        srcFormat.mFormatFlags = kAudioFormatFlagsNativeFloatPacked | kLinearPCMFormatFlagIsNonInterleaved;
        srcFormat.mBytesPerPacket = sizeof(AudioUnitSampleType);
        srcFormat.mFramesPerPacket = 1;
        srcFormat.mBytesPerFrame = sizeof(AudioUnitSampleType);
        srcFormat.mChannelsPerFrame = fDevNumInChans;
        srcFormat.mBitsPerChannel = 32;
        
        PrintStreamDesc(&srcFormat);
        
        err = AudioUnitSetProperty(fAUHAL, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Output, 1, &srcFormat, sizeof(AudioStreamBasicDescription));
        if (err != noErr) {
            printf("Error calling AudioUnitSetProperty - kAudioUnitProperty_StreamFormat kAudioUnitScope_Output\n");
            printError(err);
        }
    }
    
    if (fDevNumOutChans > 0) {
        outSize = sizeof(AudioStreamBasicDescription);
        err = AudioUnitGetProperty(fAUHAL, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Input, 0, &dstFormat, &outSize);
        if (err != noErr) {
            printf("Error calling AudioUnitGetProperty - kAudioUnitProperty_StreamFormat kAudioUnitScope_Input\n");
            printError(err);
        }
        PrintStreamDesc(&dstFormat);
        
        dstFormat.mFormatID = kAudioFormatLinearPCM;
        dstFormat.mFormatFlags = kAudioFormatFlagsNativeFloatPacked | kLinearPCMFormatFlagIsNonInterleaved;
        dstFormat.mBytesPerPacket = sizeof(AudioUnitSampleType);
        dstFormat.mFramesPerPacket = 1;
        dstFormat.mBytesPerFrame = sizeof(AudioUnitSampleType);
        dstFormat.mChannelsPerFrame = fDevNumOutChans;
        dstFormat.mBitsPerChannel = 32;
        
        PrintStreamDesc(&dstFormat);
        
        err = AudioUnitSetProperty(fAUHAL, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Input, 0, &dstFormat, sizeof(AudioStreamBasicDescription));
        if (err != noErr) {
            printf("Error calling AudioUnitSetProperty - kAudioUnitProperty_StreamFormat kAudioUnitScope_Input\n");
            printError(err);
        }
    }
    
    if (fDevNumInChans > 0 && fDevNumOutChans == 0) {
        AURenderCallbackStruct output;
        output.inputProc = Render;
        output.inputProcRefCon = this;
        err = AudioUnitSetProperty(fAUHAL, kAudioOutputUnitProperty_SetInputCallback, kAudioUnitScope_Global, 0, &output, sizeof(output));
        if (err != noErr) {
            printf("Error calling AudioUnitSetProperty - kAudioUnitProperty_SetRenderCallback 1\n");
            printError(err);
            goto error;
        }
    } else {
        AURenderCallbackStruct output;
        output.inputProc = Render;
        output.inputProcRefCon = this;
        err = AudioUnitSetProperty(fAUHAL, kAudioUnitProperty_SetRenderCallback, kAudioUnitScope_Input, 0, &output, sizeof(output));
        if (err != noErr) {
            printf("Error calling AudioUnitSetProperty - kAudioUnitProperty_SetRenderCallback 0\n");
            printError(err);
            goto error;
        }
    }
    
    // Prepare buffers
    fCAInputData = (AudioBufferList*)malloc(sizeof(float) + fDevNumInChans * sizeof(AudioBuffer));
    fCAInputData->mNumberBuffers = fDevNumInChans;
    for (int i = 0; i < fDevNumInChans; i++) {
        fCAInputData->mBuffers[i].mNumberChannels = 1;
        fCAInputData->mBuffers[i].mDataByteSize = bufferSize * sizeof(float);
        fCAInputData->mBuffers[i].mData = malloc(bufferSize * sizeof(float));
    }
    
    return NO_ERR;
    
error:
    AudioUnitUninitialize(fAUHAL);
    AudioComponentInstanceDispose(fAUHAL);
    return OPEN_ERR;
}

int TiPhoneCoreAudioRenderer::Open(dsp* dsp, int inChan, int outChan, int buffersize, int samplerate)
{
    fDSP = dsp;
    fDevNumInChans = inChan;
    fDevNumOutChans = outChan;
    
    // Initialize and configure the audio session
    OSStatus err = AudioSessionInitialize(NULL, NULL, InterruptionListener, this);
    if (err != noErr && err != kAudioSessionAlreadyInitialized) {
        printf("Couldn't initialize audio session\n");
        printError(err);
        return OPEN_ERR;
    }
    
    if (SetParameters(buffersize, samplerate) < 0) {
        printf("Cannot set parameters to CoreAudio device\n");
        return OPEN_ERR;
    }
    
    return NO_ERR;
}

int TiPhoneCoreAudioRenderer::Close()
{
 	AudioUnitUninitialize(fAUHAL);
    AudioComponentInstanceDispose(fAUHAL);
    return NO_ERR;
}

int TiPhoneCoreAudioRenderer::Start()
{
    AudioSessionSetActive(true);
	OSStatus err = AudioOutputUnitStart(fAUHAL);
    
    if (err != noErr) {
        printf("Error while opening device : device open error\n");
        return OPEN_ERR;
    } else {
        return NO_ERR;
	}
}

int TiPhoneCoreAudioRenderer::Stop()
{
    AudioSessionSetActive(false);
    OSStatus err = AudioOutputUnitStop(fAUHAL);
    
    if (err != noErr) {
        printf("Error while closing device : device close error\n");
        return OPEN_ERR;
    } else {
        return NO_ERR;
	}
}

/******************************************************************************
 *******************************************************************************
 
                                CORE AUDIO INTERFACE
 
 *******************************************************************************
 *******************************************************************************/
class iosaudio : public audio {
    
    TiPhoneCoreAudioRenderer fAudioDevice;
	int fSampleRate, fFramesPerBuf;
    
public:
    iosaudio(int srate, int fpb) : fSampleRate(srate), fFramesPerBuf(fpb) {}
	virtual ~iosaudio() {}
    
	virtual bool init(const char* /*name*/, dsp* DSP) 
    {
    	DSP->init(fSampleRate);
		if (fAudioDevice.Open(DSP, DSP->getNumInputs(), DSP->getNumOutputs(), fFramesPerBuf, fSampleRate) < 0) {
			printf("Cannot open iOS audio device\n");
    		return false;
		}
        return true;
    }
    
	virtual bool start() 
    {
		if (fAudioDevice.Start() < 0) {
			printf("Cannot start iOS audio device\n");
			return false;
		}
		return true;
	}
    
	virtual void stop() 
    {
		fAudioDevice.Stop();
		fAudioDevice.Close();
 	}
    
};


/********************END ARCHITECTURE SECTION (part 2/2)****************/


/************************************************************************
 ************************************************************************
    FAUST Architecture File
	Copyright (C) 2003-2011 GRAME, Centre National de Creation Musicale
    ---------------------------------------------------------------------
    This Architecture section is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3 of
	the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
	along with this program; If not, see <http://www.gnu.org/licenses/>.

 ************************************************************************
 ************************************************************************/
 
#ifndef __misc__
#define __misc__

#include <map>
#include <string.h>
#include <stdlib.h>

/************************************************************************
 ************************************************************************
    FAUST Architecture File
	Copyright (C) 2003-2011 GRAME, Centre National de Creation Musicale
    ---------------------------------------------------------------------
    This Architecture section is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3 of
	the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
	along with this program; If not, see <http://www.gnu.org/licenses/>.

 ************************************************************************
 ************************************************************************/
 
#ifndef __meta__
#define __meta__

struct Meta
{
    virtual void declare(const char* key, const char* value) = 0;
};

#endif


struct XXXX_Meta : std::map<const char*, const char*>
{
    void declare(const char* key, const char* value) { (*this)[key]=value; }
};

struct MY_Meta : Meta, std::map<const char*, const char*>
{
    void declare(const char* key, const char* value) { (*this)[key]=value; }
};

inline int	lsr(int x, int n)	{ return int(((unsigned int)x) >> n); }
inline int 	int2pow2(int x)		{ int r=0; while ((1<<r)<x) r++; return r; }

long lopt(char *argv[], const char *name, long def)
{
	int	i;
	for (i = 0; argv[i]; i++) if (!strcmp(argv[i], name)) return atoi(argv[i+1]);
	return def;
}

char* lopts(char *argv[], const char *name, char* def)
{
	int	i;
	for (i = 0; argv[i]; i++) if (!strcmp(argv[i], name)) return argv[i+1];
	return def;
}
#endif


/******************************************************************************
*******************************************************************************

							       VECTOR INTRINSICS

*******************************************************************************
*******************************************************************************/


/******************************************************************************
*******************************************************************************

								USER INTERFACE

*******************************************************************************
*******************************************************************************/


/********************END ARCHITECTURE SECTION (part 1/2)****************/

/**************************BEGIN USER SECTION **************************/

#ifndef FAUSTFLOAT
#define FAUSTFLOAT float
#endif  

typedef long double quad;

#ifndef FAUSTCLASS 
#define FAUSTCLASS mydsp
#endif

class mydsp : public dsp {
  private:
	FAUSTFLOAT 	fslider0;
	float 	fRec0[2];
	int 	iConst0;
	float 	fConst1;
	FAUSTFLOAT 	fslider1;
	FAUSTFLOAT 	fslider2;
	FAUSTFLOAT 	fslider3;
	FAUSTFLOAT 	fslider4;
	float 	fConst2;
	float 	fConst3;
	FAUSTFLOAT 	fslider5;
	float 	fConst4;
	FAUSTFLOAT 	fslider6;
	FAUSTFLOAT 	fslider7;
	float 	fConst5;
	FAUSTFLOAT 	fslider8;
	float 	fRec14[2];
	float 	fRec13[2];
	int 	IOTA;
	float 	fVec0[16384];
	float 	fConst6;
	int 	iConst7;
	float 	fVec1[8192];
	float 	fConst8;
	FAUSTFLOAT 	fslider9;
	float 	fVec2[1024];
	int 	iConst9;
	float 	fRec11[2];
	float 	fConst10;
	float 	fConst11;
	float 	fRec18[2];
	float 	fRec17[2];
	float 	fVec3[16384];
	float 	fConst12;
	int 	iConst13;
	float 	fVec4[2048];
	int 	iConst14;
	float 	fRec15[2];
	float 	fConst15;
	float 	fConst16;
	float 	fRec22[2];
	float 	fRec21[2];
	float 	fVec5[8192];
	float 	fConst17;
	int 	iConst18;
	float 	fVec6[2048];
	int 	iConst19;
	float 	fRec19[2];
	float 	fConst20;
	float 	fConst21;
	float 	fRec26[2];
	float 	fRec25[2];
	float 	fVec7[16384];
	float 	fConst22;
	int 	iConst23;
	float 	fVec8[2048];
	int 	iConst24;
	float 	fRec23[2];
	float 	fConst25;
	float 	fConst26;
	float 	fRec30[2];
	float 	fRec29[2];
	float 	fVec9[8192];
	float 	fConst27;
	int 	iConst28;
	float 	fVec10[8192];
	float 	fVec11[1024];
	int 	iConst29;
	float 	fRec27[2];
	float 	fConst30;
	float 	fConst31;
	float 	fRec34[2];
	float 	fRec33[2];
	float 	fVec12[8192];
	float 	fConst32;
	int 	iConst33;
	float 	fVec13[2048];
	int 	iConst34;
	float 	fRec31[2];
	float 	fConst35;
	float 	fConst36;
	float 	fRec38[2];
	float 	fRec37[2];
	float 	fVec14[8192];
	float 	fConst37;
	int 	iConst38;
	float 	fVec15[2048];
	int 	iConst39;
	float 	fRec35[2];
	float 	fConst40;
	float 	fConst41;
	float 	fRec42[2];
	float 	fRec41[2];
	float 	fVec16[8192];
	float 	fConst42;
	int 	iConst43;
	float 	fVec17[1024];
	int 	iConst44;
	float 	fRec39[2];
	float 	fRec3[3];
	float 	fRec4[3];
	float 	fRec5[3];
	float 	fRec6[3];
	float 	fRec7[3];
	float 	fRec8[3];
	float 	fRec9[3];
	float 	fRec10[3];
	float 	fRec2[3];
	float 	fRec1[3];
	FAUSTFLOAT 	fslider10;
	float 	fRec43[2];
	float 	fRec45[3];
	float 	fRec44[3];
  public:
	static void metadata(Meta* m) 	{ 
		m->declare("name", "zita_rev1");
		m->declare("effect.lib/name", "Faust Audio Effect Library");
		m->declare("effect.lib/author", "Julius O. Smith (jos at ccrma.stanford.edu)");
		m->declare("effect.lib/copyright", "Julius O. Smith III");
		m->declare("effect.lib/version", "1.33");
		m->declare("effect.lib/license", "STK-4.3");
		m->declare("filter.lib/name", "Faust Filter Library");
		m->declare("filter.lib/author", "Julius O. Smith (jos at ccrma.stanford.edu)");
		m->declare("filter.lib/copyright", "Julius O. Smith III");
		m->declare("filter.lib/version", "1.29");
		m->declare("filter.lib/license", "STK-4.3");
		m->declare("filter.lib/reference", "https://ccrma.stanford.edu/~jos/filters/");
		m->declare("music.lib/name", "Music Library");
		m->declare("music.lib/author", "GRAME");
		m->declare("music.lib/copyright", "GRAME");
		m->declare("music.lib/version", "1.0");
		m->declare("music.lib/license", "LGPL with exception");
		m->declare("math.lib/name", "Math Library");
		m->declare("math.lib/author", "GRAME");
		m->declare("math.lib/copyright", "GRAME");
		m->declare("math.lib/version", "1.0");
		m->declare("math.lib/license", "LGPL with exception");
	}

	virtual int getNumInputs() 	{ return 2; }
	virtual int getNumOutputs() 	{ return 2; }
	static void classInit(int samplingFreq) {
	}
	virtual void instanceInit(int samplingFreq) {
		fSamplingFreq = samplingFreq;
		fslider0 = -2e+01f;
		for (int i=0; i<2; i++) fRec0[i] = 0;
		iConst0 = min(192000, max(1, fSamplingFreq));
		fConst1 = (6.283185307179586f / float(iConst0));
		fslider1 = 315.0f;
		fslider2 = 0.0f;
		fslider3 = 315.0f;
		fslider4 = 0.0f;
		fConst2 = floorf((0.5f + (0.219991f * iConst0)));
		fConst3 = ((0 - (6.907755278982138f * fConst2)) / float(iConst0));
		fslider5 = 2.0f;
		fConst4 = (6.283185307179586f / float(iConst0));
		fslider6 = 6e+03f;
		fslider7 = 3.0f;
		fConst5 = (3.141592653589793f / float(iConst0));
		fslider8 = 2e+02f;
		for (int i=0; i<2; i++) fRec14[i] = 0;
		for (int i=0; i<2; i++) fRec13[i] = 0;
		IOTA = 0;
		for (int i=0; i<16384; i++) fVec0[i] = 0;
		fConst6 = floorf((0.5f + (0.019123f * iConst0)));
		iConst7 = int((int((fConst2 - fConst6)) & 16383));
		for (int i=0; i<8192; i++) fVec1[i] = 0;
		fConst8 = (0.001f * iConst0);
		fslider9 = 6e+01f;
		for (int i=0; i<1024; i++) fVec2[i] = 0;
		iConst9 = int((int((fConst6 - 1)) & 1023));
		for (int i=0; i<2; i++) fRec11[i] = 0;
		fConst10 = floorf((0.5f + (0.256891f * iConst0)));
		fConst11 = ((0 - (6.907755278982138f * fConst10)) / float(iConst0));
		for (int i=0; i<2; i++) fRec18[i] = 0;
		for (int i=0; i<2; i++) fRec17[i] = 0;
		for (int i=0; i<16384; i++) fVec3[i] = 0;
		fConst12 = floorf((0.5f + (0.027333f * iConst0)));
		iConst13 = int((int((fConst10 - fConst12)) & 16383));
		for (int i=0; i<2048; i++) fVec4[i] = 0;
		iConst14 = int((int((fConst12 - 1)) & 2047));
		for (int i=0; i<2; i++) fRec15[i] = 0;
		fConst15 = floorf((0.5f + (0.192303f * iConst0)));
		fConst16 = ((0 - (6.907755278982138f * fConst15)) / float(iConst0));
		for (int i=0; i<2; i++) fRec22[i] = 0;
		for (int i=0; i<2; i++) fRec21[i] = 0;
		for (int i=0; i<8192; i++) fVec5[i] = 0;
		fConst17 = floorf((0.5f + (0.029291f * iConst0)));
		iConst18 = int((int((fConst15 - fConst17)) & 8191));
		for (int i=0; i<2048; i++) fVec6[i] = 0;
		iConst19 = int((int((fConst17 - 1)) & 2047));
		for (int i=0; i<2; i++) fRec19[i] = 0;
		fConst20 = floorf((0.5f + (0.210389f * iConst0)));
		fConst21 = ((0 - (6.907755278982138f * fConst20)) / float(iConst0));
		for (int i=0; i<2; i++) fRec26[i] = 0;
		for (int i=0; i<2; i++) fRec25[i] = 0;
		for (int i=0; i<16384; i++) fVec7[i] = 0;
		fConst22 = floorf((0.5f + (0.024421f * iConst0)));
		iConst23 = int((int((fConst20 - fConst22)) & 16383));
		for (int i=0; i<2048; i++) fVec8[i] = 0;
		iConst24 = int((int((fConst22 - 1)) & 2047));
		for (int i=0; i<2; i++) fRec23[i] = 0;
		fConst25 = floorf((0.5f + (0.125f * iConst0)));
		fConst26 = ((0 - (6.907755278982138f * fConst25)) / float(iConst0));
		for (int i=0; i<2; i++) fRec30[i] = 0;
		for (int i=0; i<2; i++) fRec29[i] = 0;
		for (int i=0; i<8192; i++) fVec9[i] = 0;
		fConst27 = floorf((0.5f + (0.013458f * iConst0)));
		iConst28 = int((int((fConst25 - fConst27)) & 8191));
		for (int i=0; i<8192; i++) fVec10[i] = 0;
		for (int i=0; i<1024; i++) fVec11[i] = 0;
		iConst29 = int((int((fConst27 - 1)) & 1023));
		for (int i=0; i<2; i++) fRec27[i] = 0;
		fConst30 = floorf((0.5f + (0.127837f * iConst0)));
		fConst31 = ((0 - (6.907755278982138f * fConst30)) / float(iConst0));
		for (int i=0; i<2; i++) fRec34[i] = 0;
		for (int i=0; i<2; i++) fRec33[i] = 0;
		for (int i=0; i<8192; i++) fVec12[i] = 0;
		fConst32 = floorf((0.5f + (0.031604f * iConst0)));
		iConst33 = int((int((fConst30 - fConst32)) & 8191));
		for (int i=0; i<2048; i++) fVec13[i] = 0;
		iConst34 = int((int((fConst32 - 1)) & 2047));
		for (int i=0; i<2; i++) fRec31[i] = 0;
		fConst35 = floorf((0.5f + (0.174713f * iConst0)));
		fConst36 = ((0 - (6.907755278982138f * fConst35)) / float(iConst0));
		for (int i=0; i<2; i++) fRec38[i] = 0;
		for (int i=0; i<2; i++) fRec37[i] = 0;
		for (int i=0; i<8192; i++) fVec14[i] = 0;
		fConst37 = floorf((0.5f + (0.022904f * iConst0)));
		iConst38 = int((int((fConst35 - fConst37)) & 8191));
		for (int i=0; i<2048; i++) fVec15[i] = 0;
		iConst39 = int((int((fConst37 - 1)) & 2047));
		for (int i=0; i<2; i++) fRec35[i] = 0;
		fConst40 = floorf((0.5f + (0.153129f * iConst0)));
		fConst41 = ((0 - (6.907755278982138f * fConst40)) / float(iConst0));
		for (int i=0; i<2; i++) fRec42[i] = 0;
		for (int i=0; i<2; i++) fRec41[i] = 0;
		for (int i=0; i<8192; i++) fVec16[i] = 0;
		fConst42 = floorf((0.5f + (0.020346f * iConst0)));
		iConst43 = int((int((fConst40 - fConst42)) & 8191));
		for (int i=0; i<1024; i++) fVec17[i] = 0;
		iConst44 = int((int((fConst42 - 1)) & 1023));
		for (int i=0; i<2; i++) fRec39[i] = 0;
		for (int i=0; i<3; i++) fRec3[i] = 0;
		for (int i=0; i<3; i++) fRec4[i] = 0;
		for (int i=0; i<3; i++) fRec5[i] = 0;
		for (int i=0; i<3; i++) fRec6[i] = 0;
		for (int i=0; i<3; i++) fRec7[i] = 0;
		for (int i=0; i<3; i++) fRec8[i] = 0;
		for (int i=0; i<3; i++) fRec9[i] = 0;
		for (int i=0; i<3; i++) fRec10[i] = 0;
		for (int i=0; i<3; i++) fRec2[i] = 0;
		for (int i=0; i<3; i++) fRec1[i] = 0;
		fslider10 = 0.0f;
		for (int i=0; i<2; i++) fRec43[i] = 0;
		for (int i=0; i<3; i++) fRec45[i] = 0;
		for (int i=0; i<3; i++) fRec44[i] = 0;
	}
	virtual void init(int samplingFreq) {
		classInit(samplingFreq);
		instanceInit(samplingFreq);
	}
	virtual void buildUserInterface(UI* interface) {
		interface->declare(0, "0", "");
		interface->declare(0, "tooltip", "~ ZITA REV1 FEEDBACK DELAY NETWORK (FDN) & SCHROEDER ALLPASS-COMB REVERBERATOR (8x8). See Faust's effect.lib for documentation and references");
		interface->openHorizontalBox("Zita_Rev1");
		interface->declare(0, "1", "");
		interface->openHorizontalBox("Input");
		interface->declare(&fslider9, "1", "");
		interface->declare(&fslider9, "style", "knob");
		interface->declare(&fslider9, "tooltip", "Delay in ms before reverberation begins");
		interface->declare(&fslider9, "unit", "ms");
		interface->addVerticalSlider("In Delay", &fslider9, 6e+01f, 2e+01f, 1e+02f, 1.0f);
		interface->closeBox();
		interface->declare(0, "2", "");
		interface->openHorizontalBox("Decay Times in Bands (see tooltips)");
		interface->declare(&fslider8, "1", "");
		interface->declare(&fslider8, "style", "knob");
		interface->declare(&fslider8, "tooltip", "Crossover frequency (Hz) separating low and middle frequencies");
		interface->declare(&fslider8, "unit", "Hz");
		interface->addVerticalSlider("LF X", &fslider8, 2e+02f, 5e+01f, 1e+03f, 1.0f);
		interface->declare(&fslider7, "2", "");
		interface->declare(&fslider7, "style", "knob");
		interface->declare(&fslider7, "tooltip", "T60 = time (in seconds) to decay 60dB in low-frequency band");
		interface->declare(&fslider7, "unit", "s");
		interface->addVerticalSlider("Low RT60", &fslider7, 3.0f, 1.0f, 8.0f, 0.1f);
		interface->declare(&fslider5, "3", "");
		interface->declare(&fslider5, "style", "knob");
		interface->declare(&fslider5, "tooltip", "T60 = time (in seconds) to decay 60dB in middle band");
		interface->declare(&fslider5, "unit", "s");
		interface->addVerticalSlider("Mid RT60", &fslider5, 2.0f, 1.0f, 8.0f, 0.1f);
		interface->declare(&fslider6, "4", "");
		interface->declare(&fslider6, "style", "knob");
		interface->declare(&fslider6, "tooltip", "Frequency (Hz) at which the high-frequency T60 is half the middle-band's T60");
		interface->declare(&fslider6, "unit", "Hz");
		interface->addVerticalSlider("HF Damping", &fslider6, 6e+03f, 1.5e+03f, 2.352e+04f, 1.0f);
		interface->closeBox();
		interface->declare(0, "3", "");
		interface->openHorizontalBox("RM Peaking Equalizer 1");
		interface->declare(&fslider3, "1", "");
		interface->declare(&fslider3, "style", "knob");
		interface->declare(&fslider3, "tooltip", "Center-frequency of second-order Regalia-Mitra peaking equalizer section 1");
		interface->declare(&fslider3, "unit", "Hz");
		interface->addVerticalSlider("Eq1 Freq", &fslider3, 315.0f, 4e+01f, 2.5e+03f, 1.0f);
		interface->declare(&fslider4, "2", "");
		interface->declare(&fslider4, "style", "knob");
		interface->declare(&fslider4, "tooltip", "Peak level in dB of second-order Regalia-Mitra peaking equalizer section 1");
		interface->declare(&fslider4, "unit", "dB");
		interface->addVerticalSlider("Eq1 Level", &fslider4, 0.0f, -15.0f, 15.0f, 0.1f);
		interface->closeBox();
		interface->declare(0, "4", "");
		interface->openHorizontalBox("RM Peaking Equalizer 2");
		interface->declare(&fslider1, "1", "");
		interface->declare(&fslider1, "style", "knob");
		interface->declare(&fslider1, "tooltip", "Center-frequency of second-order Regalia-Mitra peaking equalizer section 2");
		interface->declare(&fslider1, "unit", "Hz");
		interface->addVerticalSlider("Eq2 Freq", &fslider1, 315.0f, 4e+01f, 2.5e+03f, 1.0f);
		interface->declare(&fslider2, "2", "");
		interface->declare(&fslider2, "style", "knob");
		interface->declare(&fslider2, "tooltip", "Peak level in dB of second-order Regalia-Mitra peaking equalizer section 2");
		interface->declare(&fslider2, "unit", "dB");
		interface->addVerticalSlider("Eq2 Level", &fslider2, 0.0f, -15.0f, 15.0f, 0.1f);
		interface->closeBox();
		interface->declare(0, "5", "");
		interface->openHorizontalBox("Output");
		interface->declare(&fslider10, "1", "");
		interface->declare(&fslider10, "style", "knob");
		interface->declare(&fslider10, "tooltip", "-1 = dry, 1 = wet");
		interface->addVerticalSlider("Dry/Wet Mix", &fslider10, 0.0f, -1.0f, 1.0f, 0.01f);
		interface->declare(&fslider0, "2", "");
		interface->declare(&fslider0, "style", "knob");
		interface->declare(&fslider0, "tooltip", "Output scale factor");
		interface->declare(&fslider0, "unit", "dB");
		interface->addVerticalSlider("Level", &fslider0, -2e+01f, -7e+01f, 4e+01f, 0.1f);
		interface->closeBox();
		interface->closeBox();
	}
	virtual void compute (int count, FAUSTFLOAT** input, FAUSTFLOAT** output) {
		float 	fSlow0 = (0.0010000000000000009f * powf(10,(0.05f * fslider0)));
		float 	fSlow1 = fslider1;
		float 	fSlow2 = powf(10,(0.05f * fslider2));
		float 	fSlow3 = (fConst1 * (fSlow1 / sqrtf(max((float)0, fSlow2))));
		float 	fSlow4 = ((1.0f - fSlow3) / (1.0f + fSlow3));
		float 	fSlow5 = ((0 - cosf((fConst1 * fSlow1))) * (1 + fSlow4));
		float 	fSlow6 = fslider3;
		float 	fSlow7 = powf(10,(0.05f * fslider4));
		float 	fSlow8 = (fConst1 * (fSlow6 / sqrtf(max((float)0, fSlow7))));
		float 	fSlow9 = ((1.0f - fSlow8) / (1.0f + fSlow8));
		float 	fSlow10 = ((0 - cosf((fConst1 * fSlow6))) * (1 + fSlow9));
		float 	fSlow11 = fslider5;
		float 	fSlow12 = expf((fConst3 / fSlow11));
		float 	fSlow13 = faustpower<2>(fSlow12);
		float 	fSlow14 = cosf((fConst4 * fslider6));
		float 	fSlow15 = (1.0f - (fSlow13 * fSlow14));
		float 	fSlow16 = (1.0f - fSlow13);
		float 	fSlow17 = (fSlow15 / fSlow16);
		float 	fSlow18 = sqrtf(max((float)0, ((faustpower<2>(fSlow15) / faustpower<2>(fSlow16)) - 1.0f)));
		float 	fSlow19 = (fSlow17 - fSlow18);
		float 	fSlow20 = (fSlow12 * ((1.0f + fSlow18) - fSlow17));
		float 	fSlow21 = fslider7;
		float 	fSlow22 = ((expf((fConst3 / fSlow21)) / fSlow12) - 1);
		float 	fSlow23 = (1.0f / tanf((fConst5 * fslider8)));
		float 	fSlow24 = (1 + fSlow23);
		float 	fSlow25 = (0 - ((1 - fSlow23) / fSlow24));
		float 	fSlow26 = (1.0f / fSlow24);
		int 	iSlow27 = int((int((fConst8 * fslider9)) & 8191));
		float 	fSlow28 = expf((fConst11 / fSlow11));
		float 	fSlow29 = faustpower<2>(fSlow28);
		float 	fSlow30 = (1.0f - (fSlow14 * fSlow29));
		float 	fSlow31 = (1.0f - fSlow29);
		float 	fSlow32 = (fSlow30 / fSlow31);
		float 	fSlow33 = sqrtf(max((float)0, ((faustpower<2>(fSlow30) / faustpower<2>(fSlow31)) - 1.0f)));
		float 	fSlow34 = (fSlow32 - fSlow33);
		float 	fSlow35 = (fSlow28 * ((1.0f + fSlow33) - fSlow32));
		float 	fSlow36 = ((expf((fConst11 / fSlow21)) / fSlow28) - 1);
		float 	fSlow37 = expf((fConst16 / fSlow11));
		float 	fSlow38 = faustpower<2>(fSlow37);
		float 	fSlow39 = (1.0f - (fSlow14 * fSlow38));
		float 	fSlow40 = (1.0f - fSlow38);
		float 	fSlow41 = (fSlow39 / fSlow40);
		float 	fSlow42 = sqrtf(max((float)0, ((faustpower<2>(fSlow39) / faustpower<2>(fSlow40)) - 1.0f)));
		float 	fSlow43 = (fSlow41 - fSlow42);
		float 	fSlow44 = (fSlow37 * ((1.0f + fSlow42) - fSlow41));
		float 	fSlow45 = ((expf((fConst16 / fSlow21)) / fSlow37) - 1);
		float 	fSlow46 = expf((fConst21 / fSlow11));
		float 	fSlow47 = faustpower<2>(fSlow46);
		float 	fSlow48 = (1.0f - (fSlow14 * fSlow47));
		float 	fSlow49 = (1.0f - fSlow47);
		float 	fSlow50 = (fSlow48 / fSlow49);
		float 	fSlow51 = sqrtf(max((float)0, ((faustpower<2>(fSlow48) / faustpower<2>(fSlow49)) - 1.0f)));
		float 	fSlow52 = (fSlow50 - fSlow51);
		float 	fSlow53 = (fSlow46 * ((1.0f + fSlow51) - fSlow50));
		float 	fSlow54 = ((expf((fConst21 / fSlow21)) / fSlow46) - 1);
		float 	fSlow55 = expf((fConst26 / fSlow11));
		float 	fSlow56 = faustpower<2>(fSlow55);
		float 	fSlow57 = (1.0f - (fSlow14 * fSlow56));
		float 	fSlow58 = (1.0f - fSlow56);
		float 	fSlow59 = (fSlow57 / fSlow58);
		float 	fSlow60 = sqrtf(max((float)0, ((faustpower<2>(fSlow57) / faustpower<2>(fSlow58)) - 1.0f)));
		float 	fSlow61 = (fSlow59 - fSlow60);
		float 	fSlow62 = (fSlow55 * ((1.0f + fSlow60) - fSlow59));
		float 	fSlow63 = ((expf((fConst26 / fSlow21)) / fSlow55) - 1);
		float 	fSlow64 = expf((fConst31 / fSlow11));
		float 	fSlow65 = faustpower<2>(fSlow64);
		float 	fSlow66 = (1.0f - (fSlow14 * fSlow65));
		float 	fSlow67 = (1.0f - fSlow65);
		float 	fSlow68 = (fSlow66 / fSlow67);
		float 	fSlow69 = sqrtf(max((float)0, ((faustpower<2>(fSlow66) / faustpower<2>(fSlow67)) - 1.0f)));
		float 	fSlow70 = (fSlow68 - fSlow69);
		float 	fSlow71 = (fSlow64 * ((1.0f + fSlow69) - fSlow68));
		float 	fSlow72 = ((expf((fConst31 / fSlow21)) / fSlow64) - 1);
		float 	fSlow73 = expf((fConst36 / fSlow11));
		float 	fSlow74 = faustpower<2>(fSlow73);
		float 	fSlow75 = (1.0f - (fSlow14 * fSlow74));
		float 	fSlow76 = (1.0f - fSlow74);
		float 	fSlow77 = (fSlow75 / fSlow76);
		float 	fSlow78 = sqrtf(max((float)0, ((faustpower<2>(fSlow75) / faustpower<2>(fSlow76)) - 1.0f)));
		float 	fSlow79 = (fSlow77 - fSlow78);
		float 	fSlow80 = (fSlow73 * ((1.0f + fSlow78) - fSlow77));
		float 	fSlow81 = ((expf((fConst36 / fSlow21)) / fSlow73) - 1);
		float 	fSlow82 = expf((fConst41 / fSlow11));
		float 	fSlow83 = faustpower<2>(fSlow82);
		float 	fSlow84 = (1.0f - (fSlow14 * fSlow83));
		float 	fSlow85 = (1.0f - fSlow83);
		float 	fSlow86 = (fSlow84 / fSlow85);
		float 	fSlow87 = sqrtf(max((float)0, ((faustpower<2>(fSlow84) / faustpower<2>(fSlow85)) - 1.0f)));
		float 	fSlow88 = (fSlow86 - fSlow87);
		float 	fSlow89 = (fSlow82 * ((1.0f + fSlow87) - fSlow86));
		float 	fSlow90 = ((expf((fConst41 / fSlow21)) / fSlow82) - 1);
		float 	fSlow91 = (0.0010000000000000009f * fslider10);
		FAUSTFLOAT* input0 = input[0];
		FAUSTFLOAT* input1 = input[1];
		FAUSTFLOAT* output0 = output[0];
		FAUSTFLOAT* output1 = output[1];
		for (int i=0; i<count; i++) {
			fRec0[0] = ((0.999f * fRec0[1]) + fSlow0);
			float fTemp0 = (fSlow5 * fRec1[1]);
			float fTemp1 = (fSlow10 * fRec2[1]);
			fRec14[0] = ((fSlow25 * fRec14[1]) + (fSlow26 * (fRec10[1] + fRec10[2])));
			fRec13[0] = ((fSlow19 * fRec13[1]) + (fSlow20 * (fRec10[1] + (fSlow22 * fRec14[0]))));
			fVec0[IOTA&16383] = (1e-20f + (0.35355339059327373f * fRec13[0]));
			float fTemp2 = (float)input1[i];
			fVec1[IOTA&8191] = fTemp2;
			float fTemp3 = (0.3f * fVec1[(IOTA-iSlow27)&8191]);
			float fTemp4 = (((0.6f * fRec11[1]) + fVec0[(IOTA-iConst7)&16383]) - fTemp3);
			fVec2[IOTA&1023] = fTemp4;
			fRec11[0] = fVec2[(IOTA-iConst9)&1023];
			float 	fRec12 = (0 - (0.6f * fVec2[IOTA&1023]));
			fRec18[0] = ((fSlow25 * fRec18[1]) + (fSlow26 * (fRec6[1] + fRec6[2])));
			fRec17[0] = ((fSlow34 * fRec17[1]) + (fSlow35 * (fRec6[1] + (fSlow36 * fRec18[0]))));
			fVec3[IOTA&16383] = (1e-20f + (0.35355339059327373f * fRec17[0]));
			float fTemp5 = (((0.6f * fRec15[1]) + fVec3[(IOTA-iConst13)&16383]) - fTemp3);
			fVec4[IOTA&2047] = fTemp5;
			fRec15[0] = fVec4[(IOTA-iConst14)&2047];
			float 	fRec16 = (0 - (0.6f * fVec4[IOTA&2047]));
			fRec22[0] = ((fSlow25 * fRec22[1]) + (fSlow26 * (fRec8[1] + fRec8[2])));
			fRec21[0] = ((fSlow43 * fRec21[1]) + (fSlow44 * (fRec8[1] + (fSlow45 * fRec22[0]))));
			fVec5[IOTA&8191] = (1e-20f + (0.35355339059327373f * fRec21[0]));
			float fTemp6 = (fVec5[(IOTA-iConst18)&8191] + (fTemp3 + (0.6f * fRec19[1])));
			fVec6[IOTA&2047] = fTemp6;
			fRec19[0] = fVec6[(IOTA-iConst19)&2047];
			float 	fRec20 = (0 - (0.6f * fVec6[IOTA&2047]));
			fRec26[0] = ((fSlow25 * fRec26[1]) + (fSlow26 * (fRec4[1] + fRec4[2])));
			fRec25[0] = ((fSlow52 * fRec25[1]) + (fSlow53 * (fRec4[1] + (fSlow54 * fRec26[0]))));
			fVec7[IOTA&16383] = (1e-20f + (0.35355339059327373f * fRec25[0]));
			float fTemp7 = (fTemp3 + ((0.6f * fRec23[1]) + fVec7[(IOTA-iConst23)&16383]));
			fVec8[IOTA&2047] = fTemp7;
			fRec23[0] = fVec8[(IOTA-iConst24)&2047];
			float 	fRec24 = (0 - (0.6f * fVec8[IOTA&2047]));
			fRec30[0] = ((fSlow25 * fRec30[1]) + (fSlow26 * (fRec9[1] + fRec9[2])));
			fRec29[0] = ((fSlow61 * fRec29[1]) + (fSlow62 * (fRec9[1] + (fSlow63 * fRec30[0]))));
			fVec9[IOTA&8191] = (1e-20f + (0.35355339059327373f * fRec29[0]));
			float fTemp8 = (float)input0[i];
			fVec10[IOTA&8191] = fTemp8;
			float fTemp9 = (0.3f * fVec10[(IOTA-iSlow27)&8191]);
			float fTemp10 = (fVec9[(IOTA-iConst28)&8191] - (fTemp9 + (0.6f * fRec27[1])));
			fVec11[IOTA&1023] = fTemp10;
			fRec27[0] = fVec11[(IOTA-iConst29)&1023];
			float 	fRec28 = (0.6f * fVec11[IOTA&1023]);
			fRec34[0] = ((fSlow25 * fRec34[1]) + (fSlow26 * (fRec5[1] + fRec5[2])));
			fRec33[0] = ((fSlow70 * fRec33[1]) + (fSlow71 * (fRec5[1] + (fSlow72 * fRec34[0]))));
			fVec12[IOTA&8191] = (1e-20f + (0.35355339059327373f * fRec33[0]));
			float fTemp11 = (fVec12[(IOTA-iConst33)&8191] - (fTemp9 + (0.6f * fRec31[1])));
			fVec13[IOTA&2047] = fTemp11;
			fRec31[0] = fVec13[(IOTA-iConst34)&2047];
			float 	fRec32 = (0.6f * fVec13[IOTA&2047]);
			fRec38[0] = ((fSlow25 * fRec38[1]) + (fSlow26 * (fRec7[1] + fRec7[2])));
			fRec37[0] = ((fSlow79 * fRec37[1]) + (fSlow80 * (fRec7[1] + (fSlow81 * fRec38[0]))));
			fVec14[IOTA&8191] = (1e-20f + (0.35355339059327373f * fRec37[0]));
			float fTemp12 = ((fTemp9 + fVec14[(IOTA-iConst38)&8191]) - (0.6f * fRec35[1]));
			fVec15[IOTA&2047] = fTemp12;
			fRec35[0] = fVec15[(IOTA-iConst39)&2047];
			float 	fRec36 = (0.6f * fVec15[IOTA&2047]);
			fRec42[0] = ((fSlow25 * fRec42[1]) + (fSlow26 * (fRec3[1] + fRec3[2])));
			fRec41[0] = ((fSlow88 * fRec41[1]) + (fSlow89 * (fRec3[1] + (fSlow90 * fRec42[0]))));
			fVec16[IOTA&8191] = (1e-20f + (0.35355339059327373f * fRec41[0]));
			float fTemp13 = ((fVec16[(IOTA-iConst43)&8191] + fTemp9) - (0.6f * fRec39[1]));
			fVec17[IOTA&1023] = fTemp13;
			fRec39[0] = fVec17[(IOTA-iConst44)&1023];
			float 	fRec40 = (0.6f * fVec17[IOTA&1023]);
			float fTemp14 = (fRec40 + fRec36);
			float fTemp15 = (fRec28 + (fRec32 + fTemp14));
			fRec3[0] = (fRec11[1] + (fRec15[1] + (fRec19[1] + (fRec23[1] + (fRec27[1] + (fRec31[1] + (fRec35[1] + (fRec39[1] + (fRec12 + (fRec16 + (fRec20 + (fRec24 + fTemp15))))))))))));
			fRec4[0] = (0 - ((fRec11[1] + (fRec15[1] + (fRec19[1] + (fRec23[1] + (fRec12 + (fRec16 + (fRec24 + fRec20))))))) - (fRec27[1] + (fRec31[1] + (fRec35[1] + (fRec39[1] + fTemp15))))));
			float fTemp16 = (fRec32 + fRec28);
			fRec5[0] = (0 - ((fRec11[1] + (fRec15[1] + (fRec27[1] + (fRec31[1] + (fRec12 + (fRec16 + fTemp16)))))) - (fRec19[1] + (fRec23[1] + (fRec35[1] + (fRec39[1] + (fRec20 + (fRec24 + fTemp14))))))));
			fRec6[0] = (0 - ((fRec19[1] + (fRec23[1] + (fRec27[1] + (fRec31[1] + (fRec20 + (fRec24 + fTemp16)))))) - (fRec11[1] + (fRec15[1] + (fRec35[1] + (fRec39[1] + (fRec12 + (fRec16 + fTemp14))))))));
			float fTemp17 = (fRec36 + fRec28);
			float fTemp18 = (fRec40 + fRec32);
			fRec7[0] = (0 - ((fRec11[1] + (fRec19[1] + (fRec27[1] + (fRec35[1] + (fRec12 + (fRec20 + fTemp17)))))) - (fRec15[1] + (fRec23[1] + (fRec31[1] + (fRec39[1] + (fRec16 + (fRec24 + fTemp18))))))));
			fRec8[0] = (0 - ((fRec15[1] + (fRec23[1] + (fRec27[1] + (fRec35[1] + (fRec16 + (fRec24 + fTemp17)))))) - (fRec11[1] + (fRec19[1] + (fRec31[1] + (fRec39[1] + (fRec12 + (fRec20 + fTemp18))))))));
			float fTemp19 = (fRec36 + fRec32);
			float fTemp20 = (fRec40 + fRec28);
			fRec9[0] = (0 - ((fRec15[1] + (fRec19[1] + (fRec31[1] + (fRec35[1] + (fRec16 + (fRec20 + fTemp19)))))) - (fRec11[1] + (fRec23[1] + (fRec27[1] + (fRec39[1] + (fRec12 + (fRec24 + fTemp20))))))));
			fRec10[0] = (0 - ((fRec11[1] + (fRec23[1] + (fRec31[1] + (fRec35[1] + (fRec12 + (fRec24 + fTemp19)))))) - (fRec15[1] + (fRec19[1] + (fRec27[1] + (fRec39[1] + (fRec16 + (fRec20 + fTemp20))))))));
			float fTemp21 = (0.37f * (fRec4[0] + fRec5[0]));
			fRec2[0] = (0 - ((fTemp1 + (fSlow9 * fRec2[2])) - fTemp21));
			float fTemp22 = (fSlow9 * fRec2[0]);
			float fTemp23 = (0.5f * ((fTemp22 + (fRec2[2] + (fTemp21 + fTemp1))) + (fSlow7 * ((fTemp22 + (fTemp1 + fRec2[2])) - fTemp21))));
			fRec1[0] = (0 - ((fTemp0 + (fSlow4 * fRec1[2])) - fTemp23));
			float fTemp24 = (fSlow4 * fRec1[0]);
			fRec43[0] = ((0.999f * fRec43[1]) + fSlow91);
			float fTemp25 = (1.0f + fRec43[0]);
			float fTemp26 = (1.0f - (0.5f * fTemp25));
			output0[i] = (FAUSTFLOAT)(fRec0[0] * ((0.25f * (((fTemp24 + (fRec1[2] + (fTemp23 + fTemp0))) + (fSlow2 * ((fTemp24 + (fTemp0 + fRec1[2])) - fTemp23))) * fTemp25)) + (fVec10[IOTA&8191] * fTemp26)));
			float fTemp27 = (fSlow5 * fRec44[1]);
			float fTemp28 = (fSlow10 * fRec45[1]);
			float fTemp29 = (0.37f * (fRec4[0] - fRec5[0]));
			fRec45[0] = (0 - ((fTemp28 + (fSlow9 * fRec45[2])) - fTemp29));
			float fTemp30 = (fSlow9 * fRec45[0]);
			float fTemp31 = (0.5f * ((fTemp30 + (fRec45[2] + (fTemp29 + fTemp28))) + (fSlow7 * ((fTemp30 + (fTemp28 + fRec45[2])) - fTemp29))));
			fRec44[0] = (0 - ((fTemp27 + (fSlow4 * fRec44[2])) - fTemp31));
			float fTemp32 = (fSlow4 * fRec44[0]);
			output1[i] = (FAUSTFLOAT)(fRec0[0] * ((0.25f * (fTemp25 * ((fTemp32 + (fTemp27 + (fRec44[2] + fTemp31))) + (fSlow2 * ((fTemp32 + (fRec44[2] + fTemp27)) - fTemp31))))) + (fVec1[IOTA&8191] * fTemp26)));
			// post processing
			fRec44[2] = fRec44[1]; fRec44[1] = fRec44[0];
			fRec45[2] = fRec45[1]; fRec45[1] = fRec45[0];
			fRec43[1] = fRec43[0];
			fRec1[2] = fRec1[1]; fRec1[1] = fRec1[0];
			fRec2[2] = fRec2[1]; fRec2[1] = fRec2[0];
			fRec10[2] = fRec10[1]; fRec10[1] = fRec10[0];
			fRec9[2] = fRec9[1]; fRec9[1] = fRec9[0];
			fRec8[2] = fRec8[1]; fRec8[1] = fRec8[0];
			fRec7[2] = fRec7[1]; fRec7[1] = fRec7[0];
			fRec6[2] = fRec6[1]; fRec6[1] = fRec6[0];
			fRec5[2] = fRec5[1]; fRec5[1] = fRec5[0];
			fRec4[2] = fRec4[1]; fRec4[1] = fRec4[0];
			fRec3[2] = fRec3[1]; fRec3[1] = fRec3[0];
			fRec39[1] = fRec39[0];
			fRec41[1] = fRec41[0];
			fRec42[1] = fRec42[0];
			fRec35[1] = fRec35[0];
			fRec37[1] = fRec37[0];
			fRec38[1] = fRec38[0];
			fRec31[1] = fRec31[0];
			fRec33[1] = fRec33[0];
			fRec34[1] = fRec34[0];
			fRec27[1] = fRec27[0];
			fRec29[1] = fRec29[0];
			fRec30[1] = fRec30[0];
			fRec23[1] = fRec23[0];
			fRec25[1] = fRec25[0];
			fRec26[1] = fRec26[0];
			fRec19[1] = fRec19[0];
			fRec21[1] = fRec21[0];
			fRec22[1] = fRec22[0];
			fRec15[1] = fRec15[0];
			fRec17[1] = fRec17[0];
			fRec18[1] = fRec18[0];
			fRec11[1] = fRec11[0];
			IOTA = IOTA+1;
			fRec13[1] = fRec13[0];
			fRec14[1] = fRec14[0];
			fRec0[1] = fRec0[0];
		}
	}
};



/***************************END USER SECTION ***************************/

/*******************BEGIN ARCHITECTURE SECTION (part 2/2)***************/

mydsp DSP;

/********************END ARCHITECTURE SECTION (part 2/2)****************/


