/*
    Copyright (C) 2001-2013

    Developers:

    Paul Davis
    Jack O'Quin
    Steve Harris
    Rui Nuno Capela
    Devin Anderson
    Kai Vehmanen
    Ian Esten
    Dmitry Baikov
    Torben Hohn
    Rohan Drape
    Stephane Letz
    Christian Schoenebeck

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

    You are permitted to statically link against a binary version of
    the JACK library obtained from

        http://jackaudio.org/downloads/JackiOS-SDK.zip

    and distribute the resulting application without complying with section 6
    (requiring recipients of your application to be able to relink against
    modified versions of libjack).

    This permission is granted if and ONLY if the resulting application binary
    issue is intended for execution in any one of the following environments:

        * Apple Inc. iOS

    Additional environments may be added to this list in the future.
    All other terms of the LGPL continue to apply.

*/

#ifndef __statistics_h__
#define __statistics_h__

#ifdef __cplusplus
extern "C"
{
#endif

#include <jack/types.h>

/**
 * @file statistics.h
 * @brief Statistical informations from the JACK server.
 *
 * During execution, the JACK server collects a set of statistical
 * informations, which can be retrieved with the set of functions in this
 * file. The purpose is to allow i.e. the user to monitor the real-time
 * stability of the currently running JACK system with all its clients and
 * connections.
 */

/**
 * @return the maximum delay reported by the backend since
 * startup or reset.  When compared to the period size in usecs, this
 * can be used to estimate the ideal period size for a given setup.
 */
float jack_get_max_delayed_usecs (jack_client_t *client);

/**
 * @return the delay in microseconds due to the most recent XRUN
 * occurrence.  This probably only makes sense when called from a @ref
 * JackXRunCallback defined using jack_set_xrun_callback().
 */
float jack_get_xrun_delayed_usecs (jack_client_t *client);

/**
 * Reset the maximum delay counter.  This would be useful
 * to estimate the effect that a change to the configuration of a running
 * system (e.g. toggling kernel preemption) has on the delay
 * experienced by JACK, without having to restart the JACK engine.
 */
void jack_reset_max_delayed_usecs (jack_client_t *client);

#ifdef __cplusplus
}
#endif

#endif /* __statistics_h__ */
