/*
    Copyright (C) 2001-2013

    Developers:

    Paul Davis
    Jack O'Quin
    Steve Harris
    Rui Nuno Capela
    Devin Anderson
    Kai Vehmanen
    Ian Esten
    Dmitry Baikov
    Torben Hohn
    Rohan Drape
    Stephane Letz
    Christian Schoenebeck

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

    You are permitted to statically link against a binary version of
    the JACK library obtained from

        http://jackaudio.org/downloads/JackiOS-SDK.zip

    and distribute the resulting application without complying with section 6
    (requiring recipients of your application to be able to relink against
    modified versions of libjack).

    This permission is granted if and ONLY if the resulting application binary
    issue is intended for execution in any one of the following environments:

        * Apple Inc. iOS

    Additional environments may be added to this list in the future.
    All other terms of the LGPL continue to apply.

*/

#ifndef __jack_systemdeps_h__
#define __jack_systemdeps_h__

/**
 * @file
 * @brief System dependent type definitions.
 */

#ifndef POST_PACKED_STRUCTURE

    #ifdef __GNUC__
        /* POST_PACKED_STRUCTURE needs to be a macro which
           expands into a compiler directive. The directive must
           tell the compiler to arrange the preceding structure
           declaration so that it is packed on byte-boundaries rather 
           than use the natural alignment of the processor and/or
           compiler.
        */

        #define PRE_PACKED_STRUCTURE
        #if IS_APPLE_IOS
            #define POST_PACKED_STRUCTURE 
        #else   
            #define POST_PACKED_STRUCTURE __attribute__((__packed__))
        #endif

    #else
    
        #ifdef _MSC_VER
            #define PRE_PACKED_STRUCTURE1 __pragma(pack(push,1))
            #define PRE_PACKED_STRUCTURE    PRE_PACKED_STRUCTURE1
            /* PRE_PACKED_STRUCTURE needs to be a macro which
            expands into a compiler directive. The directive must
            tell the compiler to arrange the following structure
            declaration so that it is packed on byte-boundaries rather
            than use the natural alignment of the processor and/or
            compiler.
            */
            #define POST_PACKED_STRUCTURE ;__pragma(pack(pop))
            /* and POST_PACKED_STRUCTURE needs to be a macro which
            restores the packing to its previous setting */
        #else
            #define PRE_PACKED_STRUCTURE
            #define POST_PACKED_STRUCTURE
        #endif /* _MSC_VER */

    #endif /* __GNUC__ */

#endif

#if defined(WIN32) && !defined(__CYGWIN__) && !defined(GNU_WIN32)

    #include <windows.h>

    #ifdef _MSC_VER     /* Microsoft compiler */
        #define __inline__ inline
        #if (!defined(int8_t) && !defined(_STDINT_H))
            #define __int8_t_defined
            typedef char int8_t;
            typedef unsigned char uint8_t;
            typedef short int16_t;
            typedef unsigned short uint16_t;
            typedef long int32_t;
            typedef unsigned long uint32_t;
            typedef LONGLONG int64_t;
            typedef ULONGLONG uint64_t;
        #endif
    #elif __MINGW32__   /* MINGW */
        #include <stdint.h>
        #include <sys/types.h>
    #else               /* other compilers ...*/
        #include <inttypes.h>
        #include <pthread.h>
        #include <sys/types.h>
    #endif

    #if !defined(_PTHREAD_H) && !defined(PTHREAD_WIN32)
        /**
         *  to make jack API independent of different thread implementations,
         *  we define jack_native_thread_t to HANDLE here.
         */
        typedef HANDLE jack_native_thread_t;
    #else
        #ifdef PTHREAD_WIN32            // Added by JE - 10-10-2011
            #include <ptw32/pthread.h>  // Makes sure we #include the ptw32 version !
        #endif
        /**
         *  to make jack API independent of different thread implementations,
         *  we define jack_native_thread_t to pthread_t here.
         */
        typedef pthread_t jack_native_thread_t;
    #endif

#endif /* WIN32 && !__CYGWIN__ && !GNU_WIN32 */

#if defined(__APPLE__) || defined(__linux__) || defined(__sun__) || defined(sun) || defined(__unix__) || defined(__CYGWIN__) || defined(GNU_WIN32)

    #if defined(__CYGWIN__) || defined(GNU_WIN32)
        #include <stdint.h>
    #endif
        #include <inttypes.h>
        #include <pthread.h>
        #include <sys/types.h>

        /**
         *  to make jack API independent of different thread implementations,
         *  we define jack_native_thread_t to pthread_t here.
         */
        typedef pthread_t jack_native_thread_t;

#endif /* __APPLE__ || __linux__ || __sun__ || sun */

#if defined(__arm__)
    #undef POST_PACKED_STRUCTURE
    #define POST_PACKED_STRUCTURE
#endif /* __arm__ */

#endif /* __jack_systemdeps_h__ */
