//-----------------------------------------------------
// name: "Kisana"
// author: "Yann Orlarey"
//
// Code generated with Faust 0.9.59 (http://faust.grame.fr)
//-----------------------------------------------------
/* link with  */
#include <math.h>
#include <cmath>
template <int N> inline float faustpower(float x) 		{ return powf(x,N); } 
template <int N> inline double faustpower(double x) 	{ return pow(x,N); }
template <int N> inline int faustpower(int x) 			{ return faustpower<N/2>(x) * faustpower<N-N/2>(x); } 
template <> 	 inline int faustpower<0>(int x) 		{ return 1; }
template <> 	 inline int faustpower<1>(int x) 		{ return x; }
/************************************************************************

	IMPORTANT NOTE : this file contains two clearly delimited sections :
	the ARCHITECTURE section (in two parts) and the USER section. Each section
	is governed by its own copyright and license. Please check individually
	each section for license and copyright information.
*************************************************************************/

/*******************BEGIN ARCHITECTURE SECTION (part 1/2)****************/

/************************************************************************
    FAUST Architecture File
	Copyright (C) 2003-2011 GRAME, Centre National de Creation Musicale
    ---------------------------------------------------------------------
    This Architecture section is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3 of
	the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
	along with this program; If not, see <http://www.gnu.org/licenses/>.

	EXCEPTION : As a special exception, you may create a larger work
	that contains this FAUST architecture section and distribute
	that work under terms of your choice, so long as this FAUST
	architecture section is not modified.


 ************************************************************************
 ************************************************************************/

#define JACK_IOS 1

#import "FICocoaUI.h"

#ifndef __jack_dsp__
#define __jack_dsp__

#include <stdio.h>
#include <cstdlib>
#include <list>
#include <string.h>
#include <jack/jack.h>
#ifdef JACK_IOS
#include <jack/custom.h>
#endif

/******************************************************************************
*******************************************************************************

						An abstraction layer over audio layer

*******************************************************************************
*******************************************************************************/

#ifndef __audio__
#define __audio__
			
class dsp;

typedef void (* shutdown_callback)(const char* message, void* arg);

typedef void (* buffer_size_callback)(int frames, void* arg);

class audio {
    
 public:
			 audio() {}
	virtual ~audio() {}
	
	virtual bool init(const char* name, dsp*)               = 0;
	virtual bool start()                                    = 0;
	virtual void stop()                                     = 0;
    virtual void shutdown(shutdown_callback cb, void* arg)  {}
    
};
					
#endif

/******************************************************************************
*******************************************************************************

								FAUST DSP

*******************************************************************************
*******************************************************************************/

#ifndef __dsp__
#define __dsp__

#ifndef FAUSTFLOAT
#define FAUSTFLOAT float
#endif

class UI;

//----------------------------------------------------------------
//  signal processor definition
//----------------------------------------------------------------

class dsp {
 protected:
	int fSamplingFreq;
 public:
	dsp() {}
	virtual ~dsp() {}

	virtual int getNumInputs() 										= 0;
	virtual int getNumOutputs() 									= 0;
	virtual void buildUserInterface(UI* interface) 					= 0;
	virtual void init(int samplingRate) 							= 0;
 	virtual void compute(int len, FAUSTFLOAT** inputs, FAUSTFLOAT** outputs) 	= 0;
};

// On Intel set FZ (Flush to Zero) and DAZ (Denormals Are Zero)
// flags to avoid costly denormals
#ifdef __SSE__
    #include <xmmintrin.h>
    #ifdef __SSE2__
        #define AVOIDDENORMALS _mm_setcsr(_mm_getcsr() | 0x8040)
    #else
        #define AVOIDDENORMALS _mm_setcsr(_mm_getcsr() | 0x8000)
    #endif
#else
    #define AVOIDDENORMALS
#endif

#endif

/******************************************************************************
*******************************************************************************

							JACK AUDIO INTERFACE

*******************************************************************************
*******************************************************************************/

class jackaudio : public audio {

    private:

        dsp*			fDsp;               // FAUST DSP
        jack_client_t*	fClient;            // JACK client
    
        int				fNumInChans;		// number of input channels
        int				fNumOutChans;       // number of output channels
    
        jack_port_t**	fInputPorts;        // JACK input ports
        jack_port_t**	fOutputPorts;       // JACK output ports
    
        shutdown_callback fShutdown;        // Shutdown callback to be called by libjack
        void*           fShutdownArg;
        void*           fIconData;
        int             fIconSize;
        bool            fAutoConnect;
        
        std::list<std::pair<std::string, std::string> > fConnections;		// Connections list
    
        static int  _jack_srate(jack_nframes_t nframes, void* arg);
        static void _jack_shutdown(void* arg);
        static void _jack_info_shutdown(jack_status_t code, const char* reason, void* arg);
        static int  _jack_process(jack_nframes_t nframes, void* arg);
        static int  _jack_buffersize(jack_nframes_t nframes, void* arg);
    #ifdef _OPENMP
        static void* _jack_thread(void* arg);
    #endif
        void shutdown(const char* message)
        {
            if (fShutdown) {
                fShutdown(message, fShutdownArg);
            } else {
                exit(1); // By default
            }
        }
        
        // Save client connections
        void save_connections()
        {
            fConnections.clear();
            
             for (int i = 0; i < fNumInChans; i++) {
                const char** connected_port = jack_port_get_all_connections(fClient, fInputPorts[i]);
                if (connected_port != NULL) {
                    for (int port = 0; connected_port[port]; port++) {
                        fConnections.push_back(make_pair(connected_port[port], jack_port_name(fInputPorts[i])));
                        printf("INPUT %s ==> %s\n", connected_port[port], jack_port_name(fInputPorts[i]));
                    }
                    jack_free(connected_port);
                }
            }
       
            for (int i = 0; i < fNumOutChans; i++) {
                const char** connected_port = jack_port_get_all_connections(fClient, fOutputPorts[i]);
                if (connected_port != NULL) {
                    for (int port = 0; connected_port[port]; port++) {
                        fConnections.push_back(make_pair(jack_port_name(fOutputPorts[i]), connected_port[port]));
                        printf("OUTPUT %s ==> %s\n", jack_port_name(fOutputPorts[i]), connected_port[port]);
                    }
                    jack_free(connected_port);
                }
            }
        }

        // Restore previous client connections
        void restore_connections()
        {
            list<pair<string, string> >::const_iterator it;
            for (it = fConnections.begin(); it != fConnections.end(); it++) {
                pair<string, string> connection = *it;
                jack_connect(fClient, connection.first.c_str(), connection.second.c_str());
            }
        }
        
        // Connect to physical inputs/outputs
        void default_connections()
        {
            char** physicalOutPorts = (char**)jack_get_ports(fClient, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsOutput);
            if (physicalOutPorts != NULL) {
                for (int i = 0; i < fNumInChans && physicalOutPorts[i]; i++) {
                    jack_connect(fClient, physicalOutPorts[i], jack_port_name(fInputPorts[i]));
                }
                jack_free(physicalOutPorts);
            }
            
            char** physicalInPorts = (char**)jack_get_ports(fClient, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsInput);
            if (physicalInPorts != NULL) {
                for (int i = 0; i < fNumOutChans && physicalInPorts[i]; i++) {
                    jack_connect(fClient, jack_port_name(fOutputPorts[i]), physicalInPorts[i]);
                }
                jack_free(physicalInPorts);
            }
        }

    public:
        jackaudio(const void* icon_data = 0, size_t icon_size = 0, bool auto_connect = true) 
            : fDsp(0), fClient(0), fNumInChans(0), fNumOutChans(0), 
            fInputPorts(0), fOutputPorts(0), 
            fShutdown(0), fShutdownArg(0),
            fAutoConnect(auto_connect)
        {
            if (icon_data) {
                fIconData = malloc(icon_size);
                fIconSize = icon_size;
                memcpy(fIconData, icon_data, icon_size);
            } else {
                fIconData = NULL;
                fIconSize = 0;
            }
        }
        virtual ~jackaudio() 
        { 
            stop(); 
            if (fIconData) {
                free(fIconData);
            }
        }
        
        jack_client_t* getClient() { return fClient; }

        virtual bool init(const char* name, dsp* DSP) 
        {
            fDsp = DSP;
            if ((fClient = jack_client_open(name, JackNullOption, NULL)) == 0) {
                fprintf(stderr, "JACK server not running ?\n");
                return false;
            }
        #ifdef JACK_IOS
            jack_custom_publish_data(fClient, "icon.png", fIconData, fIconSize);
        #endif
            
        #ifdef _OPENMP
            jack_set_process_thread(fClient, _jack_thread, this);
        #else
            jack_set_process_callback(fClient, _jack_process, this);
        #endif

            jack_set_sample_rate_callback(fClient, _jack_srate, this);
            jack_set_buffer_size_callback(fClient, _jack_buffersize, this);
            jack_on_info_shutdown(fClient, _jack_info_shutdown, this);

            fNumInChans  = fDsp->getNumInputs();
            fNumOutChans = fDsp->getNumOutputs();
            
            fInputPorts = new jack_port_t*[fNumInChans];
            fOutputPorts = new jack_port_t*[fNumOutChans];
        
            for (int i = 0; i < fNumInChans; i++) {
                char buf[256];
                snprintf(buf, 256, "in_%d", i);
                fInputPorts[i] = jack_port_register(fClient, buf, JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
            }
            for (int i = 0; i < fNumOutChans; i++) {
                char buf[256];
                snprintf(buf, 256, "out_%d", i);
                fOutputPorts[i] = jack_port_register(fClient, buf, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
            }
            fDsp->init(jack_get_sample_rate(fClient));
            return true;
        }

        virtual bool start() 
        {
            if (jack_activate(fClient)) {
                fprintf(stderr, "Cannot activate client");
                return false;
            }
            
            if (fConnections.size() > 0) {
                restore_connections();
            } else if (fAutoConnect) {
                default_connections();
            }
            return true;
        }

        virtual void stop() 
        {
            if (fClient) {
                save_connections();
                jack_deactivate(fClient);
                for (int i = 0; i < fNumInChans; i++) {
                    jack_port_unregister(fClient, fInputPorts[i]);
                }
                for (int i = 0; i < fNumOutChans; i++) {
                    jack_port_unregister(fClient, fOutputPorts[i]);
                }
                jack_client_close(fClient);
                fClient = 0;
                
                delete[] fInputPorts;
                delete[] fOutputPorts;
            }
        }
    
        virtual void shutdown(shutdown_callback cb, void* arg)
        {
            fShutdown = cb;
            fShutdownArg = arg;
        }

        // jack callbacks
        int	process(jack_nframes_t nframes) 
        {
            AVOIDDENORMALS;
            // Retrieve JACK inputs/output audio buffers
            float* fInChannel[fNumInChans];
            for (int i = 0; i < fNumInChans; i++) {
                fInChannel[i] = (float*)jack_port_get_buffer(fInputPorts[i], nframes);
            }
            float* fOutChannel[fNumOutChans];
            for (int i = 0; i < fNumOutChans; i++) {
                fOutChannel[i] = (float*)jack_port_get_buffer(fOutputPorts[i], nframes);
            }
            fDsp->compute(nframes, fInChannel, fOutChannel);
            return 0;
        }

    #ifdef _OPENMP
        void process_thread() 
        {
            jack_nframes_t nframes;
            while (1) {
                nframes = jack_cycle_wait(fClient);
                process(nframes);
                jack_cycle_signal(fClient, 0);
            }
        }
    #endif
};

//----------------------------------------------------------------------------
// Jack Callbacks
//----------------------------------------------------------------------------

int jackaudio::_jack_srate(jack_nframes_t nframes, void* arg)
{
  	fprintf(stdout, "The sample rate is now %u/sec\n", nframes);
	return 0;
}

int jackaudio::_jack_buffersize(jack_nframes_t nframes, void* arg)
{
  	fprintf(stdout, "The buffer size is now %u/sec\n", nframes);
	return 0;
}

void jackaudio::_jack_info_shutdown(jack_status_t code, const char* reason, void* arg)
{
    fprintf(stderr, "%s\n", reason);
	jackaudio* audio = (jackaudio*)arg;
    audio->shutdown(reason);
}

int jackaudio::_jack_process(jack_nframes_t nframes, void* arg)
{
	jackaudio* audio = (jackaudio*)arg;
	return audio->process(nframes);
}

#ifdef _OPENMP
void* jackaudio::_jack_thread(void* arg)
{
	jackaudio* audio = (jackaudio*)arg;
	audio->process_thread();
	return 0;
}
#endif

#endif
/************************************************************************

	IMPORTANT NOTE : this file contains two clearly delimited sections :
	the ARCHITECTURE section (in two parts) and the USER section. Each section
	is governed by its own copyright and license. Please check individually
	each section for license and copyright information.
*************************************************************************/

/*******************BEGIN ARCHITECTURE SECTION (part 1/2)****************/

/************************************************************************
    FAUST Architecture File
	Copyright (C) 2003-2011 GRAME, Centre National de Creation Musicale
    ---------------------------------------------------------------------
    This Architecture section is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3 of
	the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
	along with this program; If not, see <http://www.gnu.org/licenses/>.

	EXCEPTION : As a special exception, you may create a larger work
	that contains this FAUST architecture section and distribute
	that work under terms of your choice, so long as this FAUST
	architecture section is not modified.


 ************************************************************************
 ************************************************************************/
/* link with  */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <math.h>
#include <errno.h>
#include <time.h>


#include <AudioToolbox/AudioConverter.h>
#include <AudioToolbox/AudioServices.h>
#include <AudioUnit/AudioUnit.h>

using namespace std;

/******************************************************************************
*******************************************************************************

							COREAUDIO INTERFACE

*******************************************************************************
*******************************************************************************/

#define OPEN_ERR -1
#define NO_ERR 0

class TiPhoneCoreAudioRenderer
{

private:

    AudioUnit fAUHAL;

    int	fDevNumInChans;
    int	fDevNumOutChans;

    int fHWNumInChans;
    int fHWNumOutChans;
    
    dsp* fDSP;

    AudioBufferList* fCAInputData;

    static OSStatus Render(void *inRefCon,
                           AudioUnitRenderActionFlags *ioActionFlags,
                           const AudioTimeStamp *inTimeStamp,
                           UInt32 inBusNumber,
                           UInt32 inNumberFrames,
                           AudioBufferList *ioData);

    static void InterruptionListener(void *inClientData, UInt32 inInterruption);
    
    static void AudioSessionPropertyListener(void* inClientData,
                                             AudioSessionPropertyID inID,
                                             UInt32 inDataSize,
                                             const void* inData);
    
    OSStatus Render(AudioUnitRenderActionFlags *ioActionFlags,
                     const AudioTimeStamp *inTimeStamp,
                     UInt32 inNumberFrames,
                     AudioBufferList *ioData);
                     
    int SetupMixing();

public:

    TiPhoneCoreAudioRenderer()
        :fAUHAL(0), fDevNumInChans(0), fDevNumOutChans(0),
        fHWNumInChans(0), fHWNumOutChans(0),
        fDSP(0), fCAInputData(NULL)
    {}

    virtual ~TiPhoneCoreAudioRenderer()
    {
        if (fCAInputData) {
            for (int i = 0; i < fDevNumInChans; i++) {
                free(fCAInputData->mBuffers[i].mData);
            }
            free(fCAInputData);
        }
    }

    int SetParameters(int bufferSize, int sampleRate);

    int Open(dsp* dsp, int inChan, int outChan, int bufferSize, int sampleRate);
    int Close();

    int Start();
    int Stop();

};

typedef TiPhoneCoreAudioRenderer* TiPhoneCoreAudioRendererPtr;

static void PrintStreamDesc(AudioStreamBasicDescription *inDesc)
{
    printf("- - - - - - - - - - - - - - - - - - - -\n");
    printf("  Sample Rate:%f\n", inDesc->mSampleRate);
    printf("  Format ID:%.*s\n", (int) sizeof(inDesc->mFormatID), (char*)&inDesc->mFormatID);
    printf("  Format Flags:%lX\n", inDesc->mFormatFlags);
    printf("  Bytes per Packet:%ld\n", inDesc->mBytesPerPacket);
    printf("  Frames per Packet:%ld\n", inDesc->mFramesPerPacket);
    printf("  Bytes per Frame:%ld\n", inDesc->mBytesPerFrame);
    printf("  Channels per Frame:%ld\n", inDesc->mChannelsPerFrame);
    printf("  Bits per Channel:%ld\n", inDesc->mBitsPerChannel);
    printf("- - - - - - - - - - - - - - - - - - - -\n");
}

static void printError(OSStatus err)
{
    switch (err) {
      	case kAudioConverterErr_FormatNotSupported:
            printf("error code : kAudioConverterErr_FormatNotSupported\n");
            break;
        case kAudioConverterErr_OperationNotSupported:
            printf("error code : kAudioConverterErr_OperationNotSupported\n");
            break;
        case kAudioConverterErr_PropertyNotSupported:
            printf("error code : kAudioConverterErr_PropertyNotSupported\n");
            break;
        case kAudioConverterErr_InvalidInputSize:
            printf("error code : kAudioConverterErr_InvalidInputSize\n");
            break;
        case kAudioConverterErr_InvalidOutputSize:
            printf("error code : kAudioConverterErr_InvalidOutputSize\n");
            break;
        case kAudioConverterErr_UnspecifiedError:
            printf("error code : kAudioConverterErr_UnspecifiedError\n");
            break;
        case kAudioConverterErr_BadPropertySizeError:
            printf("error code : kAudioConverterErr_BadPropertySizeError\n");
            break;
        case kAudioConverterErr_RequiresPacketDescriptionsError:
            printf("error code : kAudioConverterErr_RequiresPacketDescriptionsError\n");
            break;
        case kAudioConverterErr_InputSampleRateOutOfRange:
            printf("error code : kAudioConverterErr_InputSampleRateOutOfRange\n");
            break;
        case kAudioConverterErr_OutputSampleRateOutOfRange:
            printf("error code : kAudioConverterErr_OutputSampleRateOutOfRange\n");
            break;
        default:
            printf("error code : unknown\n");
            break;
    }
}

OSStatus TiPhoneCoreAudioRenderer::Render(void *inRefCon,
                                         AudioUnitRenderActionFlags *ioActionFlags,
                                         const AudioTimeStamp *inTimeStamp,
                                         UInt32,
                                         UInt32 inNumberFrames,
                                         AudioBufferList *ioData)
{
    return static_cast<TiPhoneCoreAudioRendererPtr>(inRefCon)->Render(ioActionFlags, inTimeStamp, inNumberFrames, ioData);
}

OSStatus TiPhoneCoreAudioRenderer::Render(AudioUnitRenderActionFlags *ioActionFlags,
                                         const AudioTimeStamp *inTimeStamp,
                                         UInt32 inNumberFrames,
                                         AudioBufferList *ioData)
{
    AudioUnitRender(fAUHAL, ioActionFlags, inTimeStamp, 1, inNumberFrames, fCAInputData);
    
    float* fInChannel[fDevNumInChans];
    float* fOutChannel[fDevNumOutChans];
    
    if (fHWNumInChans == 1) {
        // Mono ==> stereo
        for (int chan = 0; chan < fDevNumInChans; chan++) {
            fInChannel[chan] = (float*)fCAInputData->mBuffers[0].mData;
        }
    } else {
        for (int chan = 0; chan < fDevNumInChans; chan++) {
            fInChannel[chan] = (float*)fCAInputData->mBuffers[chan].mData;
        }
    }
    
    for (int chan = 0; chan < fDevNumOutChans; chan++) {
        fOutChannel[chan] = (float*)ioData->mBuffers[chan].mData;
    }
    
    fDSP->compute((int)inNumberFrames, fInChannel, fOutChannel);
 	return 0;
}

void TiPhoneCoreAudioRenderer::InterruptionListener(void *inClientData, UInt32 inInterruption)
{
	TiPhoneCoreAudioRenderer *obj = (TiPhoneCoreAudioRenderer*)inClientData;
	printf("Session interrupted! --- %s ---", inInterruption == kAudioSessionBeginInterruption ? "Begin Interruption" : "End Interruption");

	if (inInterruption == kAudioSessionEndInterruption) {
		// make sure we are again the active session
		AudioSessionSetActive(true);
        obj->SetupMixing();
        AudioOutputUnitStart(obj->fAUHAL);
	}

	if (inInterruption == kAudioSessionBeginInterruption) {
		AudioOutputUnitStop(obj->fAUHAL);
    }
}

int TiPhoneCoreAudioRenderer::SetupMixing()
{
    UInt32 allowMixing = true;
    OSStatus err = AudioSessionSetProperty(kAudioSessionProperty_OverrideCategoryMixWithOthers, sizeof(allowMixing), &allowMixing);
    if (err != noErr) {
        printf("Could not set audio session mixing\n");
        printError(err);
        return -1;
    } else {
        return 0;
    }
}
   
void TiPhoneCoreAudioRenderer::AudioSessionPropertyListener(void* inClientData,
                                                 AudioSessionPropertyID inID,
                                                 UInt32 inDataSize,
                                                 const void* inData)
{
    TiPhoneCoreAudioRenderer *obj = (TiPhoneCoreAudioRenderer*)inData;
    switch (inID) {
        case kAudioSessionProperty_ServerDied: {
            printf("kAudioSessionProperty_ServerDied\n");
            break;
        }
        case kAudioSessionProperty_AudioRouteChange: {
            printf("kAudioSessionProperty_AudioRouteChange\n");
            obj->SetupMixing();
            break;
        }
        case kAudioSessionProperty_AudioInputAvailable: {
            printf("kAudioSessionProperty_AudioInputAvailable\n");
            obj->SetupMixing();
            break;
        }
    }
}

int TiPhoneCoreAudioRenderer::SetParameters(int bufferSize, int samplerate)
{
    OSStatus err;
    UInt32 outSize;
    UInt32 enableIO;
	AudioStreamBasicDescription srcFormat, dstFormat;
    
    printf("OpenDefault fDevNumInChans = %d fDevNumOutChans = %d bufferSize = %d samplerate = %d\n", fDevNumInChans, fDevNumOutChans, bufferSize, samplerate);
     
    err = AudioSessionSetActive(true);
    if (err != noErr) {
        printf("Couldn't set audio session active\n");
        printError(err);
        return OPEN_ERR;
    }
    
    AudioSessionAddPropertyListener(kAudioSessionProperty_AudioRouteChange, AudioSessionPropertyListener, this);
    AudioSessionAddPropertyListener(kAudioSessionProperty_AudioInputAvailable, AudioSessionPropertyListener, this);
    AudioSessionAddPropertyListener(kAudioSessionProperty_ServerDied, AudioSessionPropertyListener, this);
    
    UInt32 audioCategory;
    if ((fDevNumInChans > 0) && (fDevNumOutChans > 0)) {
        audioCategory = kAudioSessionCategory_PlayAndRecord;
        printf("kAudioSessionCategory_PlayAndRecord\n");
    } else if (fDevNumInChans > 0) {
        audioCategory = kAudioSessionCategory_RecordAudio;
        printf("kAudioSessionCategory_RecordAudio\n");
    } else  if (fDevNumOutChans > 0) {
        audioCategory = kAudioSessionCategory_MediaPlayback;
        printf("kAudioSessionCategory_MediaPlayback\n");
    }
    
	err = AudioSessionSetProperty(kAudioSessionProperty_AudioCategory, sizeof(audioCategory), &audioCategory);
    if (err != noErr) {
        printf("Couldn't set audio category\n");
        printError(err);
        return OPEN_ERR;
    }
    
    if (SetupMixing() < 0) {
        return OPEN_ERR;
    }
    
    Float64 hwSampleRate;
    outSize = sizeof(hwSampleRate);
	err = AudioSessionGetProperty(kAudioSessionProperty_CurrentHardwareSampleRate, &outSize, &hwSampleRate);
    if (err != noErr) {
        printf("Couldn't get hw sample rate\n");
        printError(err);
        return OPEN_ERR;
    } else {
        printf("Get hw sample rate %f\n", hwSampleRate);
    }
    
    Float32 hwBufferSize;
    outSize = sizeof(hwBufferSize);
	err = AudioSessionGetProperty(kAudioSessionProperty_CurrentHardwareIOBufferDuration, &outSize, &hwBufferSize);
    if (err != noErr) {
        printf("Couldn't get hw buffer duration\n");
        printError(err);
        return OPEN_ERR;
    } else {
        printf("Get hw buffer duration %f\n", hwBufferSize);
    }
    
    outSize = sizeof(fHWNumInChans);
	err = AudioSessionGetProperty(kAudioSessionProperty_CurrentHardwareInputNumberChannels, &outSize, &fHWNumInChans);
    if (err != noErr) {
        printf("Couldn't get hw input channels\n");
        fHWNumInChans = 0;
        printError(err);
    } else {
        printf("Get hw input channels %d\n", fHWNumInChans);
    }
    
    if (fDevNumInChans > fHWNumInChans) {
        if ((fDevNumInChans == 2 && fHWNumInChans == 1)) { // Only mono ==> stereo is accepted
            printf("hardware ins = %d wanted = %d, so mono to stereo\n", fHWNumInChans, fDevNumInChans);
        } else {
            printf("Error: hardware ins = %d wanted = %d\n", fHWNumInChans, fDevNumInChans);
            return OPEN_ERR;
        }
    }
    
    outSize = sizeof(fHWNumOutChans);
	err = AudioSessionGetProperty(kAudioSessionProperty_CurrentHardwareOutputNumberChannels, &outSize, &fHWNumOutChans);
    if (err != noErr) {
        printf("Couldn't get hw output channels\n");
        printError(err);
        fHWNumOutChans = 0;
    } else {
        printf("Get hw output channels %d\n", fHWNumOutChans);
    }
    
    if (fDevNumOutChans > fHWNumOutChans) {
        printf("Error: hardware outs = %d wanted = %d\n", fHWNumOutChans, fDevNumOutChans);
        return OPEN_ERR;
    }
    
    Float32 preferredPeriodDuration = float(bufferSize) / float(samplerate);
    printf("preferredPeriodDuration %f \n", preferredPeriodDuration);
    
	err = AudioSessionSetProperty(kAudioSessionProperty_PreferredHardwareIOBufferDuration, sizeof(preferredPeriodDuration), &preferredPeriodDuration);
    if (err != noErr) {
        printf("Couldn't set i/o buffer duration\n");
        printError(err);
        return OPEN_ERR;
    }
    
    Float32 actualPeriodDuration;
    
    err = AudioSessionGetProperty(kAudioSessionProperty_PreferredHardwareIOBufferDuration, &outSize, &actualPeriodDuration);
    if (err != noErr) {
        printf("Couldn't get hw buffer duration\n");
        printError(err);
        return OPEN_ERR;
    }
    
    printf("preferredPeriodDuration %f actualPeriodDuration %f\n", preferredPeriodDuration, actualPeriodDuration);
    if (preferredPeriodDuration != actualPeriodDuration) {
        printf("Couldn't set hw buffer duration\n");
        return OPEN_ERR;
    }
    
    Float64 preferredSamplerate = float(samplerate);
	err = AudioSessionSetProperty(kAudioSessionProperty_PreferredHardwareSampleRate, sizeof(preferredSamplerate), &preferredSamplerate);
    if (err != noErr) {
        printf("Couldn't set i/o sample rate\n");
        printError(err);
        return OPEN_ERR;
    }
    
    // AUHAL
    AudioComponentDescription cd = {kAudioUnitType_Output, kAudioUnitSubType_RemoteIO, kAudioUnitManufacturer_Apple, 0, 0};
    AudioComponent HALOutput = AudioComponentFindNext(NULL, &cd);
    
    err = AudioComponentInstanceNew(HALOutput, &fAUHAL);
    if (err != noErr) {
		printf("Error calling OpenAComponent\n");
        printError(err);
        goto error;
	}
    
    enableIO = 1;
    err = AudioUnitSetProperty(fAUHAL, kAudioOutputUnitProperty_EnableIO, kAudioUnitScope_Output, 0, &enableIO, sizeof(enableIO));
    if (err != noErr) {
        printf("Error calling AudioUnitSetProperty - kAudioOutputUnitProperty_EnableIO, kAudioUnitScope_Output\n");
        printError(err);
        goto error;
    }
    
    enableIO = 1;
    err = AudioUnitSetProperty(fAUHAL, kAudioOutputUnitProperty_EnableIO, kAudioUnitScope_Input, 1, &enableIO, sizeof(enableIO));
    if (err != noErr) {
        printf("Error calling AudioUnitSetProperty - kAudioOutputUnitProperty_EnableIO, kAudioUnitScope_Input\n");
        printError(err);
        goto error;
    }
    
    UInt32 maxFPS;
    outSize = sizeof(maxFPS);
    err = AudioUnitGetProperty(fAUHAL, kAudioUnitProperty_MaximumFramesPerSlice, kAudioUnitScope_Global, 0, &maxFPS, &outSize);
    if (err != noErr) {
        printf("Couldn't get kAudioUnitProperty_MaximumFramesPerSlice\n");
        printError(err);
        goto error;
    } else {
        printf("Get kAudioUnitProperty_MaximumFramesPerSlice %d\n", (unsigned int)maxFPS);
    }
    
    err = AudioUnitSetProperty(fAUHAL, kAudioUnitProperty_MaximumFramesPerSlice, kAudioUnitScope_Global, 1, (UInt32*)&bufferSize, sizeof(UInt32));
    if (err != noErr) {
        printf("Error calling AudioUnitSetProperty - kAudioUnitProperty_MaximumFramesPerSlice\n");
        printError(err);
        goto error;
    }
    
    err = AudioUnitSetProperty(fAUHAL, kAudioUnitProperty_MaximumFramesPerSlice, kAudioUnitScope_Global, 0, (UInt32*)&bufferSize, sizeof(UInt32));
    if (err != noErr) {
        printf("Error calling AudioUnitSetProperty - kAudioUnitProperty_MaximumFramesPerSlice\n");
        printError(err);
        goto error;
    }
    
    err = AudioUnitInitialize(fAUHAL);
    if (err != noErr) {
		printf("Cannot initialize AUHAL unit\n");
		printError(err);
        goto error;
	}
    
    // Setting format
    
    if (fDevNumInChans > 0) {
        outSize = sizeof(AudioStreamBasicDescription);
        err = AudioUnitGetProperty(fAUHAL, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Output, 1, &srcFormat, &outSize);
        if (err != noErr) {
            printf("Error calling AudioUnitGetProperty - kAudioUnitProperty_StreamFormat kAudioUnitScope_Output\n");
            printError(err);
        }
        PrintStreamDesc(&srcFormat);
        
        srcFormat.mFormatID = kAudioFormatLinearPCM;
        srcFormat.mFormatFlags = kAudioFormatFlagsNativeFloatPacked | kLinearPCMFormatFlagIsNonInterleaved;
        srcFormat.mBytesPerPacket = sizeof(AudioUnitSampleType);
        srcFormat.mFramesPerPacket = 1;
        srcFormat.mBytesPerFrame = sizeof(AudioUnitSampleType);
        srcFormat.mChannelsPerFrame = fDevNumInChans;
        srcFormat.mBitsPerChannel = 32;
        
        PrintStreamDesc(&srcFormat);
        
        err = AudioUnitSetProperty(fAUHAL, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Output, 1, &srcFormat, sizeof(AudioStreamBasicDescription));
        if (err != noErr) {
            printf("Error calling AudioUnitSetProperty - kAudioUnitProperty_StreamFormat kAudioUnitScope_Output\n");
            printError(err);
        }
    }
    
    if (fDevNumOutChans > 0) {
        outSize = sizeof(AudioStreamBasicDescription);
        err = AudioUnitGetProperty(fAUHAL, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Input, 0, &dstFormat, &outSize);
        if (err != noErr) {
            printf("Error calling AudioUnitGetProperty - kAudioUnitProperty_StreamFormat kAudioUnitScope_Input\n");
            printError(err);
        }
        PrintStreamDesc(&dstFormat);
        
        dstFormat.mFormatID = kAudioFormatLinearPCM;
        dstFormat.mFormatFlags = kAudioFormatFlagsNativeFloatPacked | kLinearPCMFormatFlagIsNonInterleaved;
        dstFormat.mBytesPerPacket = sizeof(AudioUnitSampleType);
        dstFormat.mFramesPerPacket = 1;
        dstFormat.mBytesPerFrame = sizeof(AudioUnitSampleType);
        dstFormat.mChannelsPerFrame = fDevNumOutChans;
        dstFormat.mBitsPerChannel = 32;
        
        PrintStreamDesc(&dstFormat);
        
        err = AudioUnitSetProperty(fAUHAL, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Input, 0, &dstFormat, sizeof(AudioStreamBasicDescription));
        if (err != noErr) {
            printf("Error calling AudioUnitSetProperty - kAudioUnitProperty_StreamFormat kAudioUnitScope_Input\n");
            printError(err);
        }
    }
    
    if (fDevNumInChans > 0 && fDevNumOutChans == 0) {
        AURenderCallbackStruct output;
        output.inputProc = Render;
        output.inputProcRefCon = this;
        err = AudioUnitSetProperty(fAUHAL, kAudioOutputUnitProperty_SetInputCallback, kAudioUnitScope_Global, 0, &output, sizeof(output));
        if (err != noErr) {
            printf("Error calling AudioUnitSetProperty - kAudioUnitProperty_SetRenderCallback 1\n");
            printError(err);
            goto error;
        }
    } else {
        AURenderCallbackStruct output;
        output.inputProc = Render;
        output.inputProcRefCon = this;
        err = AudioUnitSetProperty(fAUHAL, kAudioUnitProperty_SetRenderCallback, kAudioUnitScope_Input, 0, &output, sizeof(output));
        if (err != noErr) {
            printf("Error calling AudioUnitSetProperty - kAudioUnitProperty_SetRenderCallback 0\n");
            printError(err);
            goto error;
        }
    }
    
    // Prepare buffers
    fCAInputData = (AudioBufferList*)malloc(sizeof(float) + fDevNumInChans * sizeof(AudioBuffer));
    fCAInputData->mNumberBuffers = fDevNumInChans;
    for (int i = 0; i < fDevNumInChans; i++) {
        fCAInputData->mBuffers[i].mNumberChannels = 1;
        fCAInputData->mBuffers[i].mDataByteSize = bufferSize * sizeof(float);
        fCAInputData->mBuffers[i].mData = malloc(bufferSize * sizeof(float));
    }
    
    return NO_ERR;
    
error:
    AudioUnitUninitialize(fAUHAL);
    AudioComponentInstanceDispose(fAUHAL);
    return OPEN_ERR;
}

int TiPhoneCoreAudioRenderer::Open(dsp* dsp, int inChan, int outChan, int buffersize, int samplerate)
{
    fDSP = dsp;
    fDevNumInChans = inChan;
    fDevNumOutChans = outChan;
    
    // Initialize and configure the audio session
    OSStatus err = AudioSessionInitialize(NULL, NULL, InterruptionListener, this);
    if (err != noErr && err != kAudioSessionAlreadyInitialized) {
        printf("Couldn't initialize audio session\n");
        printError(err);
        return OPEN_ERR;
    }
    
    if (SetParameters(buffersize, samplerate) < 0) {
        printf("Cannot set parameters to CoreAudio device\n");
        return OPEN_ERR;
    }
    
    return NO_ERR;
}

int TiPhoneCoreAudioRenderer::Close()
{
 	AudioUnitUninitialize(fAUHAL);
    AudioComponentInstanceDispose(fAUHAL);
    return NO_ERR;
}

int TiPhoneCoreAudioRenderer::Start()
{
    AudioSessionSetActive(true);
	OSStatus err = AudioOutputUnitStart(fAUHAL);
    
    if (err != noErr) {
        printf("Error while opening device : device open error\n");
        return OPEN_ERR;
    } else {
        return NO_ERR;
	}
}

int TiPhoneCoreAudioRenderer::Stop()
{
    AudioSessionSetActive(false);
    OSStatus err = AudioOutputUnitStop(fAUHAL);
    
    if (err != noErr) {
        printf("Error while closing device : device close error\n");
        return OPEN_ERR;
    } else {
        return NO_ERR;
	}
}

/******************************************************************************
 *******************************************************************************
 
                                CORE AUDIO INTERFACE
 
 *******************************************************************************
 *******************************************************************************/
class iosaudio : public audio {
    
    TiPhoneCoreAudioRenderer fAudioDevice;
	int fSampleRate, fFramesPerBuf;
    
public:
    iosaudio(int srate, int fpb) : fSampleRate(srate), fFramesPerBuf(fpb) {}
	virtual ~iosaudio() {}
    
	virtual bool init(const char* /*name*/, dsp* DSP) 
    {
    	DSP->init(fSampleRate);
		if (fAudioDevice.Open(DSP, DSP->getNumInputs(), DSP->getNumOutputs(), fFramesPerBuf, fSampleRate) < 0) {
			printf("Cannot open iOS audio device\n");
    		return false;
		}
        return true;
    }
    
	virtual bool start() 
    {
		if (fAudioDevice.Start() < 0) {
			printf("Cannot start iOS audio device\n");
			return false;
		}
		return true;
	}
    
	virtual void stop() 
    {
		fAudioDevice.Stop();
		fAudioDevice.Close();
 	}
    
};


/********************END ARCHITECTURE SECTION (part 2/2)****************/


/************************************************************************
 ************************************************************************
    FAUST Architecture File
	Copyright (C) 2003-2011 GRAME, Centre National de Creation Musicale
    ---------------------------------------------------------------------
    This Architecture section is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3 of
	the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
	along with this program; If not, see <http://www.gnu.org/licenses/>.

 ************************************************************************
 ************************************************************************/
 
#ifndef __misc__
#define __misc__

#include <map>
#include <string.h>
#include <stdlib.h>

/************************************************************************
 ************************************************************************
    FAUST Architecture File
	Copyright (C) 2003-2011 GRAME, Centre National de Creation Musicale
    ---------------------------------------------------------------------
    This Architecture section is free software; you can redistribute it
    and/or modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3 of
	the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
	along with this program; If not, see <http://www.gnu.org/licenses/>.

 ************************************************************************
 ************************************************************************/
 
#ifndef __meta__
#define __meta__

struct Meta
{
    virtual void declare(const char* key, const char* value) = 0;
};

#endif


struct XXXX_Meta : std::map<const char*, const char*>
{
    void declare(const char* key, const char* value) { (*this)[key]=value; }
};

struct MY_Meta : Meta, std::map<const char*, const char*>
{
    void declare(const char* key, const char* value) { (*this)[key]=value; }
};

inline int	lsr(int x, int n)	{ return int(((unsigned int)x) >> n); }
inline int 	int2pow2(int x)		{ int r=0; while ((1<<r)<x) r++; return r; }

long lopt(char *argv[], const char *name, long def)
{
	int	i;
	for (i = 0; argv[i]; i++) if (!strcmp(argv[i], name)) return atoi(argv[i+1]);
	return def;
}

char* lopts(char *argv[], const char *name, char* def)
{
	int	i;
	for (i = 0; argv[i]; i++) if (!strcmp(argv[i], name)) return argv[i+1];
	return def;
}
#endif


/******************************************************************************
*******************************************************************************

							       VECTOR INTRINSICS

*******************************************************************************
*******************************************************************************/


/******************************************************************************
*******************************************************************************

								USER INTERFACE

*******************************************************************************
*******************************************************************************/


/********************END ARCHITECTURE SECTION (part 1/2)****************/

/**************************BEGIN USER SECTION **************************/

#ifndef FAUSTFLOAT
#define FAUSTFLOAT float
#endif  

typedef long double quad;

#ifndef FAUSTCLASS 
#define FAUSTCLASS mydsp
#endif

class mydsp : public dsp {
  private:
	class SIG0 {
	  private:
		int 	fSamplingFreq;
	  public:
		int getNumInputs() 	{ return 0; }
		int getNumOutputs() 	{ return 1; }
		void init(int samplingFreq) {
			fSamplingFreq = samplingFreq;
		}
		void fill (int count, float output[]) {
			// SECTION : 1
			for (int i=0; i<count; i++) {
				output[i] = 0.0f;
			}
		}
	};


	int 	iVec0_perm[4];
	int 	iConst0;
	float 	fConst1;
	float 	fConst2;
	float 	fConst3;
	float 	fRec5_perm[4];
	float 	fConst4;
	float 	fRec6_perm[4];
	int 	iRec11_perm[4];
	int 	iConst5;
	int 	iRec14_perm[4];
	int 	iConst6;
	int 	iRec13_perm[4];
	int 	iYec0_perm[4];
	FAUSTFLOAT 	fslider0;
	int 	iVec1_perm[4];
	int 	iRec15_perm[4];
	int 	iRec16_perm[4];
	float 	ftbl0[16];
	int 	iYec1_perm[4];
	float 	fRec12_perm[4];
	FAUSTFLOAT 	fslider1;
	float 	fYec2[128];
	int 	fYec2_idx;
	int 	fYec2_idx_save;
	float 	fRec10_perm[4];
	FAUSTFLOAT 	fslider2;
	int 	iVec2_perm[4];
	int 	iRec19_perm[4];
	float 	ftbl1[16];
	int 	iYec3_perm[4];
	float 	fRec18_perm[4];
	FAUSTFLOAT 	fslider3;
	float 	fYec4[512];
	int 	fYec4_idx;
	int 	fYec4_idx_save;
	float 	fRec17_perm[4];
	int 	iYec5_perm[4];
	float 	fRec21_perm[4];
	float 	fYec6[512];
	int 	fYec6_idx;
	int 	fYec6_idx_save;
	float 	fRec20_perm[4];
	int 	iYec7_perm[4];
	float 	fRec23_perm[4];
	float 	fYec8[512];
	int 	fYec8_idx;
	int 	fYec8_idx_save;
	float 	fRec22_perm[4];
	int 	iYec9_perm[4];
	float 	fRec25_perm[4];
	float 	fYec10[256];
	int 	fYec10_idx;
	int 	fYec10_idx_save;
	float 	fRec24_perm[4];
	int 	iYec11_perm[4];
	float 	fRec27_perm[4];
	float 	fYec12[256];
	int 	fYec12_idx;
	int 	fYec12_idx_save;
	float 	fRec26_perm[4];
	int 	iYec13_perm[4];
	float 	fRec29_perm[4];
	float 	fYec14[256];
	int 	fYec14_idx;
	int 	fYec14_idx_save;
	float 	fRec28_perm[4];
	int 	iYec15_perm[4];
	float 	fRec31_perm[4];
	float 	fYec16[256];
	int 	fYec16_idx;
	int 	fYec16_idx_save;
	float 	fRec30_perm[4];
	int 	iYec17_perm[4];
	float 	fRec33_perm[4];
	float 	fYec18[256];
	int 	fYec18_idx;
	int 	fYec18_idx_save;
	float 	fRec32_perm[4];
	int 	iYec19_perm[4];
	float 	fRec35_perm[4];
	float 	fYec20[256];
	int 	fYec20_idx;
	int 	fYec20_idx_save;
	float 	fRec34_perm[4];
	int 	iYec21_perm[4];
	float 	fRec37_perm[4];
	float 	fYec22[256];
	int 	fYec22_idx;
	int 	fYec22_idx_save;
	float 	fRec36_perm[4];
	int 	iYec23_perm[4];
	float 	fRec39_perm[4];
	float 	fYec24[128];
	int 	fYec24_idx;
	int 	fYec24_idx_save;
	float 	fRec38_perm[4];
	int 	iYec25_perm[4];
	float 	fRec41_perm[4];
	float 	fYec26[256];
	int 	fYec26_idx;
	int 	fYec26_idx_save;
	float 	fRec40_perm[4];
	int 	iYec27_perm[4];
	float 	fRec43_perm[4];
	float 	fYec28[256];
	int 	fYec28_idx;
	int 	fYec28_idx_save;
	float 	fRec42_perm[4];
	int 	iYec29_perm[4];
	float 	fRec45_perm[4];
	float 	fYec30[256];
	int 	fYec30_idx;
	int 	fYec30_idx_save;
	float 	fRec44_perm[4];
	int 	iYec31_perm[4];
	float 	fRec47_perm[4];
	float 	fYec32[256];
	int 	fYec32_idx;
	int 	fYec32_idx_save;
	float 	fRec46_perm[4];
	int 	iYec33_perm[4];
	float 	fRec49_perm[4];
	float 	fYec34[256];
	int 	fYec34_idx;
	int 	fYec34_idx_save;
	float 	fRec48_perm[4];
	int 	iYec35_perm[4];
	float 	fRec51_perm[4];
	float 	fYec36[128];
	int 	fYec36_idx;
	int 	fYec36_idx_save;
	float 	fRec50_perm[4];
	int 	iYec37_perm[4];
	float 	fRec53_perm[4];
	float 	fYec38[128];
	int 	fYec38_idx;
	int 	fYec38_idx_save;
	float 	fRec52_perm[4];
	int 	iYec39_perm[4];
	float 	fRec55_perm[4];
	float 	fYec40[128];
	int 	fYec40_idx;
	int 	fYec40_idx_save;
	float 	fRec54_perm[4];
	int 	iYec41_perm[4];
	float 	fRec57_perm[4];
	float 	fYec42[128];
	int 	fYec42_idx;
	int 	fYec42_idx_save;
	float 	fRec56_perm[4];
	int 	iYec43_perm[4];
	float 	fRec59_perm[4];
	float 	fYec44[128];
	int 	fYec44_idx;
	int 	fYec44_idx_save;
	float 	fRec58_perm[4];
	float 	fConst7;
	float 	fConst8;
	float 	fRec9_perm[4];
	float 	fConst9;
	float 	fConst10;
	float 	fRec8_perm[4];
	float 	fRec60_perm[4];
	float 	fRec61_perm[4];
	float 	fRec7_perm[4];
	float 	fConst11;
	float 	fConst12;
	float 	fConst13;
	float 	fConst14;
	FAUSTFLOAT 	fslider4;
	FAUSTFLOAT 	fslider5;
	float 	fRec4_perm[4];
	float 	fConst15;
	float 	fRec3_perm[4];
	float 	fConst16;
	float 	fRec2_perm[4];
	float 	fConst17;
	float 	fRec1_perm[4];
	float 	fRec0_perm[4];
	FAUSTFLOAT 	fslider6;
	int 	iVec3_perm[4];
	int 	iRec72_perm[4];
	float 	ftbl2[16];
	int 	iYec45_perm[4];
	float 	fRec71_perm[4];
	FAUSTFLOAT 	fslider7;
	float 	fYec46[64];
	int 	fYec46_idx;
	int 	fYec46_idx_save;
	float 	fRec70_perm[4];
	int 	iYec47_perm[4];
	float 	fRec74_perm[4];
	float 	fYec48[128];
	int 	fYec48_idx;
	int 	fYec48_idx_save;
	float 	fRec73_perm[4];
	int 	iYec49_perm[4];
	float 	fRec76_perm[4];
	float 	fYec50[128];
	int 	fYec50_idx;
	int 	fYec50_idx_save;
	float 	fRec75_perm[4];
	int 	iYec51_perm[4];
	float 	fRec78_perm[4];
	float 	fYec52[128];
	int 	fYec52_idx;
	int 	fYec52_idx_save;
	float 	fRec77_perm[4];
	int 	iYec53_perm[4];
	float 	fRec80_perm[4];
	float 	fYec54[128];
	int 	fYec54_idx;
	int 	fYec54_idx_save;
	float 	fRec79_perm[4];
	int 	iYec55_perm[4];
	float 	fRec82_perm[4];
	float 	fYec56[128];
	int 	fYec56_idx;
	int 	fYec56_idx_save;
	float 	fRec81_perm[4];
	int 	iYec57_perm[4];
	float 	fRec84_perm[4];
	float 	fYec58[128];
	int 	fYec58_idx;
	int 	fYec58_idx_save;
	float 	fRec83_perm[4];
	int 	iYec59_perm[4];
	float 	fRec86_perm[4];
	float 	fYec60[128];
	int 	fYec60_idx;
	int 	fYec60_idx_save;
	float 	fRec85_perm[4];
	int 	iYec61_perm[4];
	float 	fRec88_perm[4];
	float 	fYec62[64];
	int 	fYec62_idx;
	int 	fYec62_idx_save;
	float 	fRec87_perm[4];
	int 	iYec63_perm[4];
	float 	fRec90_perm[4];
	float 	fYec64[64];
	int 	fYec64_idx;
	int 	fYec64_idx_save;
	float 	fRec89_perm[4];
	int 	iYec65_perm[4];
	float 	fRec92_perm[4];
	float 	fYec66[64];
	int 	fYec66_idx;
	int 	fYec66_idx_save;
	float 	fRec91_perm[4];
	float 	fRec69_perm[4];
	float 	fRec68_perm[4];
	float 	fRec93_perm[4];
	float 	fRec94_perm[4];
	float 	fRec67_perm[4];
	float 	fRec66_perm[4];
	float 	fRec65_perm[4];
	float 	fRec64_perm[4];
	float 	fRec63_perm[4];
	float 	fRec62_perm[4];
	float 	fRec102_perm[4];
	float 	fRec101_perm[4];
	float 	fRec103_perm[4];
	float 	fRec104_perm[4];
	float 	fRec100_perm[4];
	float 	fRec99_perm[4];
	float 	fRec98_perm[4];
	float 	fRec97_perm[4];
	float 	fRec96_perm[4];
	float 	fRec95_perm[4];
	float 	fRec112_perm[4];
	float 	fRec111_perm[4];
	float 	fRec113_perm[4];
	float 	fRec114_perm[4];
	float 	fRec110_perm[4];
	float 	fRec109_perm[4];
	float 	fRec108_perm[4];
	float 	fRec107_perm[4];
	float 	fRec106_perm[4];
	float 	fRec105_perm[4];
  public:
	static void metadata(Meta* m) 	{ 
		m->declare("name", "Kisana");
		m->declare("author", "Yann Orlarey");
		m->declare("effect.lib/name", "Faust Audio Effect Library");
		m->declare("effect.lib/author", "Julius O. Smith (jos at ccrma.stanford.edu)");
		m->declare("effect.lib/copyright", "Julius O. Smith III");
		m->declare("effect.lib/version", "1.33");
		m->declare("effect.lib/license", "STK-4.3");
		m->declare("filter.lib/name", "Faust Filter Library");
		m->declare("filter.lib/author", "Julius O. Smith (jos at ccrma.stanford.edu)");
		m->declare("filter.lib/copyright", "Julius O. Smith III");
		m->declare("filter.lib/version", "1.29");
		m->declare("filter.lib/license", "STK-4.3");
		m->declare("filter.lib/reference", "https://ccrma.stanford.edu/~jos/filters/");
		m->declare("music.lib/name", "Music Library");
		m->declare("music.lib/author", "GRAME");
		m->declare("music.lib/copyright", "GRAME");
		m->declare("music.lib/version", "1.0");
		m->declare("music.lib/license", "LGPL with exception");
		m->declare("math.lib/name", "Math Library");
		m->declare("math.lib/author", "GRAME");
		m->declare("math.lib/copyright", "GRAME");
		m->declare("math.lib/version", "1.0");
		m->declare("math.lib/license", "LGPL with exception");
		m->declare("oscillator.lib/name", "Faust Oscillator Library");
		m->declare("oscillator.lib/author", "Julius O. Smith (jos at ccrma.stanford.edu)");
		m->declare("oscillator.lib/copyright", "Julius O. Smith III");
		m->declare("oscillator.lib/version", "1.11");
		m->declare("oscillator.lib/license", "STK-4.3");
	}

	virtual int getNumInputs() 	{ return 0; }
	virtual int getNumOutputs() 	{ return 4; }
	static void classInit(int samplingFreq) {
	}
	virtual void instanceInit(int samplingFreq) {
		fSamplingFreq = samplingFreq;
		for (int i=0; i<4; i++) iVec0_perm[i]=0;
		iConst0 = min(192000, max(1, fSamplingFreq));
		fConst1 = (3.141592653589793f / float(iConst0));
		fConst2 = sinf(fConst1);
		fConst3 = cosf(fConst1);
		for (int i=0; i<4; i++) fRec5_perm[i]=0;
		fConst4 = (0 - fConst2);
		for (int i=0; i<4; i++) fRec6_perm[i]=0;
		for (int i=0; i<4; i++) iRec11_perm[i]=0;
		iConst5 = int((0.16666666666666666f * iConst0));
		for (int i=0; i<4; i++) iRec14_perm[i]=0;
		iConst6 = (60 * iConst0);
		for (int i=0; i<4; i++) iRec13_perm[i]=0;
		for (int i=0; i<4; i++) iYec0_perm[i]=0;
		fslider0 = 0.0f;
		for (int i=0; i<4; i++) iVec1_perm[i]=0;
		for (int i=0; i<4; i++) iRec15_perm[i]=0;
		for (int i=0; i<4; i++) iRec16_perm[i]=0;
		SIG0 sig0;
		sig0.init(samplingFreq);
		sig0.fill(16,ftbl0);
		for (int i=0; i<4; i++) iYec1_perm[i]=0;
		for (int i=0; i<4; i++) fRec12_perm[i]=0;
		fslider1 = -6.0f;
		for (int i=0; i<128; i++) fYec2[i]=0;
		fYec2_idx = 0;
		fYec2_idx_save = 0;
		for (int i=0; i<4; i++) fRec10_perm[i]=0;
		fslider2 = 0.0f;
		for (int i=0; i<4; i++) iVec2_perm[i]=0;
		for (int i=0; i<4; i++) iRec19_perm[i]=0;
		sig0.init(samplingFreq);
		sig0.fill(16,ftbl1);
		for (int i=0; i<4; i++) iYec3_perm[i]=0;
		for (int i=0; i<4; i++) fRec18_perm[i]=0;
		fslider3 = -6.0f;
		for (int i=0; i<512; i++) fYec4[i]=0;
		fYec4_idx = 0;
		fYec4_idx_save = 0;
		for (int i=0; i<4; i++) fRec17_perm[i]=0;
		for (int i=0; i<4; i++) iYec5_perm[i]=0;
		for (int i=0; i<4; i++) fRec21_perm[i]=0;
		for (int i=0; i<512; i++) fYec6[i]=0;
		fYec6_idx = 0;
		fYec6_idx_save = 0;
		for (int i=0; i<4; i++) fRec20_perm[i]=0;
		for (int i=0; i<4; i++) iYec7_perm[i]=0;
		for (int i=0; i<4; i++) fRec23_perm[i]=0;
		for (int i=0; i<512; i++) fYec8[i]=0;
		fYec8_idx = 0;
		fYec8_idx_save = 0;
		for (int i=0; i<4; i++) fRec22_perm[i]=0;
		for (int i=0; i<4; i++) iYec9_perm[i]=0;
		for (int i=0; i<4; i++) fRec25_perm[i]=0;
		for (int i=0; i<256; i++) fYec10[i]=0;
		fYec10_idx = 0;
		fYec10_idx_save = 0;
		for (int i=0; i<4; i++) fRec24_perm[i]=0;
		for (int i=0; i<4; i++) iYec11_perm[i]=0;
		for (int i=0; i<4; i++) fRec27_perm[i]=0;
		for (int i=0; i<256; i++) fYec12[i]=0;
		fYec12_idx = 0;
		fYec12_idx_save = 0;
		for (int i=0; i<4; i++) fRec26_perm[i]=0;
		for (int i=0; i<4; i++) iYec13_perm[i]=0;
		for (int i=0; i<4; i++) fRec29_perm[i]=0;
		for (int i=0; i<256; i++) fYec14[i]=0;
		fYec14_idx = 0;
		fYec14_idx_save = 0;
		for (int i=0; i<4; i++) fRec28_perm[i]=0;
		for (int i=0; i<4; i++) iYec15_perm[i]=0;
		for (int i=0; i<4; i++) fRec31_perm[i]=0;
		for (int i=0; i<256; i++) fYec16[i]=0;
		fYec16_idx = 0;
		fYec16_idx_save = 0;
		for (int i=0; i<4; i++) fRec30_perm[i]=0;
		for (int i=0; i<4; i++) iYec17_perm[i]=0;
		for (int i=0; i<4; i++) fRec33_perm[i]=0;
		for (int i=0; i<256; i++) fYec18[i]=0;
		fYec18_idx = 0;
		fYec18_idx_save = 0;
		for (int i=0; i<4; i++) fRec32_perm[i]=0;
		for (int i=0; i<4; i++) iYec19_perm[i]=0;
		for (int i=0; i<4; i++) fRec35_perm[i]=0;
		for (int i=0; i<256; i++) fYec20[i]=0;
		fYec20_idx = 0;
		fYec20_idx_save = 0;
		for (int i=0; i<4; i++) fRec34_perm[i]=0;
		for (int i=0; i<4; i++) iYec21_perm[i]=0;
		for (int i=0; i<4; i++) fRec37_perm[i]=0;
		for (int i=0; i<256; i++) fYec22[i]=0;
		fYec22_idx = 0;
		fYec22_idx_save = 0;
		for (int i=0; i<4; i++) fRec36_perm[i]=0;
		for (int i=0; i<4; i++) iYec23_perm[i]=0;
		for (int i=0; i<4; i++) fRec39_perm[i]=0;
		for (int i=0; i<128; i++) fYec24[i]=0;
		fYec24_idx = 0;
		fYec24_idx_save = 0;
		for (int i=0; i<4; i++) fRec38_perm[i]=0;
		for (int i=0; i<4; i++) iYec25_perm[i]=0;
		for (int i=0; i<4; i++) fRec41_perm[i]=0;
		for (int i=0; i<256; i++) fYec26[i]=0;
		fYec26_idx = 0;
		fYec26_idx_save = 0;
		for (int i=0; i<4; i++) fRec40_perm[i]=0;
		for (int i=0; i<4; i++) iYec27_perm[i]=0;
		for (int i=0; i<4; i++) fRec43_perm[i]=0;
		for (int i=0; i<256; i++) fYec28[i]=0;
		fYec28_idx = 0;
		fYec28_idx_save = 0;
		for (int i=0; i<4; i++) fRec42_perm[i]=0;
		for (int i=0; i<4; i++) iYec29_perm[i]=0;
		for (int i=0; i<4; i++) fRec45_perm[i]=0;
		for (int i=0; i<256; i++) fYec30[i]=0;
		fYec30_idx = 0;
		fYec30_idx_save = 0;
		for (int i=0; i<4; i++) fRec44_perm[i]=0;
		for (int i=0; i<4; i++) iYec31_perm[i]=0;
		for (int i=0; i<4; i++) fRec47_perm[i]=0;
		for (int i=0; i<256; i++) fYec32[i]=0;
		fYec32_idx = 0;
		fYec32_idx_save = 0;
		for (int i=0; i<4; i++) fRec46_perm[i]=0;
		for (int i=0; i<4; i++) iYec33_perm[i]=0;
		for (int i=0; i<4; i++) fRec49_perm[i]=0;
		for (int i=0; i<256; i++) fYec34[i]=0;
		fYec34_idx = 0;
		fYec34_idx_save = 0;
		for (int i=0; i<4; i++) fRec48_perm[i]=0;
		for (int i=0; i<4; i++) iYec35_perm[i]=0;
		for (int i=0; i<4; i++) fRec51_perm[i]=0;
		for (int i=0; i<128; i++) fYec36[i]=0;
		fYec36_idx = 0;
		fYec36_idx_save = 0;
		for (int i=0; i<4; i++) fRec50_perm[i]=0;
		for (int i=0; i<4; i++) iYec37_perm[i]=0;
		for (int i=0; i<4; i++) fRec53_perm[i]=0;
		for (int i=0; i<128; i++) fYec38[i]=0;
		fYec38_idx = 0;
		fYec38_idx_save = 0;
		for (int i=0; i<4; i++) fRec52_perm[i]=0;
		for (int i=0; i<4; i++) iYec39_perm[i]=0;
		for (int i=0; i<4; i++) fRec55_perm[i]=0;
		for (int i=0; i<128; i++) fYec40[i]=0;
		fYec40_idx = 0;
		fYec40_idx_save = 0;
		for (int i=0; i<4; i++) fRec54_perm[i]=0;
		for (int i=0; i<4; i++) iYec41_perm[i]=0;
		for (int i=0; i<4; i++) fRec57_perm[i]=0;
		for (int i=0; i<128; i++) fYec42[i]=0;
		fYec42_idx = 0;
		fYec42_idx_save = 0;
		for (int i=0; i<4; i++) fRec56_perm[i]=0;
		for (int i=0; i<4; i++) iYec43_perm[i]=0;
		for (int i=0; i<4; i++) fRec59_perm[i]=0;
		for (int i=0; i<128; i++) fYec44[i]=0;
		fYec44_idx = 0;
		fYec44_idx_save = 0;
		for (int i=0; i<4; i++) fRec58_perm[i]=0;
		fConst7 = expf((0 - (1e+01f / float(iConst0))));
		fConst8 = (1.0f - fConst7);
		for (int i=0; i<4; i++) fRec9_perm[i]=0;
		fConst9 = (1413.7166941154069f / float(iConst0));
		fConst10 = (2827.4333882308138f / float(iConst0));
		for (int i=0; i<4; i++) fRec8_perm[i]=0;
		for (int i=0; i<4; i++) fRec60_perm[i]=0;
		for (int i=0; i<4; i++) fRec61_perm[i]=0;
		for (int i=0; i<4; i++) fRec7_perm[i]=0;
		fConst11 = expf((0 - (3141.592653589793f / float(iConst0))));
		fConst12 = (0 - (2 * fConst11));
		fConst13 = (1.5f / float(iConst0));
		fConst14 = faustpower<2>(fConst11);
		fslider4 = 0.0f;
		fslider5 = 0.0f;
		for (int i=0; i<4; i++) fRec4_perm[i]=0;
		fConst15 = (2.25f / float(iConst0));
		for (int i=0; i<4; i++) fRec3_perm[i]=0;
		fConst16 = (3.375f / float(iConst0));
		for (int i=0; i<4; i++) fRec2_perm[i]=0;
		fConst17 = (5.0625f / float(iConst0));
		for (int i=0; i<4; i++) fRec1_perm[i]=0;
		for (int i=0; i<4; i++) fRec0_perm[i]=0;
		fslider6 = 0.0f;
		for (int i=0; i<4; i++) iVec3_perm[i]=0;
		for (int i=0; i<4; i++) iRec72_perm[i]=0;
		sig0.init(samplingFreq);
		sig0.fill(16,ftbl2);
		for (int i=0; i<4; i++) iYec45_perm[i]=0;
		for (int i=0; i<4; i++) fRec71_perm[i]=0;
		fslider7 = -6.0f;
		for (int i=0; i<64; i++) fYec46[i]=0;
		fYec46_idx = 0;
		fYec46_idx_save = 0;
		for (int i=0; i<4; i++) fRec70_perm[i]=0;
		for (int i=0; i<4; i++) iYec47_perm[i]=0;
		for (int i=0; i<4; i++) fRec74_perm[i]=0;
		for (int i=0; i<128; i++) fYec48[i]=0;
		fYec48_idx = 0;
		fYec48_idx_save = 0;
		for (int i=0; i<4; i++) fRec73_perm[i]=0;
		for (int i=0; i<4; i++) iYec49_perm[i]=0;
		for (int i=0; i<4; i++) fRec76_perm[i]=0;
		for (int i=0; i<128; i++) fYec50[i]=0;
		fYec50_idx = 0;
		fYec50_idx_save = 0;
		for (int i=0; i<4; i++) fRec75_perm[i]=0;
		for (int i=0; i<4; i++) iYec51_perm[i]=0;
		for (int i=0; i<4; i++) fRec78_perm[i]=0;
		for (int i=0; i<128; i++) fYec52[i]=0;
		fYec52_idx = 0;
		fYec52_idx_save = 0;
		for (int i=0; i<4; i++) fRec77_perm[i]=0;
		for (int i=0; i<4; i++) iYec53_perm[i]=0;
		for (int i=0; i<4; i++) fRec80_perm[i]=0;
		for (int i=0; i<128; i++) fYec54[i]=0;
		fYec54_idx = 0;
		fYec54_idx_save = 0;
		for (int i=0; i<4; i++) fRec79_perm[i]=0;
		for (int i=0; i<4; i++) iYec55_perm[i]=0;
		for (int i=0; i<4; i++) fRec82_perm[i]=0;
		for (int i=0; i<128; i++) fYec56[i]=0;
		fYec56_idx = 0;
		fYec56_idx_save = 0;
		for (int i=0; i<4; i++) fRec81_perm[i]=0;
		for (int i=0; i<4; i++) iYec57_perm[i]=0;
		for (int i=0; i<4; i++) fRec84_perm[i]=0;
		for (int i=0; i<128; i++) fYec58[i]=0;
		fYec58_idx = 0;
		fYec58_idx_save = 0;
		for (int i=0; i<4; i++) fRec83_perm[i]=0;
		for (int i=0; i<4; i++) iYec59_perm[i]=0;
		for (int i=0; i<4; i++) fRec86_perm[i]=0;
		for (int i=0; i<128; i++) fYec60[i]=0;
		fYec60_idx = 0;
		fYec60_idx_save = 0;
		for (int i=0; i<4; i++) fRec85_perm[i]=0;
		for (int i=0; i<4; i++) iYec61_perm[i]=0;
		for (int i=0; i<4; i++) fRec88_perm[i]=0;
		for (int i=0; i<64; i++) fYec62[i]=0;
		fYec62_idx = 0;
		fYec62_idx_save = 0;
		for (int i=0; i<4; i++) fRec87_perm[i]=0;
		for (int i=0; i<4; i++) iYec63_perm[i]=0;
		for (int i=0; i<4; i++) fRec90_perm[i]=0;
		for (int i=0; i<64; i++) fYec64[i]=0;
		fYec64_idx = 0;
		fYec64_idx_save = 0;
		for (int i=0; i<4; i++) fRec89_perm[i]=0;
		for (int i=0; i<4; i++) iYec65_perm[i]=0;
		for (int i=0; i<4; i++) fRec92_perm[i]=0;
		for (int i=0; i<64; i++) fYec66[i]=0;
		fYec66_idx = 0;
		fYec66_idx_save = 0;
		for (int i=0; i<4; i++) fRec91_perm[i]=0;
		for (int i=0; i<4; i++) fRec69_perm[i]=0;
		for (int i=0; i<4; i++) fRec68_perm[i]=0;
		for (int i=0; i<4; i++) fRec93_perm[i]=0;
		for (int i=0; i<4; i++) fRec94_perm[i]=0;
		for (int i=0; i<4; i++) fRec67_perm[i]=0;
		for (int i=0; i<4; i++) fRec66_perm[i]=0;
		for (int i=0; i<4; i++) fRec65_perm[i]=0;
		for (int i=0; i<4; i++) fRec64_perm[i]=0;
		for (int i=0; i<4; i++) fRec63_perm[i]=0;
		for (int i=0; i<4; i++) fRec62_perm[i]=0;
		for (int i=0; i<4; i++) fRec102_perm[i]=0;
		for (int i=0; i<4; i++) fRec101_perm[i]=0;
		for (int i=0; i<4; i++) fRec103_perm[i]=0;
		for (int i=0; i<4; i++) fRec104_perm[i]=0;
		for (int i=0; i<4; i++) fRec100_perm[i]=0;
		for (int i=0; i<4; i++) fRec99_perm[i]=0;
		for (int i=0; i<4; i++) fRec98_perm[i]=0;
		for (int i=0; i<4; i++) fRec97_perm[i]=0;
		for (int i=0; i<4; i++) fRec96_perm[i]=0;
		for (int i=0; i<4; i++) fRec95_perm[i]=0;
		for (int i=0; i<4; i++) fRec112_perm[i]=0;
		for (int i=0; i<4; i++) fRec111_perm[i]=0;
		for (int i=0; i<4; i++) fRec113_perm[i]=0;
		for (int i=0; i<4; i++) fRec114_perm[i]=0;
		for (int i=0; i<4; i++) fRec110_perm[i]=0;
		for (int i=0; i<4; i++) fRec109_perm[i]=0;
		for (int i=0; i<4; i++) fRec108_perm[i]=0;
		for (int i=0; i<4; i++) fRec107_perm[i]=0;
		for (int i=0; i<4; i++) fRec106_perm[i]=0;
		for (int i=0; i<4; i++) fRec105_perm[i]=0;
	}
	virtual void init(int samplingFreq) {
		classInit(samplingFreq);
		instanceInit(samplingFreq);
	}
	virtual void buildUserInterface(UI* interface) {
		interface->openVerticalBox("kisana");
		interface->openHorizontalBox("Loops");
		interface->openVerticalBox("loop48");
		interface->declare(&fslider3, "1", "");
		interface->declare(&fslider3, "style", "knob");
		interface->addVerticalSlider("level", &fslider3, -6.0f, -6e+01f, 0.0f, 0.01f);
		interface->openHorizontalBox("automat");
		interface->declare(&fslider2, "1", "");
		interface->addVerticalSlider("note", &fslider2, 0.0f, 0.0f, 11.0f, 1.0f);
		interface->closeBox();
		interface->closeBox();
		interface->openVerticalBox("loop60");
		interface->declare(&fslider1, "1", "");
		interface->declare(&fslider1, "style", "knob");
		interface->addVerticalSlider("level", &fslider1, -6.0f, -6e+01f, 0.0f, 0.01f);
		interface->openHorizontalBox("automat");
		interface->declare(&fslider0, "1", "");
		interface->addVerticalSlider("note", &fslider0, 0.0f, 0.0f, 11.0f, 1.0f);
		interface->closeBox();
		interface->closeBox();
		interface->openVerticalBox("loop72");
		interface->declare(&fslider7, "1", "");
		interface->declare(&fslider7, "style", "knob");
		interface->addVerticalSlider("level", &fslider7, -6.0f, -6e+01f, 0.0f, 0.01f);
		interface->openHorizontalBox("automat");
		interface->declare(&fslider6, "1", "");
		interface->addVerticalSlider("note", &fslider6, 0.0f, 0.0f, 11.0f, 1.0f);
		interface->closeBox();
		interface->closeBox();
		interface->closeBox();
		interface->addHorizontalSlider("Phaser", &fslider5, 0.0f, 0.0f, 0.999f, 0.001f);
		interface->addHorizontalSlider("WahWah", &fslider4, 0.0f, 0.0f, 1.0f, 0.01f);
		interface->closeBox();
	}
	virtual void compute (int fullcount, FAUSTFLOAT** input, FAUSTFLOAT** output) {
		int 	iVec0_tmp[32+4];
		float 	fRec5_tmp[32+4];
		float 	fRec6_tmp[32+4];
		int 	iRec11_tmp[32+4];
		int 	iRec14_tmp[32+4];
		int 	iRec13_tmp[32+4];
		int 	iYec0_tmp[32+4];
		int 	iZec0[32];
		int 	iVec1_tmp[32+4];
		int 	iRec15_tmp[32+4];
		int 	iRec16_tmp[32+4];
		float 	fZec1[32];
		int 	iYec1_tmp[32+4];
		float 	fRec12_tmp[32+4];
		float 	fRec10_tmp[32+4];
		int 	iVec2_tmp[32+4];
		int 	iRec19_tmp[32+4];
		float 	fZec2[32];
		int 	iYec3_tmp[32+4];
		float 	fRec18_tmp[32+4];
		float 	fRec17_tmp[32+4];
		int 	iYec5_tmp[32+4];
		float 	fRec21_tmp[32+4];
		float 	fRec20_tmp[32+4];
		int 	iYec7_tmp[32+4];
		float 	fRec23_tmp[32+4];
		float 	fRec22_tmp[32+4];
		int 	iYec9_tmp[32+4];
		float 	fRec25_tmp[32+4];
		float 	fRec24_tmp[32+4];
		int 	iYec11_tmp[32+4];
		float 	fRec27_tmp[32+4];
		float 	fRec26_tmp[32+4];
		int 	iYec13_tmp[32+4];
		float 	fRec29_tmp[32+4];
		float 	fRec28_tmp[32+4];
		int 	iYec15_tmp[32+4];
		float 	fRec31_tmp[32+4];
		float 	fRec30_tmp[32+4];
		int 	iYec17_tmp[32+4];
		float 	fRec33_tmp[32+4];
		float 	fRec32_tmp[32+4];
		int 	iYec19_tmp[32+4];
		float 	fRec35_tmp[32+4];
		float 	fRec34_tmp[32+4];
		int 	iYec21_tmp[32+4];
		float 	fRec37_tmp[32+4];
		float 	fRec36_tmp[32+4];
		int 	iYec23_tmp[32+4];
		float 	fRec39_tmp[32+4];
		float 	fRec38_tmp[32+4];
		int 	iYec25_tmp[32+4];
		float 	fRec41_tmp[32+4];
		float 	fRec40_tmp[32+4];
		int 	iYec27_tmp[32+4];
		float 	fRec43_tmp[32+4];
		float 	fRec42_tmp[32+4];
		int 	iYec29_tmp[32+4];
		float 	fRec45_tmp[32+4];
		float 	fRec44_tmp[32+4];
		int 	iYec31_tmp[32+4];
		float 	fRec47_tmp[32+4];
		float 	fRec46_tmp[32+4];
		int 	iYec33_tmp[32+4];
		float 	fRec49_tmp[32+4];
		float 	fRec48_tmp[32+4];
		int 	iYec35_tmp[32+4];
		float 	fRec51_tmp[32+4];
		float 	fRec50_tmp[32+4];
		int 	iYec37_tmp[32+4];
		float 	fRec53_tmp[32+4];
		float 	fRec52_tmp[32+4];
		int 	iYec39_tmp[32+4];
		float 	fRec55_tmp[32+4];
		float 	fRec54_tmp[32+4];
		int 	iYec41_tmp[32+4];
		float 	fRec57_tmp[32+4];
		float 	fRec56_tmp[32+4];
		int 	iYec43_tmp[32+4];
		float 	fRec59_tmp[32+4];
		float 	fRec58_tmp[32+4];
		float 	fZec3[32];
		float 	fZec4[32];
		float 	fZec5[32];
		float 	fRec9_tmp[32+4];
		float 	fZec6[32];
		float 	fZec7[32];
		float 	fRec8_tmp[32+4];
		float 	fRec60_tmp[32+4];
		float 	fRec61_tmp[32+4];
		float 	fRec7_tmp[32+4];
		float 	fZec8[32];
		float 	fZec9[32];
		float 	fZec10[32];
		float 	fRec4_tmp[32+4];
		float 	fZec11[32];
		float 	fZec12[32];
		float 	fRec3_tmp[32+4];
		float 	fZec13[32];
		float 	fZec14[32];
		float 	fRec2_tmp[32+4];
		float 	fZec15[32];
		float 	fZec16[32];
		float 	fRec1_tmp[32+4];
		float 	fRec0_tmp[32+4];
		int 	iVec3_tmp[32+4];
		int 	iRec72_tmp[32+4];
		float 	fZec17[32];
		int 	iYec45_tmp[32+4];
		float 	fRec71_tmp[32+4];
		float 	fRec70_tmp[32+4];
		int 	iYec47_tmp[32+4];
		float 	fRec74_tmp[32+4];
		float 	fRec73_tmp[32+4];
		int 	iYec49_tmp[32+4];
		float 	fRec76_tmp[32+4];
		float 	fRec75_tmp[32+4];
		int 	iYec51_tmp[32+4];
		float 	fRec78_tmp[32+4];
		float 	fRec77_tmp[32+4];
		int 	iYec53_tmp[32+4];
		float 	fRec80_tmp[32+4];
		float 	fRec79_tmp[32+4];
		int 	iYec55_tmp[32+4];
		float 	fRec82_tmp[32+4];
		float 	fRec81_tmp[32+4];
		int 	iYec57_tmp[32+4];
		float 	fRec84_tmp[32+4];
		float 	fRec83_tmp[32+4];
		int 	iYec59_tmp[32+4];
		float 	fRec86_tmp[32+4];
		float 	fRec85_tmp[32+4];
		int 	iYec61_tmp[32+4];
		float 	fRec88_tmp[32+4];
		float 	fRec87_tmp[32+4];
		int 	iYec63_tmp[32+4];
		float 	fRec90_tmp[32+4];
		float 	fRec89_tmp[32+4];
		int 	iYec65_tmp[32+4];
		float 	fRec92_tmp[32+4];
		float 	fRec91_tmp[32+4];
		float 	fZec18[32];
		float 	fZec19[32];
		float 	fRec69_tmp[32+4];
		float 	fZec20[32];
		float 	fZec21[32];
		float 	fRec68_tmp[32+4];
		float 	fRec93_tmp[32+4];
		float 	fRec94_tmp[32+4];
		float 	fRec67_tmp[32+4];
		float 	fZec22[32];
		float 	fZec23[32];
		float 	fZec24[32];
		float 	fRec66_tmp[32+4];
		float 	fZec25[32];
		float 	fZec26[32];
		float 	fRec65_tmp[32+4];
		float 	fZec27[32];
		float 	fZec28[32];
		float 	fRec64_tmp[32+4];
		float 	fZec29[32];
		float 	fZec30[32];
		float 	fRec63_tmp[32+4];
		float 	fRec62_tmp[32+4];
		float 	fZec31[32];
		float 	fZec32[32];
		float 	fRec102_tmp[32+4];
		float 	fZec33[32];
		float 	fZec34[32];
		float 	fRec101_tmp[32+4];
		float 	fRec103_tmp[32+4];
		float 	fRec104_tmp[32+4];
		float 	fRec100_tmp[32+4];
		float 	fZec35[32];
		float 	fRec99_tmp[32+4];
		float 	fZec36[32];
		float 	fRec98_tmp[32+4];
		float 	fZec37[32];
		float 	fRec97_tmp[32+4];
		float 	fZec38[32];
		float 	fRec96_tmp[32+4];
		float 	fRec95_tmp[32+4];
		float 	fZec39[32];
		float 	fZec40[32];
		float 	fRec112_tmp[32+4];
		float 	fZec41[32];
		float 	fZec42[32];
		float 	fRec111_tmp[32+4];
		float 	fRec113_tmp[32+4];
		float 	fRec114_tmp[32+4];
		float 	fRec110_tmp[32+4];
		float 	fZec43[32];
		float 	fRec109_tmp[32+4];
		float 	fZec44[32];
		float 	fRec108_tmp[32+4];
		float 	fZec45[32];
		float 	fRec107_tmp[32+4];
		float 	fZec46[32];
		float 	fRec106_tmp[32+4];
		float 	fRec105_tmp[32+4];
		int* 	iVec0 = &iVec0_tmp[4];
		float* 	fRec5 = &fRec5_tmp[4];
		float* 	fRec6 = &fRec6_tmp[4];
		int* 	iRec11 = &iRec11_tmp[4];
		int* 	iRec14 = &iRec14_tmp[4];
		int* 	iRec13 = &iRec13_tmp[4];
		int* 	iYec0 = &iYec0_tmp[4];
		int 	iSlow0 = int(fslider0);
		int* 	iVec1 = &iVec1_tmp[4];
		int* 	iRec15 = &iRec15_tmp[4];
		int* 	iRec16 = &iRec16_tmp[4];
		int 	iSlow1 = (iSlow0 <= 0.0f);
		int* 	iYec1 = &iYec1_tmp[4];
		float* 	fRec12 = &fRec12_tmp[4];
		float 	fSlow2 = (4.656612875245797e-10f * powf(10,(0.05f * fslider1)));
		float* 	fRec10 = &fRec10_tmp[4];
		int 	iSlow3 = int(fslider2);
		int* 	iVec2 = &iVec2_tmp[4];
		int* 	iRec19 = &iRec19_tmp[4];
		int 	iSlow4 = (iSlow3 <= 0.0f);
		int* 	iYec3 = &iYec3_tmp[4];
		float* 	fRec18 = &fRec18_tmp[4];
		float 	fSlow5 = (4.656612875245797e-10f * powf(10,(0.05f * fslider3)));
		float* 	fRec17 = &fRec17_tmp[4];
		int* 	iYec5 = &iYec5_tmp[4];
		float* 	fRec21 = &fRec21_tmp[4];
		float* 	fRec20 = &fRec20_tmp[4];
		int* 	iYec7 = &iYec7_tmp[4];
		float* 	fRec23 = &fRec23_tmp[4];
		float* 	fRec22 = &fRec22_tmp[4];
		int* 	iYec9 = &iYec9_tmp[4];
		float* 	fRec25 = &fRec25_tmp[4];
		float* 	fRec24 = &fRec24_tmp[4];
		int* 	iYec11 = &iYec11_tmp[4];
		float* 	fRec27 = &fRec27_tmp[4];
		float* 	fRec26 = &fRec26_tmp[4];
		int* 	iYec13 = &iYec13_tmp[4];
		float* 	fRec29 = &fRec29_tmp[4];
		float* 	fRec28 = &fRec28_tmp[4];
		int* 	iYec15 = &iYec15_tmp[4];
		float* 	fRec31 = &fRec31_tmp[4];
		float* 	fRec30 = &fRec30_tmp[4];
		int* 	iYec17 = &iYec17_tmp[4];
		float* 	fRec33 = &fRec33_tmp[4];
		float* 	fRec32 = &fRec32_tmp[4];
		int* 	iYec19 = &iYec19_tmp[4];
		float* 	fRec35 = &fRec35_tmp[4];
		float* 	fRec34 = &fRec34_tmp[4];
		int* 	iYec21 = &iYec21_tmp[4];
		float* 	fRec37 = &fRec37_tmp[4];
		float* 	fRec36 = &fRec36_tmp[4];
		int* 	iYec23 = &iYec23_tmp[4];
		float* 	fRec39 = &fRec39_tmp[4];
		float* 	fRec38 = &fRec38_tmp[4];
		int* 	iYec25 = &iYec25_tmp[4];
		float* 	fRec41 = &fRec41_tmp[4];
		float* 	fRec40 = &fRec40_tmp[4];
		int* 	iYec27 = &iYec27_tmp[4];
		float* 	fRec43 = &fRec43_tmp[4];
		float* 	fRec42 = &fRec42_tmp[4];
		int* 	iYec29 = &iYec29_tmp[4];
		float* 	fRec45 = &fRec45_tmp[4];
		float* 	fRec44 = &fRec44_tmp[4];
		int* 	iYec31 = &iYec31_tmp[4];
		float* 	fRec47 = &fRec47_tmp[4];
		float* 	fRec46 = &fRec46_tmp[4];
		int* 	iYec33 = &iYec33_tmp[4];
		float* 	fRec49 = &fRec49_tmp[4];
		float* 	fRec48 = &fRec48_tmp[4];
		int* 	iYec35 = &iYec35_tmp[4];
		float* 	fRec51 = &fRec51_tmp[4];
		float* 	fRec50 = &fRec50_tmp[4];
		int* 	iYec37 = &iYec37_tmp[4];
		float* 	fRec53 = &fRec53_tmp[4];
		float* 	fRec52 = &fRec52_tmp[4];
		int* 	iYec39 = &iYec39_tmp[4];
		float* 	fRec55 = &fRec55_tmp[4];
		float* 	fRec54 = &fRec54_tmp[4];
		int* 	iYec41 = &iYec41_tmp[4];
		float* 	fRec57 = &fRec57_tmp[4];
		float* 	fRec56 = &fRec56_tmp[4];
		int* 	iYec43 = &iYec43_tmp[4];
		float* 	fRec59 = &fRec59_tmp[4];
		float* 	fRec58 = &fRec58_tmp[4];
		float* 	fRec9 = &fRec9_tmp[4];
		float* 	fRec8 = &fRec8_tmp[4];
		float* 	fRec60 = &fRec60_tmp[4];
		float* 	fRec61 = &fRec61_tmp[4];
		float* 	fRec7 = &fRec7_tmp[4];
		float 	fSlow6 = fslider4;
		float 	fSlow7 = (1.0f - fSlow6);
		float 	fSlow8 = (0.3692744729379982f * fSlow7);
		float 	fSlow9 = fslider5;
		float* 	fRec4 = &fRec4_tmp[4];
		float* 	fRec3 = &fRec3_tmp[4];
		float* 	fRec2 = &fRec2_tmp[4];
		float* 	fRec1 = &fRec1_tmp[4];
		float* 	fRec0 = &fRec0_tmp[4];
		int 	iSlow10 = int(fslider6);
		int* 	iVec3 = &iVec3_tmp[4];
		int* 	iRec72 = &iRec72_tmp[4];
		int 	iSlow11 = (iSlow10 <= 0.0f);
		int* 	iYec45 = &iYec45_tmp[4];
		float* 	fRec71 = &fRec71_tmp[4];
		float 	fSlow12 = (4.656612875245797e-10f * powf(10,(0.05f * fslider7)));
		float* 	fRec70 = &fRec70_tmp[4];
		int* 	iYec47 = &iYec47_tmp[4];
		float* 	fRec74 = &fRec74_tmp[4];
		float* 	fRec73 = &fRec73_tmp[4];
		int* 	iYec49 = &iYec49_tmp[4];
		float* 	fRec76 = &fRec76_tmp[4];
		float* 	fRec75 = &fRec75_tmp[4];
		int* 	iYec51 = &iYec51_tmp[4];
		float* 	fRec78 = &fRec78_tmp[4];
		float* 	fRec77 = &fRec77_tmp[4];
		int* 	iYec53 = &iYec53_tmp[4];
		float* 	fRec80 = &fRec80_tmp[4];
		float* 	fRec79 = &fRec79_tmp[4];
		int* 	iYec55 = &iYec55_tmp[4];
		float* 	fRec82 = &fRec82_tmp[4];
		float* 	fRec81 = &fRec81_tmp[4];
		int* 	iYec57 = &iYec57_tmp[4];
		float* 	fRec84 = &fRec84_tmp[4];
		float* 	fRec83 = &fRec83_tmp[4];
		int* 	iYec59 = &iYec59_tmp[4];
		float* 	fRec86 = &fRec86_tmp[4];
		float* 	fRec85 = &fRec85_tmp[4];
		int* 	iYec61 = &iYec61_tmp[4];
		float* 	fRec88 = &fRec88_tmp[4];
		float* 	fRec87 = &fRec87_tmp[4];
		int* 	iYec63 = &iYec63_tmp[4];
		float* 	fRec90 = &fRec90_tmp[4];
		float* 	fRec89 = &fRec89_tmp[4];
		int* 	iYec65 = &iYec65_tmp[4];
		float* 	fRec92 = &fRec92_tmp[4];
		float* 	fRec91 = &fRec91_tmp[4];
		float* 	fRec69 = &fRec69_tmp[4];
		float* 	fRec68 = &fRec68_tmp[4];
		float* 	fRec93 = &fRec93_tmp[4];
		float* 	fRec94 = &fRec94_tmp[4];
		float* 	fRec67 = &fRec67_tmp[4];
		float* 	fRec66 = &fRec66_tmp[4];
		float* 	fRec65 = &fRec65_tmp[4];
		float* 	fRec64 = &fRec64_tmp[4];
		float* 	fRec63 = &fRec63_tmp[4];
		float* 	fRec62 = &fRec62_tmp[4];
		float* 	fRec102 = &fRec102_tmp[4];
		float* 	fRec101 = &fRec101_tmp[4];
		float* 	fRec103 = &fRec103_tmp[4];
		float* 	fRec104 = &fRec104_tmp[4];
		float* 	fRec100 = &fRec100_tmp[4];
		float* 	fRec99 = &fRec99_tmp[4];
		float* 	fRec98 = &fRec98_tmp[4];
		float* 	fRec97 = &fRec97_tmp[4];
		float* 	fRec96 = &fRec96_tmp[4];
		float* 	fRec95 = &fRec95_tmp[4];
		float* 	fRec112 = &fRec112_tmp[4];
		float* 	fRec111 = &fRec111_tmp[4];
		float* 	fRec113 = &fRec113_tmp[4];
		float* 	fRec114 = &fRec114_tmp[4];
		float* 	fRec110 = &fRec110_tmp[4];
		float* 	fRec109 = &fRec109_tmp[4];
		float* 	fRec108 = &fRec108_tmp[4];
		float* 	fRec107 = &fRec107_tmp[4];
		float* 	fRec106 = &fRec106_tmp[4];
		float* 	fRec105 = &fRec105_tmp[4];
		for (int index = 0; index < fullcount; index += 32) {
			int count = min(32, fullcount-index);
			FAUSTFLOAT* output0 = &output[0][index];
			FAUSTFLOAT* output1 = &output[1][index];
			FAUSTFLOAT* output2 = &output[2][index];
			FAUSTFLOAT* output3 = &output[3][index];
			// SECTION : 1
			// LOOP 0x7fa8e516e760
			// pre processing
			for (int i=0; i<4; i++) iVec0_tmp[i]=iVec0_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iVec0[i] = 1;
			}
			// post processing
			for (int i=0; i<4; i++) iVec0_perm[i]=iVec0_tmp[count+i];
			
			// SECTION : 2
			// LOOP 0x7fa8e51732f0
			// pre processing
			for (int i=0; i<4; i++) iRec14_tmp[i]=iRec14_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iRec14[i] = ((1 + iRec14[i-1]) % iConst5);
			}
			// post processing
			for (int i=0; i<4; i++) iRec14_perm[i]=iRec14_tmp[count+i];
			
			// SECTION : 3
			// LOOP 0x7fa8e5172f60
			// pre processing
			for (int i=0; i<4; i++) iRec13_tmp[i]=iRec13_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iRec13[i] = ((1 + iRec13[i-1]) % int((float(iConst6) / float((iRec14[i] == 0)))));
			}
			// post processing
			for (int i=0; i<4; i++) iRec13_perm[i]=iRec13_tmp[count+i];
			
			// SECTION : 4
			// LOOP 0x7fa8e51755c0
			// pre processing
			for (int i=0; i<4; i++) iYec0_tmp[i]=iYec0_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec0[i] = int((iRec13[i] == 0));
			}
			// post processing
			for (int i=0; i<4; i++) iYec0_perm[i]=iYec0_tmp[count+i];
			
			// SECTION : 5
			// LOOP 0x7fa8e51754e0
			// exec code
			for (int i=0; i<count; i++) {
				iZec0[i] = int(iYec0[i-1]);
			}
			
			// LOOP 0x7fa8e51766e0
			// pre processing
			for (int i=0; i<4; i++) iVec1_tmp[i]=iVec1_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iVec1[i] = iSlow0;
			}
			// post processing
			for (int i=0; i<4; i++) iVec1_perm[i]=iVec1_tmp[count+i];
			
			// SECTION : 6
			// LOOP 0x7fa8e5175270
			// pre processing
			for (int i=0; i<4; i++) iRec15_tmp[i]=iRec15_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iRec15[i] = ((iZec0[i])?0:(iRec15[i-1] + abs((iSlow0 - iVec1[i-1]))));
			}
			// post processing
			for (int i=0; i<4; i++) iRec15_perm[i]=iRec15_tmp[count+i];
			
			// LOOP 0x7fa8e5178850
			// pre processing
			for (int i=0; i<4; i++) iRec16_tmp[i]=iRec16_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iRec16[i] = ((iYec0[i] + iRec16[i-1]) % 15);
			}
			// post processing
			for (int i=0; i<4; i++) iRec16_perm[i]=iRec16_tmp[count+i];
			
			// LOOP 0x7fa8e51804d0
			// pre processing
			for (int i=0; i<4; i++) iVec2_tmp[i]=iVec2_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iVec2[i] = iSlow3;
			}
			// post processing
			for (int i=0; i<4; i++) iVec2_perm[i]=iVec2_tmp[count+i];
			
			// LOOP 0x7fa8e5205cd0
			// pre processing
			for (int i=0; i<4; i++) iVec3_tmp[i]=iVec3_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iVec3[i] = iSlow10;
			}
			// post processing
			for (int i=0; i<4; i++) iVec3_perm[i]=iVec3_tmp[count+i];
			
			// SECTION : 7
			// LOOP 0x7fa8e5179770
			// exec code
			for (int i=0; i<count; i++) {
				ftbl0[((int((iYec0[i] & ((iRec15[i] > 0) | iSlow1))))?iRec16[i]:15)] = iSlow0;
				fZec1[i] = ftbl0[iRec16[i]];
			}
			
			// LOOP 0x7fa8e517ffa0
			// pre processing
			for (int i=0; i<4; i++) iRec19_tmp[i]=iRec19_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iRec19[i] = ((iZec0[i])?0:(iRec19[i-1] + abs((iSlow3 - iVec2[i-1]))));
			}
			// post processing
			for (int i=0; i<4; i++) iRec19_perm[i]=iRec19_tmp[count+i];
			
			// LOOP 0x7fa8e52057a0
			// pre processing
			for (int i=0; i<4; i++) iRec72_tmp[i]=iRec72_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iRec72[i] = ((iZec0[i])?0:(iRec72[i-1] + abs((iSlow10 - iVec3[i-1]))));
			}
			// post processing
			for (int i=0; i<4; i++) iRec72_perm[i]=iRec72_tmp[count+i];
			
			// SECTION : 8
			// LOOP 0x7fa8e5182600
			// exec code
			for (int i=0; i<count; i++) {
				ftbl1[((int((iYec0[i] & ((iRec19[i] > 0) | iSlow4))))?iRec16[i]:15)] = iSlow3;
				fZec2[i] = ftbl1[iRec16[i]];
			}
			
			// LOOP 0x7fa8e51c1b50
			// pre processing
			for (int i=0; i<4; i++) iYec35_tmp[i]=iYec35_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec35[i] = (fabsf((fZec1[i] - 6)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec35_perm[i]=iYec35_tmp[count+i];
			
			// LOOP 0x7fa8e5207f20
			// exec code
			for (int i=0; i<count; i++) {
				ftbl2[((int((iYec0[i] & ((iRec72[i] > 0) | iSlow11))))?iRec16[i]:15)] = iSlow10;
				fZec17[i] = ftbl2[iRec16[i]];
			}
			
			// SECTION : 9
			// LOOP 0x7fa8e5171d70
			// pre processing
			for (int i=0; i<4; i++) iRec11_tmp[i]=iRec11_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iRec11[i] = (12345 + (1103515245 * iRec11[i-1]));
			}
			// post processing
			for (int i=0; i<4; i++) iRec11_perm[i]=iRec11_tmp[count+i];
			
			// LOOP 0x7fa8e5179630
			// pre processing
			for (int i=0; i<4; i++) iYec1_tmp[i]=iYec1_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec1[i] = (fabsf((fZec1[i] - 10)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec1_perm[i]=iYec1_tmp[count+i];
			
			// LOOP 0x7fa8e51824d0
			// pre processing
			for (int i=0; i<4; i++) iYec3_tmp[i]=iYec3_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec3[i] = (fabsf((fZec2[i] - 2)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec3_perm[i]=iYec3_tmp[count+i];
			
			// LOOP 0x7fa8e5188260
			// pre processing
			for (int i=0; i<4; i++) iYec5_tmp[i]=iYec5_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec5[i] = (fabsf((fZec2[i] - 1)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec5_perm[i]=iYec5_tmp[count+i];
			
			// LOOP 0x7fa8e518bc20
			// pre processing
			for (int i=0; i<4; i++) iYec7_tmp[i]=iYec7_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec7[i] = (fabsf((fZec2[i] - 3)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec7_perm[i]=iYec7_tmp[count+i];
			
			// LOOP 0x7fa8e518fa80
			// pre processing
			for (int i=0; i<4; i++) iYec9_tmp[i]=iYec9_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec9[i] = (fabsf((fZec2[i] - 4)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec9_perm[i]=iYec9_tmp[count+i];
			
			// LOOP 0x7fa8e5193970
			// pre processing
			for (int i=0; i<4; i++) iYec11_tmp[i]=iYec11_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec11[i] = (fabsf((fZec2[i] - 5)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec11_perm[i]=iYec11_tmp[count+i];
			
			// LOOP 0x7fa8e5197840
			// pre processing
			for (int i=0; i<4; i++) iYec13_tmp[i]=iYec13_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec13[i] = (fabsf((fZec2[i] - 6)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec13_perm[i]=iYec13_tmp[count+i];
			
			// LOOP 0x7fa8e519b6f0
			// pre processing
			for (int i=0; i<4; i++) iYec15_tmp[i]=iYec15_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec15[i] = (fabsf((fZec2[i] - 7)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec15_perm[i]=iYec15_tmp[count+i];
			
			// LOOP 0x7fa8e519f5c0
			// pre processing
			for (int i=0; i<4; i++) iYec17_tmp[i]=iYec17_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec17[i] = (fabsf((fZec2[i] - 8)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec17_perm[i]=iYec17_tmp[count+i];
			
			// LOOP 0x7fa8e51a3470
			// pre processing
			for (int i=0; i<4; i++) iYec19_tmp[i]=iYec19_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec19[i] = (fabsf((fZec2[i] - 9)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec19_perm[i]=iYec19_tmp[count+i];
			
			// LOOP 0x7fa8e51a7340
			// pre processing
			for (int i=0; i<4; i++) iYec21_tmp[i]=iYec21_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec21[i] = (fabsf((fZec2[i] - 10)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec21_perm[i]=iYec21_tmp[count+i];
			
			// LOOP 0x7fa8e51ab130
			// pre processing
			for (int i=0; i<4; i++) iYec23_tmp[i]=iYec23_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec23[i] = (fabsf((fZec2[i] - 11)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec23_perm[i]=iYec23_tmp[count+i];
			
			// LOOP 0x7fa8e51aefe0
			// pre processing
			for (int i=0; i<4; i++) iYec25_tmp[i]=iYec25_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec25[i] = (fabsf((fZec1[i] - 1)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec25_perm[i]=iYec25_tmp[count+i];
			
			// LOOP 0x7fa8e51b2be0
			// pre processing
			for (int i=0; i<4; i++) iYec27_tmp[i]=iYec27_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec27[i] = (fabsf((fZec1[i] - 2)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec27_perm[i]=iYec27_tmp[count+i];
			
			// LOOP 0x7fa8e51b67c0
			// pre processing
			for (int i=0; i<4; i++) iYec29_tmp[i]=iYec29_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec29[i] = (fabsf((fZec1[i] - 3)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec29_perm[i]=iYec29_tmp[count+i];
			
			// LOOP 0x7fa8e51ba390
			// pre processing
			for (int i=0; i<4; i++) iYec31_tmp[i]=iYec31_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec31[i] = (fabsf((fZec1[i] - 4)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec31_perm[i]=iYec31_tmp[count+i];
			
			// LOOP 0x7fa8e51bdf70
			// pre processing
			for (int i=0; i<4; i++) iYec33_tmp[i]=iYec33_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec33[i] = (fabsf((fZec1[i] - 5)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec33_perm[i]=iYec33_tmp[count+i];
			
			// LOOP 0x7fa8e51c1610
			// pre processing
			for (int i=0; i<4; i++) fRec51_tmp[i]=fRec51_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec51[i] = ((fRec51[i-1] + ((iYec35[i] - iYec35[i-1]) > 0.0f)) - (0.01186510500229472f * (fRec51[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec51_perm[i]=fRec51_tmp[count+i];
			
			// LOOP 0x7fa8e51c5720
			// pre processing
			for (int i=0; i<4; i++) iYec37_tmp[i]=iYec37_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec37[i] = (fabsf((fZec1[i] - 7)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec37_perm[i]=iYec37_tmp[count+i];
			
			// LOOP 0x7fa8e51c9510
			// pre processing
			for (int i=0; i<4; i++) iYec39_tmp[i]=iYec39_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec39[i] = (fabsf((fZec1[i] - 8)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec39_perm[i]=iYec39_tmp[count+i];
			
			// LOOP 0x7fa8e51cd2e0
			// pre processing
			for (int i=0; i<4; i++) iYec41_tmp[i]=iYec41_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec41[i] = (fabsf((fZec1[i] - 9)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec41_perm[i]=iYec41_tmp[count+i];
			
			// LOOP 0x7fa8e51d10c0
			// pre processing
			for (int i=0; i<4; i++) iYec43_tmp[i]=iYec43_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec43[i] = (fabsf((fZec1[i] - 11)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec43_perm[i]=iYec43_tmp[count+i];
			
			// LOOP 0x7fa8e5207df0
			// pre processing
			for (int i=0; i<4; i++) iYec45_tmp[i]=iYec45_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec45[i] = (fabsf((fZec17[i] - 10)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec45_perm[i]=iYec45_tmp[count+i];
			
			// LOOP 0x7fa8e520db60
			// pre processing
			for (int i=0; i<4; i++) iYec47_tmp[i]=iYec47_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec47[i] = (fabsf((fZec17[i] - 1)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec47_perm[i]=iYec47_tmp[count+i];
			
			// LOOP 0x7fa8e5211760
			// pre processing
			for (int i=0; i<4; i++) iYec49_tmp[i]=iYec49_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec49[i] = (fabsf((fZec17[i] - 2)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec49_perm[i]=iYec49_tmp[count+i];
			
			// LOOP 0x7fa8e5215340
			// pre processing
			for (int i=0; i<4; i++) iYec51_tmp[i]=iYec51_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec51[i] = (fabsf((fZec17[i] - 3)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec51_perm[i]=iYec51_tmp[count+i];
			
			// LOOP 0x7fa8e5218f10
			// pre processing
			for (int i=0; i<4; i++) iYec53_tmp[i]=iYec53_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec53[i] = (fabsf((fZec17[i] - 4)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec53_perm[i]=iYec53_tmp[count+i];
			
			// LOOP 0x7fa8e521caf0
			// pre processing
			for (int i=0; i<4; i++) iYec55_tmp[i]=iYec55_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec55[i] = (fabsf((fZec17[i] - 5)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec55_perm[i]=iYec55_tmp[count+i];
			
			// LOOP 0x7fa8e52206d0
			// pre processing
			for (int i=0; i<4; i++) iYec57_tmp[i]=iYec57_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec57[i] = (fabsf((fZec17[i] - 6)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec57_perm[i]=iYec57_tmp[count+i];
			
			// LOOP 0x7fa8e52242a0
			// pre processing
			for (int i=0; i<4; i++) iYec59_tmp[i]=iYec59_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec59[i] = (fabsf((fZec17[i] - 7)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec59_perm[i]=iYec59_tmp[count+i];
			
			// LOOP 0x7fa8e5228090
			// pre processing
			for (int i=0; i<4; i++) iYec61_tmp[i]=iYec61_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec61[i] = (fabsf((fZec17[i] - 8)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec61_perm[i]=iYec61_tmp[count+i];
			
			// LOOP 0x7fa8e522be60
			// pre processing
			for (int i=0; i<4; i++) iYec63_tmp[i]=iYec63_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec63[i] = (fabsf((fZec17[i] - 9)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec63_perm[i]=iYec63_tmp[count+i];
			
			// LOOP 0x7fa8e522fc20
			// pre processing
			for (int i=0; i<4; i++) iYec65_tmp[i]=iYec65_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				iYec65[i] = (fabsf((fZec17[i] - 11)) < 0.5f);
			}
			// post processing
			for (int i=0; i<4; i++) iYec65_perm[i]=iYec65_tmp[count+i];
			
			// SECTION : 10
			// LOOP 0x7fa8e5172c50
			// pre processing
			for (int i=0; i<4; i++) fRec12_tmp[i]=fRec12_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec12[i] = ((fRec12[i-1] + ((iYec1[i] - iYec1[i-1]) > 0.0f)) - (0.019954648526077097f * (fRec12[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec12_perm[i]=fRec12_tmp[count+i];
			
			// LOOP 0x7fa8e517fb50
			// pre processing
			for (int i=0; i<4; i++) fRec18_tmp[i]=fRec18_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec18[i] = ((fRec18[i-1] + ((iYec3[i] - iYec3[i-1]) > 0.0f)) - (0.0033295325160703805f * (fRec18[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec18_perm[i]=fRec18_tmp[count+i];
			
			// LOOP 0x7fa8e5187d20
			// pre processing
			for (int i=0; i<4; i++) fRec21_tmp[i]=fRec21_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec21[i] = ((fRec21[i-1] + ((iYec5[i] - iYec5[i-1]) > 0.0f)) - (0.00296627625057368f * (fRec21[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec21_perm[i]=fRec21_tmp[count+i];
			
			// LOOP 0x7fa8e518b6e0
			// pre processing
			for (int i=0; i<4; i++) fRec23_tmp[i]=fRec23_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec23[i] = ((fRec23[i-1] + ((iYec7[i] - iYec7[i-1]) > 0.0f)) - (0.00373727388790102f * (fRec23[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec23_perm[i]=fRec23_tmp[count+i];
			
			// LOOP 0x7fa8e518f550
			// pre processing
			for (int i=0; i<4; i++) fRec25_tmp[i]=fRec25_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec25[i] = ((fRec25[i-1] + ((iYec9[i] - iYec9[i-1]) > 0.0f)) - (0.004444392698205774f * (fRec25[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec25_perm[i]=fRec25_tmp[count+i];
			
			// LOOP 0x7fa8e5193440
			// pre processing
			for (int i=0; i<4; i++) fRec27_tmp[i]=fRec27_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec27[i] = ((fRec27[i-1] + ((iYec11[i] - iYec11[i-1]) > 0.0f)) - (0.004988662131519274f * (fRec27[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec27_perm[i]=fRec27_tmp[count+i];
			
			// LOOP 0x7fa8e5197300
			// pre processing
			for (int i=0; i<4; i++) fRec29_tmp[i]=fRec29_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec29[i] = ((fRec29[i-1] + ((iYec13[i] - iYec13[i-1]) > 0.0f)) - (0.00593255250114736f * (fRec29[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec29_perm[i]=fRec29_tmp[count+i];
			
			// LOOP 0x7fa8e519b1b0
			// pre processing
			for (int i=0; i<4; i++) fRec31_tmp[i]=fRec31_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec31[i] = ((fRec31[i-1] + ((iYec15[i] - iYec15[i-1]) > 0.0f)) - (0.006659065032140761f * (fRec31[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec31_perm[i]=fRec31_tmp[count+i];
			
			// LOOP 0x7fa8e519f080
			// pre processing
			for (int i=0; i<4; i++) fRec33_tmp[i]=fRec33_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec33[i] = ((fRec33[i-1] + ((iYec17[i] - iYec17[i-1]) > 0.0f)) - (0.00747454777580204f * (fRec33[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec33_perm[i]=fRec33_tmp[count+i];
			
			// LOOP 0x7fa8e51a2f30
			// pre processing
			for (int i=0; i<4; i++) fRec35_tmp[i]=fRec35_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec35[i] = ((fRec35[i-1] + ((iYec19[i] - iYec19[i-1]) > 0.0f)) - (0.008888785396411547f * (fRec35[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec35_perm[i]=fRec35_tmp[count+i];
			
			// LOOP 0x7fa8e51a6e10
			// pre processing
			for (int i=0; i<4; i++) fRec37_tmp[i]=fRec37_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec37[i] = ((fRec37[i-1] + ((iYec21[i] - iYec21[i-1]) > 0.0f)) - (0.009977324263038548f * (fRec37[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec37_perm[i]=fRec37_tmp[count+i];
			
			// LOOP 0x7fa8e51aac00
			// pre processing
			for (int i=0; i<4; i++) fRec39_tmp[i]=fRec39_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec39[i] = ((fRec39[i-1] + ((iYec23[i] - iYec23[i-1]) > 0.0f)) - (0.01186510500229472f * (fRec39[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec39_perm[i]=fRec39_tmp[count+i];
			
			// LOOP 0x7fa8e51aeaa0
			// pre processing
			for (int i=0; i<4; i++) fRec41_tmp[i]=fRec41_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec41[i] = ((fRec41[i-1] + ((iYec25[i] - iYec25[i-1]) > 0.0f)) - (0.00593255250114736f * (fRec41[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec41_perm[i]=fRec41_tmp[count+i];
			
			// LOOP 0x7fa8e51b26a0
			// pre processing
			for (int i=0; i<4; i++) fRec43_tmp[i]=fRec43_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec43[i] = ((fRec43[i-1] + ((iYec27[i] - iYec27[i-1]) > 0.0f)) - (0.006659065032140761f * (fRec43[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec43_perm[i]=fRec43_tmp[count+i];
			
			// LOOP 0x7fa8e51b6280
			// pre processing
			for (int i=0; i<4; i++) fRec45_tmp[i]=fRec45_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec45[i] = ((fRec45[i-1] + ((iYec29[i] - iYec29[i-1]) > 0.0f)) - (0.00747454777580204f * (fRec45[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec45_perm[i]=fRec45_tmp[count+i];
			
			// LOOP 0x7fa8e51b9e70
			// pre processing
			for (int i=0; i<4; i++) fRec47_tmp[i]=fRec47_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec47[i] = ((fRec47[i-1] + ((iYec31[i] - iYec31[i-1]) > 0.0f)) - (0.008888785396411547f * (fRec47[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec47_perm[i]=fRec47_tmp[count+i];
			
			// LOOP 0x7fa8e51bda30
			// pre processing
			for (int i=0; i<4; i++) fRec49_tmp[i]=fRec49_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec49[i] = ((fRec49[i-1] + ((iYec33[i] - iYec33[i-1]) > 0.0f)) - (0.009977324263038548f * (fRec49[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec49_perm[i]=fRec49_tmp[count+i];
			
			// LOOP 0x7fa8e51c1320
			// pre processing
			fYec36_idx = (fYec36_idx+fYec36_idx_save)&127;
			for (int i=0; i<4; i++) fRec50_tmp[i]=fRec50_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec36[(fYec36_idx+i)&127] = ((0.498352519415873f * (fRec50[i-1] + fRec50[i-2])) + (fSlow2 * (iRec11[i] * (fRec51[i] > 0.0f))));
				fRec50[i] = fYec36[(fYec36_idx+i-83)&127];
			}
			// post processing
			for (int i=0; i<4; i++) fRec50_perm[i]=fRec50_tmp[count+i];
			fYec36_idx_save = count;
			
			// LOOP 0x7fa8e51c51e0
			// pre processing
			for (int i=0; i<4; i++) fRec53_tmp[i]=fRec53_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec53[i] = ((fRec53[i-1] + ((iYec37[i] - iYec37[i-1]) > 0.0f)) - (0.013318130064281522f * (fRec53[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec53_perm[i]=fRec53_tmp[count+i];
			
			// LOOP 0x7fa8e51c8fe0
			// pre processing
			for (int i=0; i<4; i++) fRec55_tmp[i]=fRec55_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec55[i] = ((fRec55[i-1] + ((iYec39[i] - iYec39[i-1]) > 0.0f)) - (0.01494909555160408f * (fRec55[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec55_perm[i]=fRec55_tmp[count+i];
			
			// LOOP 0x7fa8e51ccdb0
			// pre processing
			for (int i=0; i<4; i++) fRec57_tmp[i]=fRec57_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec57[i] = ((fRec57[i-1] + ((iYec41[i] - iYec41[i-1]) > 0.0f)) - (0.017777570792823095f * (fRec57[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec57_perm[i]=fRec57_tmp[count+i];
			
			// LOOP 0x7fa8e51d0b80
			// pre processing
			for (int i=0; i<4; i++) fRec59_tmp[i]=fRec59_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec59[i] = ((fRec59[i-1] + ((iYec43[i] - iYec43[i-1]) > 0.0f)) - (0.02373021000458944f * (fRec59[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec59_perm[i]=fRec59_tmp[count+i];
			
			// LOOP 0x7fa8e5205350
			// pre processing
			for (int i=0; i<4; i++) fRec71_tmp[i]=fRec71_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec71[i] = ((fRec71[i-1] + ((iYec45[i] - iYec45[i-1]) > 0.0f)) - (0.039909297052154194f * (fRec71[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec71_perm[i]=fRec71_tmp[count+i];
			
			// LOOP 0x7fa8e520d620
			// pre processing
			for (int i=0; i<4; i++) fRec74_tmp[i]=fRec74_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec74[i] = ((fRec74[i-1] + ((iYec47[i] - iYec47[i-1]) > 0.0f)) - (0.01186510500229472f * (fRec74[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec74_perm[i]=fRec74_tmp[count+i];
			
			// LOOP 0x7fa8e5211230
			// pre processing
			for (int i=0; i<4; i++) fRec76_tmp[i]=fRec76_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec76[i] = ((fRec76[i-1] + ((iYec49[i] - iYec49[i-1]) > 0.0f)) - (0.013318130064281522f * (fRec76[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec76_perm[i]=fRec76_tmp[count+i];
			
			// LOOP 0x7fa8e5214e10
			// pre processing
			for (int i=0; i<4; i++) fRec78_tmp[i]=fRec78_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec78[i] = ((fRec78[i-1] + ((iYec51[i] - iYec51[i-1]) > 0.0f)) - (0.01494909555160408f * (fRec78[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec78_perm[i]=fRec78_tmp[count+i];
			
			// LOOP 0x7fa8e52189e0
			// pre processing
			for (int i=0; i<4; i++) fRec80_tmp[i]=fRec80_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec80[i] = ((fRec80[i-1] + ((iYec53[i] - iYec53[i-1]) > 0.0f)) - (0.017777570792823095f * (fRec80[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec80_perm[i]=fRec80_tmp[count+i];
			
			// LOOP 0x7fa8e521c5c0
			// pre processing
			for (int i=0; i<4; i++) fRec82_tmp[i]=fRec82_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec82[i] = ((fRec82[i-1] + ((iYec55[i] - iYec55[i-1]) > 0.0f)) - (0.019954648526077097f * (fRec82[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec82_perm[i]=fRec82_tmp[count+i];
			
			// LOOP 0x7fa8e52201a0
			// pre processing
			for (int i=0; i<4; i++) fRec84_tmp[i]=fRec84_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec84[i] = ((fRec84[i-1] + ((iYec57[i] - iYec57[i-1]) > 0.0f)) - (0.02373021000458944f * (fRec84[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec84_perm[i]=fRec84_tmp[count+i];
			
			// LOOP 0x7fa8e5223d70
			// pre processing
			for (int i=0; i<4; i++) fRec86_tmp[i]=fRec86_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec86[i] = ((fRec86[i-1] + ((iYec59[i] - iYec59[i-1]) > 0.0f)) - (0.026636260128563044f * (fRec86[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec86_perm[i]=fRec86_tmp[count+i];
			
			// LOOP 0x7fa8e5227b60
			// pre processing
			for (int i=0; i<4; i++) fRec88_tmp[i]=fRec88_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec88[i] = ((fRec88[i-1] + ((iYec61[i] - iYec61[i-1]) > 0.0f)) - (0.02989819110320816f * (fRec88[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec88_perm[i]=fRec88_tmp[count+i];
			
			// LOOP 0x7fa8e522b930
			// pre processing
			for (int i=0; i<4; i++) fRec90_tmp[i]=fRec90_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec90[i] = ((fRec90[i-1] + ((iYec63[i] - iYec63[i-1]) > 0.0f)) - (0.03555514158564619f * (fRec90[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec90_perm[i]=fRec90_tmp[count+i];
			
			// LOOP 0x7fa8e522f700
			// pre processing
			for (int i=0; i<4; i++) fRec92_tmp[i]=fRec92_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec92[i] = ((fRec92[i-1] + ((iYec65[i] - iYec65[i-1]) > 0.0f)) - (0.04746042000917888f * (fRec92[i-1] > 0.0f)));
			}
			// post processing
			for (int i=0; i<4; i++) fRec92_perm[i]=fRec92_tmp[count+i];
			
			// SECTION : 11
			// LOOP 0x7fa8e51719e0
			// pre processing
			fYec2_idx = (fYec2_idx+fYec2_idx_save)&127;
			for (int i=0; i<4; i++) fRec10_tmp[i]=fRec10_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec2[(fYec2_idx+i)&127] = ((0.4990197469153629f * (fRec10[i-1] + fRec10[i-2])) + (fSlow2 * (iRec11[i] * (fRec12[i] > 0.0f))));
				fRec10[i] = fYec2[(fYec2_idx+i-49)&127];
			}
			// post processing
			for (int i=0; i<4; i++) fRec10_perm[i]=fRec10_tmp[count+i];
			fYec2_idx_save = count;
			
			// LOOP 0x7fa8e517f7c0
			// pre processing
			fYec4_idx = (fYec4_idx+fYec4_idx_save)&511;
			for (int i=0; i<4; i++) fRec17_tmp[i]=fRec17_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec4[(fYec4_idx+i)&511] = ((0.4941537998866976f * (fRec17[i-1] + fRec17[i-2])) + (fSlow5 * (iRec11[i] * (fRec18[i] > 0.0f))));
				fRec17[i] = fYec4[(fYec4_idx+i-299)&511];
			}
			// post processing
			for (int i=0; i<4; i++) fRec17_perm[i]=fRec17_tmp[count+i];
			fYec4_idx_save = count;
			
			// LOOP 0x7fa8e5187a10
			// pre processing
			fYec6_idx = (fYec6_idx+fYec6_idx_save)&511;
			for (int i=0; i<4; i++) fRec20_tmp[i]=fRec20_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec6[(fYec6_idx+i)&511] = ((0.4934425764844625f * (fRec20[i-1] + fRec20[i-2])) + (fSlow5 * (iRec11[i] * (fRec21[i] > 0.0f))));
				fRec20[i] = fYec6[(fYec6_idx+i-336)&511];
			}
			// post processing
			for (int i=0; i<4; i++) fRec20_perm[i]=fRec20_tmp[count+i];
			fYec6_idx_save = count;
			
			// LOOP 0x7fa8e518b430
			// pre processing
			fYec8_idx = (fYec8_idx+fYec8_idx_save)&511;
			for (int i=0; i<4; i++) fRec22_tmp[i]=fRec22_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec8[(fYec8_idx+i)&511] = ((0.4947882913184981f * (fRec22[i-1] + fRec22[i-2])) + (fSlow5 * (iRec11[i] * (fRec23[i] > 0.0f))));
				fRec22[i] = fYec8[(fYec8_idx+i-266)&511];
			}
			// post processing
			for (int i=0; i<4; i++) fRec22_perm[i]=fRec22_tmp[count+i];
			fYec8_idx_save = count;
			
			// LOOP 0x7fa8e518f2a0
			// pre processing
			fYec10_idx = (fYec10_idx+fYec10_idx_save)&255;
			for (int i=0; i<4; i++) fRec24_tmp[i]=fRec24_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec10[(fYec10_idx+i)&255] = ((0.49561384415280396f * (fRec24[i-1] + fRec24[i-2])) + (fSlow5 * (iRec11[i] * (fRec25[i] > 0.0f))));
				fRec24[i] = fYec10[(fYec10_idx+i-224)&255];
			}
			// post processing
			for (int i=0; i<4; i++) fRec24_perm[i]=fRec24_tmp[count+i];
			fYec10_idx_save = count;
			
			// LOOP 0x7fa8e5193170
			// pre processing
			fYec12_idx = (fYec12_idx+fYec12_idx_save)&255;
			for (int i=0; i<4; i++) fRec26_tmp[i]=fRec26_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec12[(fYec12_idx+i)&255] = ((0.49609050335141536f * (fRec26[i-1] + fRec26[i-2])) + (fSlow5 * (iRec11[i] * (fRec27[i] > 0.0f))));
				fRec26[i] = fYec12[(fYec12_idx+i-199)&255];
			}
			// post processing
			for (int i=0; i<4; i++) fRec26_perm[i]=fRec26_tmp[count+i];
			fYec12_idx_save = count;
			
			// LOOP 0x7fa8e5197050
			// pre processing
			fYec14_idx = (fYec14_idx+fYec14_idx_save)&255;
			for (int i=0; i<4; i++) fRec28_tmp[i]=fRec28_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec14[(fYec14_idx+i)&255] = ((0.4967104672162962f * (fRec28[i-1] + fRec28[i-2])) + (fSlow5 * (iRec11[i] * (fRec29[i] > 0.0f))));
				fRec28[i] = fYec14[(fYec14_idx+i-167)&255];
			}
			// post processing
			for (int i=0; i<4; i++) fRec28_perm[i]=fRec28_tmp[count+i];
			fYec14_idx_save = count;
			
			// LOOP 0x7fa8e519aed0
			// pre processing
			fYec16_idx = (fYec16_idx+fYec16_idx_save)&255;
			for (int i=0; i<4; i++) fRec30_tmp[i]=fRec30_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec16[(fYec16_idx+i)&255] = ((0.49706830510841143f * (fRec30[i-1] + fRec30[i-2])) + (fSlow5 * (iRec11[i] * (fRec31[i] > 0.0f))));
				fRec30[i] = fYec16[(fYec16_idx+i-149)&255];
			}
			// post processing
			for (int i=0; i<4; i++) fRec30_perm[i]=fRec30_tmp[count+i];
			fYec16_idx_save = count;
			
			// LOOP 0x7fa8e519edd0
			// pre processing
			fYec18_idx = (fYec18_idx+fYec18_idx_save)&255;
			for (int i=0; i<4; i++) fRec32_tmp[i]=fRec32_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec18[(fYec18_idx+i)&255] = ((0.49738731956016835f * (fRec32[i-1] + fRec32[i-2])) + (fSlow5 * (iRec11[i] * (fRec33[i] > 0.0f))));
				fRec32[i] = fYec18[(fYec18_idx+i-132)&255];
			}
			// post processing
			for (int i=0; i<4; i++) fRec32_perm[i]=fRec32_tmp[count+i];
			fYec18_idx_save = count;
			
			// LOOP 0x7fa8e51a2c50
			// pre processing
			fYec20_idx = (fYec20_idx+fYec20_idx_save)&255;
			for (int i=0; i<4; i++) fRec34_tmp[i]=fRec34_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec20[(fYec20_idx+i)&255] = ((0.4978020912736325f * (fRec34[i-1] + fRec34[i-2])) + (fSlow5 * (iRec11[i] * (fRec35[i] > 0.0f))));
				fRec34[i] = fYec20[(fYec20_idx+i-111)&255];
			}
			// post processing
			for (int i=0; i<4; i++) fRec34_perm[i]=fRec34_tmp[count+i];
			fYec20_idx_save = count;
			
			// LOOP 0x7fa8e51a6b40
			// pre processing
			fYec22_idx = (fYec22_idx+fYec22_idx_save)&255;
			for (int i=0; i<4; i++) fRec36_tmp[i]=fRec36_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec22[(fYec22_idx+i)&255] = ((0.4980414156229456f * (fRec36[i-1] + fRec36[i-2])) + (fSlow5 * (iRec11[i] * (fRec37[i] > 0.0f))));
				fRec36[i] = fYec22[(fYec22_idx+i-99)&255];
			}
			// post processing
			for (int i=0; i<4; i++) fRec36_perm[i]=fRec36_tmp[count+i];
			fYec22_idx_save = count;
			
			// LOOP 0x7fa8e51aa930
			// pre processing
			fYec24_idx = (fYec24_idx+fYec24_idx_save)&127;
			for (int i=0; i<4; i++) fRec38_tmp[i]=fRec38_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec24[(fYec24_idx+i)&127] = ((0.498352519415873f * (fRec38[i-1] + fRec38[i-2])) + (fSlow5 * (iRec11[i] * (fRec39[i] > 0.0f))));
				fRec38[i] = fYec24[(fYec24_idx+i-83)&127];
			}
			// post processing
			for (int i=0; i<4; i++) fRec38_perm[i]=fRec38_tmp[count+i];
			fYec24_idx_save = count;
			
			// LOOP 0x7fa8e51ae7d0
			// pre processing
			fYec26_idx = (fYec26_idx+fYec26_idx_save)&255;
			for (int i=0; i<4; i++) fRec40_tmp[i]=fRec40_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec26[(fYec26_idx+i)&255] = ((0.4967104672162962f * (fRec40[i-1] + fRec40[i-2])) + (fSlow2 * (iRec11[i] * (fRec41[i] > 0.0f))));
				fRec40[i] = fYec26[(fYec26_idx+i-167)&255];
			}
			// post processing
			for (int i=0; i<4; i++) fRec40_perm[i]=fRec40_tmp[count+i];
			fYec26_idx_save = count;
			
			// LOOP 0x7fa8e51b23c0
			// pre processing
			fYec28_idx = (fYec28_idx+fYec28_idx_save)&255;
			for (int i=0; i<4; i++) fRec42_tmp[i]=fRec42_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec28[(fYec28_idx+i)&255] = ((0.49706830510841143f * (fRec42[i-1] + fRec42[i-2])) + (fSlow2 * (iRec11[i] * (fRec43[i] > 0.0f))));
				fRec42[i] = fYec28[(fYec28_idx+i-149)&255];
			}
			// post processing
			for (int i=0; i<4; i++) fRec42_perm[i]=fRec42_tmp[count+i];
			fYec28_idx_save = count;
			
			// LOOP 0x7fa8e51b5f90
			// pre processing
			fYec30_idx = (fYec30_idx+fYec30_idx_save)&255;
			for (int i=0; i<4; i++) fRec44_tmp[i]=fRec44_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec30[(fYec30_idx+i)&255] = ((0.49738731956016835f * (fRec44[i-1] + fRec44[i-2])) + (fSlow2 * (iRec11[i] * (fRec45[i] > 0.0f))));
				fRec44[i] = fYec30[(fYec30_idx+i-132)&255];
			}
			// post processing
			for (int i=0; i<4; i++) fRec44_perm[i]=fRec44_tmp[count+i];
			fYec30_idx_save = count;
			
			// LOOP 0x7fa8e51b9ba0
			// pre processing
			fYec32_idx = (fYec32_idx+fYec32_idx_save)&255;
			for (int i=0; i<4; i++) fRec46_tmp[i]=fRec46_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec32[(fYec32_idx+i)&255] = ((0.4978020912736325f * (fRec46[i-1] + fRec46[i-2])) + (fSlow2 * (iRec11[i] * (fRec47[i] > 0.0f))));
				fRec46[i] = fYec32[(fYec32_idx+i-111)&255];
			}
			// post processing
			for (int i=0; i<4; i++) fRec46_perm[i]=fRec46_tmp[count+i];
			fYec32_idx_save = count;
			
			// LOOP 0x7fa8e51bd760
			// pre processing
			fYec34_idx = (fYec34_idx+fYec34_idx_save)&255;
			for (int i=0; i<4; i++) fRec48_tmp[i]=fRec48_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec34[(fYec34_idx+i)&255] = ((0.4980414156229456f * (fRec48[i-1] + fRec48[i-2])) + (fSlow2 * (iRec11[i] * (fRec49[i] > 0.0f))));
				fRec48[i] = fYec34[(fYec34_idx+i-99)&255];
			}
			// post processing
			for (int i=0; i<4; i++) fRec48_perm[i]=fRec48_tmp[count+i];
			fYec34_idx_save = count;
			
			// LOOP 0x7fa8e51c4f10
			// pre processing
			fYec38_idx = (fYec38_idx+fYec38_idx_save)&127;
			for (int i=0; i<4; i++) fRec52_tmp[i]=fRec52_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec38[(fYec38_idx+i)&127] = ((0.49853199752293303f * (fRec52[i-1] + fRec52[i-2])) + (fSlow2 * (iRec11[i] * (fRec53[i] > 0.0f))));
				fRec52[i] = fYec38[(fYec38_idx+i-74)&127];
			}
			// post processing
			for (int i=0; i<4; i++) fRec52_perm[i]=fRec52_tmp[count+i];
			fYec38_idx_save = count;
			
			// LOOP 0x7fa8e51c8d10
			// pre processing
			fYec40_idx = (fYec40_idx+fYec40_idx_save)&127;
			for (int i=0; i<4; i++) fRec54_tmp[i]=fRec54_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec40[(fYec40_idx+i)&127] = ((0.4986919487820955f * (fRec54[i-1] + fRec54[i-2])) + (fSlow2 * (iRec11[i] * (fRec55[i] > 0.0f))));
				fRec54[i] = fYec40[(fYec40_idx+i-65)&127];
			}
			// post processing
			for (int i=0; i<4; i++) fRec54_perm[i]=fRec54_tmp[count+i];
			fYec40_idx_save = count;
			
			// LOOP 0x7fa8e51ccae0
			// pre processing
			fYec42_idx = (fYec42_idx+fYec42_idx_save)&127;
			for (int i=0; i<4; i++) fRec56_tmp[i]=fRec56_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec42[(fYec42_idx+i)&127] = ((0.4988998352743928f * (fRec56[i-1] + fRec56[i-2])) + (fSlow2 * (iRec11[i] * (fRec57[i] > 0.0f))));
				fRec56[i] = fYec42[(fYec42_idx+i-55)&127];
			}
			// post processing
			for (int i=0; i<4; i++) fRec56_perm[i]=fRec56_tmp[count+i];
			fYec42_idx_save = count;
			
			// LOOP 0x7fa8e51d08d0
			// pre processing
			fYec44_idx = (fYec44_idx+fYec44_idx_save)&127;
			for (int i=0; i<4; i++) fRec58_tmp[i]=fRec58_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec44[(fYec44_idx+i)&127] = ((0.4991755800396655f * (fRec58[i-1] + fRec58[i-2])) + (fSlow2 * (iRec11[i] * (fRec59[i] > 0.0f))));
				fRec58[i] = fYec44[(fYec44_idx+i-41)&127];
			}
			// post processing
			for (int i=0; i<4; i++) fRec58_perm[i]=fRec58_tmp[count+i];
			fYec44_idx_save = count;
			
			// LOOP 0x7fa8e51da110
			// exec code
			for (int i=0; i<count; i++) {
				fZec3[i] = (1.9148542155126764f * fRec50[i]);
			}
			
			// LOOP 0x7fa8e5204fb0
			// pre processing
			fYec46_idx = (fYec46_idx+fYec46_idx_save)&63;
			for (int i=0; i<4; i++) fRec70_tmp[i]=fRec70_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec46[(fYec46_idx+i)&63] = ((0.49950963299788465f * (fRec70[i-1] + fRec70[i-2])) + (fSlow12 * (iRec11[i] * (fRec71[i] > 0.0f))));
				fRec70[i] = fYec46[(fYec46_idx+i-24)&63];
			}
			// post processing
			for (int i=0; i<4; i++) fRec70_perm[i]=fRec70_tmp[count+i];
			fYec46_idx_save = count;
			
			// LOOP 0x7fa8e520d280
			// pre processing
			fYec48_idx = (fYec48_idx+fYec48_idx_save)&127;
			for (int i=0; i<4; i++) fRec73_tmp[i]=fRec73_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec48[(fYec48_idx+i)&127] = ((0.498352519415873f * (fRec73[i-1] + fRec73[i-2])) + (fSlow12 * (iRec11[i] * (fRec74[i] > 0.0f))));
				fRec73[i] = fYec48[(fYec48_idx+i-83)&127];
			}
			// post processing
			for (int i=0; i<4; i++) fRec73_perm[i]=fRec73_tmp[count+i];
			fYec48_idx_save = count;
			
			// LOOP 0x7fa8e5210fa0
			// pre processing
			fYec50_idx = (fYec50_idx+fYec50_idx_save)&127;
			for (int i=0; i<4; i++) fRec75_tmp[i]=fRec75_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec50[(fYec50_idx+i)&127] = ((0.49853199752293303f * (fRec75[i-1] + fRec75[i-2])) + (fSlow12 * (iRec11[i] * (fRec76[i] > 0.0f))));
				fRec75[i] = fYec50[(fYec50_idx+i-74)&127];
			}
			// post processing
			for (int i=0; i<4; i++) fRec75_perm[i]=fRec75_tmp[count+i];
			fYec50_idx_save = count;
			
			// LOOP 0x7fa8e5214b40
			// pre processing
			fYec52_idx = (fYec52_idx+fYec52_idx_save)&127;
			for (int i=0; i<4; i++) fRec77_tmp[i]=fRec77_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec52[(fYec52_idx+i)&127] = ((0.4986919487820955f * (fRec77[i-1] + fRec77[i-2])) + (fSlow12 * (iRec11[i] * (fRec78[i] > 0.0f))));
				fRec77[i] = fYec52[(fYec52_idx+i-65)&127];
			}
			// post processing
			for (int i=0; i<4; i++) fRec77_perm[i]=fRec77_tmp[count+i];
			fYec52_idx_save = count;
			
			// LOOP 0x7fa8e5218710
			// pre processing
			fYec54_idx = (fYec54_idx+fYec54_idx_save)&127;
			for (int i=0; i<4; i++) fRec79_tmp[i]=fRec79_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec54[(fYec54_idx+i)&127] = ((0.4988998352743928f * (fRec79[i-1] + fRec79[i-2])) + (fSlow12 * (iRec11[i] * (fRec80[i] > 0.0f))));
				fRec79[i] = fYec54[(fYec54_idx+i-55)&127];
			}
			// post processing
			for (int i=0; i<4; i++) fRec79_perm[i]=fRec79_tmp[count+i];
			fYec54_idx_save = count;
			
			// LOOP 0x7fa8e521c2f0
			// pre processing
			fYec56_idx = (fYec56_idx+fYec56_idx_save)&127;
			for (int i=0; i<4; i++) fRec81_tmp[i]=fRec81_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec56[(fYec56_idx+i)&127] = ((0.4990197469153629f * (fRec81[i-1] + fRec81[i-2])) + (fSlow12 * (iRec11[i] * (fRec82[i] > 0.0f))));
				fRec81[i] = fYec56[(fYec56_idx+i-49)&127];
			}
			// post processing
			for (int i=0; i<4; i++) fRec81_perm[i]=fRec81_tmp[count+i];
			fYec56_idx_save = count;
			
			// LOOP 0x7fa8e521fed0
			// pre processing
			fYec58_idx = (fYec58_idx+fYec58_idx_save)&127;
			for (int i=0; i<4; i++) fRec83_tmp[i]=fRec83_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec58[(fYec58_idx+i)&127] = ((0.4991755800396655f * (fRec83[i-1] + fRec83[i-2])) + (fSlow12 * (iRec11[i] * (fRec84[i] > 0.0f))));
				fRec83[i] = fYec58[(fYec58_idx+i-41)&127];
			}
			// post processing
			for (int i=0; i<4; i++) fRec83_perm[i]=fRec83_tmp[count+i];
			fYec58_idx_save = count;
			
			// LOOP 0x7fa8e5223aa0
			// pre processing
			fYec60_idx = (fYec60_idx+fYec60_idx_save)&127;
			for (int i=0; i<4; i++) fRec85_tmp[i]=fRec85_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec60[(fYec60_idx+i)&127] = ((0.4992654592112963f * (fRec85[i-1] + fRec85[i-2])) + (fSlow12 * (iRec11[i] * (fRec86[i] > 0.0f))));
				fRec85[i] = fYec60[(fYec60_idx+i-36)&127];
			}
			// post processing
			for (int i=0; i<4; i++) fRec85_perm[i]=fRec85_tmp[count+i];
			fYec60_idx_save = count;
			
			// LOOP 0x7fa8e52278d0
			// pre processing
			fYec62_idx = (fYec62_idx+fYec62_idx_save)&63;
			for (int i=0; i<4; i++) fRec87_tmp[i]=fRec87_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec62[(fYec62_idx+i)&63] = ((0.4993455460811158f * (fRec87[i-1] + fRec87[i-2])) + (fSlow12 * (iRec11[i] * (fRec88[i] > 0.0f))));
				fRec87[i] = fYec62[(fYec62_idx+i-32)&63];
			}
			// post processing
			for (int i=0; i<4; i++) fRec87_perm[i]=fRec87_tmp[count+i];
			fYec62_idx_save = count;
			
			// LOOP 0x7fa8e522b660
			// pre processing
			fYec64_idx = (fYec64_idx+fYec64_idx_save)&63;
			for (int i=0; i<4; i++) fRec89_tmp[i]=fRec89_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec64[(fYec64_idx+i)&63] = ((0.4994496147132325f * (fRec89[i-1] + fRec89[i-2])) + (fSlow12 * (iRec11[i] * (fRec90[i] > 0.0f))));
				fRec89[i] = fYec64[(fYec64_idx+i-27)&63];
			}
			// post processing
			for (int i=0; i<4; i++) fRec89_perm[i]=fRec89_tmp[count+i];
			fYec64_idx_save = count;
			
			// LOOP 0x7fa8e522f470
			// pre processing
			fYec66_idx = (fYec66_idx+fYec66_idx_save)&63;
			for (int i=0; i<4; i++) fRec91_tmp[i]=fRec91_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fYec66[(fYec66_idx+i)&63] = ((0.4995876199625375f * (fRec91[i-1] + fRec91[i-2])) + (fSlow12 * (iRec11[i] * (fRec92[i] > 0.0f))));
				fRec91[i] = fYec66[(fYec66_idx+i-20)&63];
			}
			// post processing
			for (int i=0; i<4; i++) fRec91_perm[i]=fRec91_tmp[count+i];
			fYec66_idx_save = count;
			
			// SECTION : 12
			// LOOP 0x7fa8e51d4d50
			// exec code
			for (int i=0; i<count; i++) {
				fZec4[i] = ((fRec10[i] + (((((((((((((((((((fRec17[i] + (0.5773502691896257f * fRec20[i])) + (1.2909944487358056f * fRec22[i])) + (1.5275252316519468f * fRec24[i])) + (1.7320508075688774f * fRec26[i])) + (1.9148542155126764f * fRec28[i])) + (2.0816659994661326f * fRec30[i])) + (2.23606797749979f * fRec32[i])) + (2.3804761428476167f * fRec34[i])) + (2.516611478423583f * fRec36[i])) + (2.6457513110645907f * fRec38[i])) + (2.6457513110645907f * fRec40[i])) + (2.516611478423583f * fRec42[i])) + (2.3804761428476167f * fRec44[i])) + (2.23606797749979f * fRec46[i])) + (2.0816659994661326f * fRec48[i])) + fZec3[i]) + (1.7320508075688774f * fRec52[i])) + (1.5275252316519468f * fRec54[i])) + (1.2909944487358056f * fRec56[i]))) + (0.5773502691896255f * fRec58[i]));
			}
			
			// LOOP 0x7fa8e52332d0
			// exec code
			for (int i=0; i<count; i++) {
				fZec18[i] = ((fRec70[i] + (((((((((((((((fZec3[i] + ((((fRec42[i] + (0.5773502691896257f * fRec40[i])) + (1.2909944487358056f * fRec44[i])) + (1.5275252316519468f * fRec46[i])) + (1.7320508075688774f * fRec48[i]))) + (2.0816659994661326f * fRec52[i])) + (2.23606797749979f * fRec54[i])) + (2.3804761428476167f * fRec56[i])) + (2.516611478423583f * fRec10[i])) + (2.6457513110645907f * fRec58[i])) + (2.6457513110645907f * fRec73[i])) + (2.516611478423583f * fRec75[i])) + (2.3804761428476167f * fRec77[i])) + (2.23606797749979f * fRec79[i])) + (2.0816659994661326f * fRec81[i])) + (1.9148542155126764f * fRec83[i])) + (1.7320508075688774f * fRec85[i])) + (1.5275252316519468f * fRec87[i])) + (1.2909944487358056f * fRec89[i]))) + (0.5773502691896255f * fRec91[i]));
			}
			
			// LOOP 0x7fa8e5261880
			// exec code
			for (int i=0; i<count; i++) {
				fZec31[i] = ((0.9770084209183945f * fRec91[i]) + ((0.9293203772845852f * fRec70[i]) + ((0.8790490729915326f * fRec89[i]) + ((0.8257228238447705f * fRec87[i]) + ((0.7687061147858074f * fRec85[i]) + ((0.7071067811865476f * fRec83[i]) + (((((0.21320071635561044f * fRec73[i]) + (0.3692744729379982f * fRec75[i])) + (0.4767312946227962f * fRec77[i])) + (0.5640760748177662f * fRec79[i])) + (0.6396021490668313f * fRec81[i]))))))));
			}
			
			// LOOP 0x7fa8e527e620
			// exec code
			for (int i=0; i<count; i++) {
				fZec39[i] = ((((0.4767312946227962f * fRec34[i]) + ((0.5640760748177662f * fRec32[i]) + ((0.6396021490668313f * fRec30[i]) + ((0.7071067811865476f * fRec28[i]) + ((0.7687061147858073f * fRec26[i]) + ((((0.9770084209183945f * fRec20[i]) + (0.9293203772845852f * fRec17[i])) + (0.8790490729915326f * fRec22[i])) + (0.8257228238447705f * fRec24[i]))))))) + (0.3692744729379982f * fRec36[i])) + (0.21320071635561033f * fRec38[i]));
			}
			
			// SECTION : 13
			// LOOP 0x7fa8e51d4b70
			// exec code
			for (int i=0; i<count; i++) {
				fZec5[i] = fabsf((0.3692744729379982f * fZec4[i]));
			}
			
			// LOOP 0x7fa8e52331b0
			// exec code
			for (int i=0; i<count; i++) {
				fZec19[i] = fabsf((0.3692744729379982f * fZec18[i]));
			}
			
			// LOOP 0x7fa8e52617a0
			// exec code
			for (int i=0; i<count; i++) {
				fZec32[i] = fabsf(fZec31[i]);
			}
			
			// LOOP 0x7fa8e527e540
			// exec code
			for (int i=0; i<count; i++) {
				fZec40[i] = fabsf(fZec39[i]);
			}
			
			// SECTION : 14
			// LOOP 0x7fa8e5171730
			// pre processing
			for (int i=0; i<4; i++) fRec9_tmp[i]=fRec9_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec9[i] = ((fConst7 * max(fZec5[i], fRec9[i-1])) + (fConst8 * fZec5[i]));
			}
			// post processing
			for (int i=0; i<4; i++) fRec9_perm[i]=fRec9_tmp[count+i];
			
			// LOOP 0x7fa8e5204bf0
			// pre processing
			for (int i=0; i<4; i++) fRec69_tmp[i]=fRec69_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec69[i] = ((fConst7 * max(fZec19[i], fRec69[i-1])) + (fConst8 * fZec19[i]));
			}
			// post processing
			for (int i=0; i<4; i++) fRec69_perm[i]=fRec69_tmp[count+i];
			
			// LOOP 0x7fa8e5261420
			// pre processing
			for (int i=0; i<4; i++) fRec102_tmp[i]=fRec102_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec102[i] = ((fConst7 * max(fZec32[i], fRec102[i-1])) + (fConst8 * fZec32[i]));
			}
			// post processing
			for (int i=0; i<4; i++) fRec102_perm[i]=fRec102_tmp[count+i];
			
			// LOOP 0x7fa8e527e1c0
			// pre processing
			for (int i=0; i<4; i++) fRec112_tmp[i]=fRec112_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec112[i] = ((fConst7 * max(fZec40[i], fRec112[i-1])) + (fConst8 * fZec40[i]));
			}
			// post processing
			for (int i=0; i<4; i++) fRec112_perm[i]=fRec112_tmp[count+i];
			
			// SECTION : 15
			// LOOP 0x7fa8e51e9080
			// exec code
			for (int i=0; i<count; i++) {
				fZec6[i] = powf(2.0f,(2.3f * fRec9[i]));
			}
			
			// LOOP 0x7fa8e52472d0
			// exec code
			for (int i=0; i<count; i++) {
				fZec20[i] = powf(2.0f,(2.3f * fRec69[i]));
			}
			
			// LOOP 0x7fa8e5268860
			// exec code
			for (int i=0; i<count; i++) {
				fZec33[i] = powf(2.0f,(2.3f * fRec102[i]));
			}
			
			// LOOP 0x7fa8e5283770
			// exec code
			for (int i=0; i<count; i++) {
				fZec41[i] = powf(2.0f,(2.3f * fRec112[i]));
			}
			
			// SECTION : 16
			// LOOP 0x7fa8e516e1d0
			// pre processing
			for (int i=0; i<4; i++) fRec5_tmp[i]=fRec5_perm[i];
			for (int i=0; i<4; i++) fRec6_tmp[i]=fRec6_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec5[i] = ((fConst2 * fRec6[i-1]) + (fConst3 * fRec5[i-1]));
				fRec6[i] = ((1 + ((fConst3 * fRec6[i-1]) + (fConst4 * fRec5[i-1]))) - iVec0[i-1]);
			}
			// post processing
			for (int i=0; i<4; i++) fRec6_perm[i]=fRec6_tmp[count+i];
			for (int i=0; i<4; i++) fRec5_perm[i]=fRec5_tmp[count+i];
			
			// LOOP 0x7fa8e51e8ac0
			// exec code
			for (int i=0; i<count; i++) {
				fZec7[i] = (1 - (fConst9 * (fZec6[i] / powf(2.0f,(1.0f + (2.0f * (1.0f - fRec9[i])))))));
			}
			
			// LOOP 0x7fa8e5247100
			// exec code
			for (int i=0; i<count; i++) {
				fZec21[i] = (1 - (fConst9 * (fZec20[i] / powf(2.0f,(1.0f + (2.0f * (1.0f - fRec69[i])))))));
			}
			
			// LOOP 0x7fa8e5268690
			// exec code
			for (int i=0; i<count; i++) {
				fZec34[i] = (1 - (fConst9 * (fZec33[i] / powf(2.0f,(1.0f + (2.0f * (1.0f - fRec102[i])))))));
			}
			
			// LOOP 0x7fa8e52835a0
			// exec code
			for (int i=0; i<count; i++) {
				fZec42[i] = (1 - (fConst9 * (fZec41[i] / powf(2.0f,(1.0f + (2.0f * (1.0f - fRec112[i])))))));
			}
			
			// SECTION : 17
			// LOOP 0x7fa8e51714a0
			// pre processing
			for (int i=0; i<4; i++) fRec8_tmp[i]=fRec8_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec8[i] = ((0.999f * fRec8[i-1]) + (0.0010000000000000009f * (0 - (2.0f * (fZec7[i] * cosf((fConst10 * fZec6[i])))))));
			}
			// post processing
			for (int i=0; i<4; i++) fRec8_perm[i]=fRec8_tmp[count+i];
			
			// LOOP 0x7fa8e51eb360
			// pre processing
			for (int i=0; i<4; i++) fRec60_tmp[i]=fRec60_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec60[i] = ((0.999f * fRec60[i-1]) + (0.0010000000000000009f * faustpower<2>(fZec7[i])));
			}
			// post processing
			for (int i=0; i<4; i++) fRec60_perm[i]=fRec60_tmp[count+i];
			
			// LOOP 0x7fa8e51edd50
			// pre processing
			for (int i=0; i<4; i++) fRec61_tmp[i]=fRec61_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec61[i] = ((0.0001000000000000001f * powf(4.0f,fRec9[i])) + (0.999f * fRec61[i-1]));
			}
			// post processing
			for (int i=0; i<4; i++) fRec61_perm[i]=fRec61_tmp[count+i];
			
			// LOOP 0x7fa8e51f2af0
			// exec code
			for (int i=0; i<count; i++) {
				fZec8[i] = (628.3185307179587f + (2199.1148575128555f * (1 - fRec5[i])));
			}
			
			// LOOP 0x7fa8e5204800
			// pre processing
			for (int i=0; i<4; i++) fRec68_tmp[i]=fRec68_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec68[i] = ((0.999f * fRec68[i-1]) + (0.0010000000000000009f * (0 - (2.0f * (fZec21[i] * cosf((fConst10 * fZec20[i])))))));
			}
			// post processing
			for (int i=0; i<4; i++) fRec68_perm[i]=fRec68_tmp[count+i];
			
			// LOOP 0x7fa8e5249380
			// pre processing
			for (int i=0; i<4; i++) fRec93_tmp[i]=fRec93_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec93[i] = ((0.999f * fRec93[i-1]) + (0.0010000000000000009f * faustpower<2>(fZec21[i])));
			}
			// post processing
			for (int i=0; i<4; i++) fRec93_perm[i]=fRec93_tmp[count+i];
			
			// LOOP 0x7fa8e524cd20
			// pre processing
			for (int i=0; i<4; i++) fRec94_tmp[i]=fRec94_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec94[i] = ((0.999f * fRec94[i-1]) + (0.0001000000000000001f * powf(4.0f,fRec69[i])));
			}
			// post processing
			for (int i=0; i<4; i++) fRec94_perm[i]=fRec94_tmp[count+i];
			
			// LOOP 0x7fa8e5253bc0
			// exec code
			for (int i=0; i<count; i++) {
				fZec22[i] = (628.3185307179587f + (2199.1148575128555f * (1 - fRec6[i])));
			}
			
			// LOOP 0x7fa8e5261030
			// pre processing
			for (int i=0; i<4; i++) fRec101_tmp[i]=fRec101_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec101[i] = ((0.999f * fRec101[i-1]) + (0.0010000000000000009f * (0 - (2.0f * (fZec34[i] * cosf((fConst10 * fZec33[i])))))));
			}
			// post processing
			for (int i=0; i<4; i++) fRec101_perm[i]=fRec101_tmp[count+i];
			
			// LOOP 0x7fa8e526a940
			// pre processing
			for (int i=0; i<4; i++) fRec103_tmp[i]=fRec103_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec103[i] = ((0.999f * fRec103[i-1]) + (0.0010000000000000009f * faustpower<2>(fZec34[i])));
			}
			// post processing
			for (int i=0; i<4; i++) fRec103_perm[i]=fRec103_tmp[count+i];
			
			// LOOP 0x7fa8e526d600
			// pre processing
			for (int i=0; i<4; i++) fRec104_tmp[i]=fRec104_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec104[i] = ((0.999f * fRec104[i-1]) + (0.0001000000000000001f * powf(4.0f,fRec102[i])));
			}
			// post processing
			for (int i=0; i<4; i++) fRec104_perm[i]=fRec104_tmp[count+i];
			
			// LOOP 0x7fa8e527ddd0
			// pre processing
			for (int i=0; i<4; i++) fRec111_tmp[i]=fRec111_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec111[i] = ((0.999f * fRec111[i-1]) + (0.0010000000000000009f * (0 - (2.0f * (fZec42[i] * cosf((fConst10 * fZec41[i])))))));
			}
			// post processing
			for (int i=0; i<4; i++) fRec111_perm[i]=fRec111_tmp[count+i];
			
			// LOOP 0x7fa8e5285810
			// pre processing
			for (int i=0; i<4; i++) fRec113_tmp[i]=fRec113_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec113[i] = ((0.999f * fRec113[i-1]) + (0.0010000000000000009f * faustpower<2>(fZec42[i])));
			}
			// post processing
			for (int i=0; i<4; i++) fRec113_perm[i]=fRec113_tmp[count+i];
			
			// LOOP 0x7fa8e5288510
			// pre processing
			for (int i=0; i<4; i++) fRec114_tmp[i]=fRec114_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec114[i] = ((0.999f * fRec114[i-1]) + (0.0001000000000000001f * powf(4.0f,fRec112[i])));
			}
			// post processing
			for (int i=0; i<4; i++) fRec114_perm[i]=fRec114_tmp[count+i];
			
			// SECTION : 18
			// LOOP 0x7fa8e51711b0
			// pre processing
			for (int i=0; i<4; i++) fRec7_tmp[i]=fRec7_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec7[i] = (0 - (((fRec8[i] * fRec7[i-1]) + (fRec60[i] * fRec7[i-2])) - (0.3692744729379982f * (fRec61[i] * fZec4[i]))));
			}
			// post processing
			for (int i=0; i<4; i++) fRec7_perm[i]=fRec7_tmp[count+i];
			
			// LOOP 0x7fa8e51f2630
			// exec code
			for (int i=0; i<count; i++) {
				fZec9[i] = cosf((fConst13 * fZec8[i]));
			}
			
			// LOOP 0x7fa8e51f8830
			// exec code
			for (int i=0; i<count; i++) {
				fZec11[i] = cosf((fConst15 * fZec8[i]));
			}
			
			// LOOP 0x7fa8e51fa660
			// exec code
			for (int i=0; i<count; i++) {
				fZec13[i] = cosf((fConst16 * fZec8[i]));
			}
			
			// LOOP 0x7fa8e5200410
			// exec code
			for (int i=0; i<count; i++) {
				fZec15[i] = cosf((fConst17 * fZec8[i]));
			}
			
			// LOOP 0x7fa8e5204470
			// pre processing
			for (int i=0; i<4; i++) fRec67_tmp[i]=fRec67_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec67[i] = (0 - (((fRec68[i] * fRec67[i-1]) + (fRec93[i] * fRec67[i-2])) - (0.3692744729379982f * (fRec94[i] * fZec18[i]))));
			}
			// post processing
			for (int i=0; i<4; i++) fRec67_perm[i]=fRec67_tmp[count+i];
			
			// LOOP 0x7fa8e5253ae0
			// exec code
			for (int i=0; i<count; i++) {
				fZec23[i] = cosf((fConst13 * fZec22[i]));
			}
			
			// LOOP 0x7fa8e5259920
			// exec code
			for (int i=0; i<count; i++) {
				fZec25[i] = cosf((fConst15 * fZec22[i]));
			}
			
			// LOOP 0x7fa8e525b3d0
			// exec code
			for (int i=0; i<count; i++) {
				fZec27[i] = cosf((fConst16 * fZec22[i]));
			}
			
			// LOOP 0x7fa8e525cfb0
			// exec code
			for (int i=0; i<count; i++) {
				fZec29[i] = cosf((fConst17 * fZec22[i]));
			}
			
			// LOOP 0x7fa8e5260ca0
			// pre processing
			for (int i=0; i<4; i++) fRec100_tmp[i]=fRec100_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec100[i] = (0 - (((fRec101[i] * fRec100[i-1]) + (fRec103[i] * fRec100[i-2])) - (fRec104[i] * fZec31[i])));
			}
			// post processing
			for (int i=0; i<4; i++) fRec100_perm[i]=fRec100_tmp[count+i];
			
			// LOOP 0x7fa8e527da40
			// pre processing
			for (int i=0; i<4; i++) fRec110_tmp[i]=fRec110_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fRec110[i] = (0 - (((fRec111[i] * fRec110[i-1]) + (fRec113[i] * fRec110[i-2])) - (fRec114[i] * fZec39[i])));
			}
			// post processing
			for (int i=0; i<4; i++) fRec110_perm[i]=fRec110_tmp[count+i];
			
			// SECTION : 19
			// LOOP 0x7fa8e516d2d0
			// pre processing
			for (int i=0; i<4; i++) fRec4_tmp[i]=fRec4_perm[i];
			for (int i=0; i<4; i++) fRec3_tmp[i]=fRec3_perm[i];
			for (int i=0; i<4; i++) fRec2_tmp[i]=fRec2_perm[i];
			for (int i=0; i<4; i++) fRec1_tmp[i]=fRec1_perm[i];
			for (int i=0; i<4; i++) fRec0_tmp[i]=fRec0_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fZec10[i] = (fZec9[i] * fRec4[i-1]);
				fRec4[i] = (0 - (((fConst12 * fZec10[i]) + (fConst14 * fRec4[i-2])) - (((fSlow6 * (fRec7[i] - fRec7[i-1])) + (fSlow8 * fZec4[i])) + (fSlow9 * fRec0[i-1]))));
				fZec12[i] = (fZec11[i] * fRec3[i-1]);
				fRec3[i] = ((fRec4[i-2] + (fConst12 * (fZec10[i] - fZec12[i]))) + (fConst14 * (fRec4[i] - fRec3[i-2])));
				fZec14[i] = (fZec13[i] * fRec2[i-1]);
				fRec2[i] = ((fRec3[i-2] + (fConst12 * (fZec12[i] - fZec14[i]))) + (fConst14 * (fRec3[i] - fRec2[i-2])));
				fZec16[i] = (fZec15[i] * fRec1[i-1]);
				fRec1[i] = ((fRec2[i-2] + (fConst12 * (fZec14[i] - fZec16[i]))) + (fConst14 * (fRec2[i] - fRec1[i-2])));
				fRec0[i] = ((fConst14 * fRec1[i]) + ((fConst12 * fZec16[i]) + fRec1[i-2]));
			}
			// post processing
			for (int i=0; i<4; i++) fRec0_perm[i]=fRec0_tmp[count+i];
			for (int i=0; i<4; i++) fRec1_perm[i]=fRec1_tmp[count+i];
			for (int i=0; i<4; i++) fRec2_perm[i]=fRec2_tmp[count+i];
			for (int i=0; i<4; i++) fRec3_perm[i]=fRec3_tmp[count+i];
			for (int i=0; i<4; i++) fRec4_perm[i]=fRec4_tmp[count+i];
			
			// LOOP 0x7fa8e52033a0
			// pre processing
			for (int i=0; i<4; i++) fRec66_tmp[i]=fRec66_perm[i];
			for (int i=0; i<4; i++) fRec65_tmp[i]=fRec65_perm[i];
			for (int i=0; i<4; i++) fRec64_tmp[i]=fRec64_perm[i];
			for (int i=0; i<4; i++) fRec63_tmp[i]=fRec63_perm[i];
			for (int i=0; i<4; i++) fRec62_tmp[i]=fRec62_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fZec24[i] = (fZec23[i] * fRec66[i-1]);
				fRec66[i] = (0 - (((fConst12 * fZec24[i]) + (fConst14 * fRec66[i-2])) - (((fSlow6 * (fRec67[i] - fRec67[i-1])) + (fSlow8 * fZec18[i])) + (fSlow9 * fRec62[i-1]))));
				fZec26[i] = (fZec25[i] * fRec65[i-1]);
				fRec65[i] = ((fRec66[i-2] + (fConst12 * (fZec24[i] - fZec26[i]))) + (fConst14 * (fRec66[i] - fRec65[i-2])));
				fZec28[i] = (fZec27[i] * fRec64[i-1]);
				fRec64[i] = ((fRec65[i-2] + (fConst12 * (fZec26[i] - fZec28[i]))) + (fConst14 * (fRec65[i] - fRec64[i-2])));
				fZec30[i] = (fZec29[i] * fRec63[i-1]);
				fRec63[i] = ((fRec64[i-2] + (fConst12 * (fZec28[i] - fZec30[i]))) + (fConst14 * (fRec64[i] - fRec63[i-2])));
				fRec62[i] = ((fConst14 * fRec63[i]) + ((fConst12 * fZec30[i]) + fRec63[i-2]));
			}
			// post processing
			for (int i=0; i<4; i++) fRec62_perm[i]=fRec62_tmp[count+i];
			for (int i=0; i<4; i++) fRec63_perm[i]=fRec63_tmp[count+i];
			for (int i=0; i<4; i++) fRec64_perm[i]=fRec64_tmp[count+i];
			for (int i=0; i<4; i++) fRec65_perm[i]=fRec65_tmp[count+i];
			for (int i=0; i<4; i++) fRec66_perm[i]=fRec66_tmp[count+i];
			
			// LOOP 0x7fa8e525fc10
			// pre processing
			for (int i=0; i<4; i++) fRec99_tmp[i]=fRec99_perm[i];
			for (int i=0; i<4; i++) fRec98_tmp[i]=fRec98_perm[i];
			for (int i=0; i<4; i++) fRec97_tmp[i]=fRec97_perm[i];
			for (int i=0; i<4; i++) fRec96_tmp[i]=fRec96_perm[i];
			for (int i=0; i<4; i++) fRec95_tmp[i]=fRec95_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fZec35[i] = (fZec9[i] * fRec99[i-1]);
				fRec99[i] = (0 - (((fConst12 * fZec35[i]) + (fConst14 * fRec99[i-2])) - (((fSlow6 * (fRec100[i] - fRec100[i-1])) + (fSlow7 * fZec31[i])) + (fSlow9 * fRec95[i-1]))));
				fZec36[i] = (fZec11[i] * fRec98[i-1]);
				fRec98[i] = ((fRec99[i-2] + (fConst12 * (fZec35[i] - fZec36[i]))) + (fConst14 * (fRec99[i] - fRec98[i-2])));
				fZec37[i] = (fZec13[i] * fRec97[i-1]);
				fRec97[i] = ((fRec98[i-2] + (fConst12 * (fZec36[i] - fZec37[i]))) + (fConst14 * (fRec98[i] - fRec97[i-2])));
				fZec38[i] = (fZec15[i] * fRec96[i-1]);
				fRec96[i] = ((fRec97[i-2] + (fConst12 * (fZec37[i] - fZec38[i]))) + (fConst14 * (fRec97[i] - fRec96[i-2])));
				fRec95[i] = ((fConst14 * fRec96[i]) + ((fConst12 * fZec38[i]) + fRec96[i-2]));
			}
			// post processing
			for (int i=0; i<4; i++) fRec95_perm[i]=fRec95_tmp[count+i];
			for (int i=0; i<4; i++) fRec96_perm[i]=fRec96_tmp[count+i];
			for (int i=0; i<4; i++) fRec97_perm[i]=fRec97_tmp[count+i];
			for (int i=0; i<4; i++) fRec98_perm[i]=fRec98_tmp[count+i];
			for (int i=0; i<4; i++) fRec99_perm[i]=fRec99_tmp[count+i];
			
			// LOOP 0x7fa8e527c990
			// pre processing
			for (int i=0; i<4; i++) fRec109_tmp[i]=fRec109_perm[i];
			for (int i=0; i<4; i++) fRec108_tmp[i]=fRec108_perm[i];
			for (int i=0; i<4; i++) fRec107_tmp[i]=fRec107_perm[i];
			for (int i=0; i<4; i++) fRec106_tmp[i]=fRec106_perm[i];
			for (int i=0; i<4; i++) fRec105_tmp[i]=fRec105_perm[i];
			// exec code
			for (int i=0; i<count; i++) {
				fZec43[i] = (fZec23[i] * fRec109[i-1]);
				fRec109[i] = (0 - (((fConst12 * fZec43[i]) + (fConst14 * fRec109[i-2])) - (((fSlow6 * (fRec110[i] - fRec110[i-1])) + (fSlow7 * fZec39[i])) + (fSlow9 * fRec105[i-1]))));
				fZec44[i] = (fZec25[i] * fRec108[i-1]);
				fRec108[i] = ((fRec109[i-2] + (fConst12 * (fZec43[i] - fZec44[i]))) + (fConst14 * (fRec109[i] - fRec108[i-2])));
				fZec45[i] = (fZec27[i] * fRec107[i-1]);
				fRec107[i] = ((fRec108[i-2] + (fConst12 * (fZec44[i] - fZec45[i]))) + (fConst14 * (fRec108[i] - fRec107[i-2])));
				fZec46[i] = (fZec29[i] * fRec106[i-1]);
				fRec106[i] = ((fRec107[i-2] + (fConst12 * (fZec45[i] - fZec46[i]))) + (fConst14 * (fRec107[i] - fRec106[i-2])));
				fRec105[i] = ((fConst14 * fRec106[i]) + ((fConst12 * fZec46[i]) + fRec106[i-2]));
			}
			// post processing
			for (int i=0; i<4; i++) fRec105_perm[i]=fRec105_tmp[count+i];
			for (int i=0; i<4; i++) fRec106_perm[i]=fRec106_tmp[count+i];
			for (int i=0; i<4; i++) fRec107_perm[i]=fRec107_tmp[count+i];
			for (int i=0; i<4; i++) fRec108_perm[i]=fRec108_tmp[count+i];
			for (int i=0; i<4; i++) fRec109_perm[i]=fRec109_tmp[count+i];
			
			// SECTION : 20
			// LOOP 0x7fa8e516d0d0
			// exec code
			for (int i=0; i<count; i++) {
				output0[i] = (FAUSTFLOAT)(0 - fRec0[i]);
			}
			
			// LOOP 0x7fa8e5203240
			// exec code
			for (int i=0; i<count; i++) {
				output1[i] = (FAUSTFLOAT)(0 - fRec62[i]);
			}
			
			// LOOP 0x7fa8e525fab0
			// exec code
			for (int i=0; i<count; i++) {
				output2[i] = (FAUSTFLOAT)(0 - fRec95[i]);
			}
			
			// LOOP 0x7fa8e527c830
			// exec code
			for (int i=0; i<count; i++) {
				output3[i] = (FAUSTFLOAT)(0 - fRec105[i]);
			}
			
		}
	}
};



/***************************END USER SECTION ***************************/

/*******************BEGIN ARCHITECTURE SECTION (part 2/2)***************/

mydsp DSP;

/********************END ARCHITECTURE SECTION (part 2/2)****************/


